/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

// sample connect/add ddo/disconnect from DL
/*
 * Filename       : TAddDDOSampDL.cpp
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>
#include <DKEnvironment.hpp>

void displayDDO(DKDDO* ddo);

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TAddDDOSampDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    DKDatastoreDL dsDL; // must be outside try/catch block

    try {
    DKPid pid;
    DKAny aValue;
    DKString indexclass = "DLSAMPLE";
    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect(libsrv,userid,pw);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    pid.setObjectType(indexclass);

    DKDDO* pddo = new DKDDO(&dsDL,pid);
  
    aValue = DK_CM_DOCUMENT;
    pddo->addProperty(DK_CM_PROPERTY_ITEM_TYPE,aValue);

       short dataId = 0;
    
       DKString* attrName = new DKString[12];
       attrName[0] = "DLSEARCH_Author";
       attrName[1] = "DLSEARCH_Publisher";
       attrName[2] = "DLSEARCH_DocType";
       attrName[3] = "DLSEARCH_Date";
       attrName[4] = "DLSEARCH_Title";
       attrName[5] = "DLSEARCH_Subject";
       attrName[6] = "DLSEARCH_Authors";
       attrName[7] = "DLSEARCH_Editors";
       attrName[8] = "DLSEARCH_Source";
       attrName[9] = "DLSEARCH_Volume";
       attrName[10] = "DLSEARCH_Issue";
       attrName[11] = "DLSEARCH_Relation";

        DKAny* attrValue = new DKAny[12];
        attrValue[0] = DKString("Robert");
        attrValue[1] = DKString("IBM");
        attrValue[2] = DKString("html");
        attrValue[3] = DKString("2000");
        attrValue[4] = DKString("Time after time");
        attrValue[5] = DKString("Space and Time");
        attrValue[6] = DKString("Robert, Jonathan");
        attrValue[7] = DKString("Green publishing");
        attrValue[8] = DKString("Space Quarterly");
        attrValue[9] = DKString("2");
        attrValue[10] = DKString("1");
        attrValue[11] = DKString("");

       short* attr_prop_type = new short[12];
       attr_prop_type[0] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[1] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[2] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[3] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[4] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[5] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[6] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[7] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[8] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[9] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[10] = DK_CM_DATAITEM_TYPE_STRING;
       attr_prop_type[11] = DK_CM_DATAITEM_TYPE_STRING;

       short attrCount = 12;

       short i;
       for (i=0; i < attrCount; i++)
       {
        dataId = pddo->addData(attrName[i]);
        aValue = attr_prop_type[i];
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, aValue); // mandatory
        aValue = DK_CM_TRUE;
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, aValue); // optional
        aValue = DK_CM_ATTRIBUTE_READWRITE;
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT, aValue); // optional
        aValue = attrValue[i];
        pddo->setData(dataId, aValue);
       }

       cout << " displaying the ddo before adding it " << endl;
       // display ddo 
       displayDDO(pddo);

       cout << " adding the ddo " << endl;
       pddo->add();

       pid = *((DKPid*)pddo->getPidObject());
       cout << " the new added ddo itemid = " << pid.getPrimaryId() << endl;

       delete pddo;

       cout << " retrieve the newly added ddo, with itemid = " << pid.getPrimaryId() << endl;

       DKDDO* nddo = new DKDDO(&dsDL,pid);
        nddo->retrieve();

       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
       displayDDO(nddo);
    
        delete nddo;

    dsDL.disconnect();
    cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;

}

void displayDDO(DKDDO* ddo) {
     DKString dataName;
     DKAny any;

     /* --------------------------------------------- */
     cout << " item id[" << ((DKPid*)ddo->getPidObject())->getPrimaryId() << "]  " << endl;
     for (unsigned short j = 1; j <= ddo->dataCount(); j++) {
         dataName = ddo->getDataName(j);
         cout << " attr[" << j << "] = "
              << dataName  << ", \t\t value "
              << ddo->getData(j) << endl;
     }
     /* ------------------------------------------------*/
}
