/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

 // sample list content classes information from DL
 /*
 * Filename       : TContentCRUDDL.cpp
 * Description    : test add/update/delete of content definition information 
 *                  for DKDatastoreAdminDL
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKContentDefDL.hpp>
#include <DKDatastoreAdminDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKSequentialCollection.hpp>
#include <DKEnvironment.hpp>

void  listContentCls(DKDatastoreAdminDL* adminDL, DKString mimeType);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TContentCRUDDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    // ### content definition for add
    long id = 0;        //dl content class id
    DKString type = "CLOBDLA"; //dl content class name 
    DKString mType = "text/plain"; //mime type
    DKString descr = "This is a character LOB object."; //description
    DKString ext = "txt";  //the file extension
    // ### content definition for update
    DKString mType2 = "text/plain2a"; //mime type
    DKString descr2 = "This is a character LOB object. - 2a"; //description
    DKString ext2 = "txt-2a";  //the file extension


   try {     
      DKDatastoreDL dsDL;
      cout << "connecting to datastore" << endl;
      dsDL.connect(libsrv,userid,pw);
      cout << "datastore connected " << libsrv << " userid - " << userid << endl;

      // ### add a new content definition to DL
      DKDatastoreAdminDL* adminDL = new DKDatastoreAdminDL(&dsDL);
      DKContentDefDL cntDef;
      //cntDef.setId(id); //id is generated by system
      cntDef.setType(type);
      cntDef.setDescription(descr);
      cntDef.setMimeType(mType);
      cntDef.setFileExtension(ext);
      cout <<"about to add a new content class definition..."<<endl;
      cout <<"  cclass id = "<<cntDef.getId()<<endl;
      cout <<"  cclass type = "<<cntDef.getType()<<endl;
      cout <<"  cclass mimeType = "<<cntDef.getMimeType()<<endl;
      cout <<"  cclass descr = "<<cntDef.getDescription()<<endl;
      cout <<"  cclass fileExt = "<<cntDef.getFileExtension()<<endl; 
      adminDL->addContentDef(cntDef);
      cout <<"add successfully !"<<endl;
      id = cntDef.getId();
      cout <<"added content class id = "<<id<<endl;
      listContentCls(adminDL, mType);

     // ### to slow a bit for CM DB
     #ifdef WIN32
       Sleep(150);     
     #elif AIX
       sleep(150);
     #endif    

      // ### update the newly added content class information
      cntDef.setId(id);
      cntDef.setType(type);
      cntDef.setDescription(descr2);
      cntDef.setMimeType(mType2);
      cntDef.setFileExtension(ext2);
      cout <<"about to update the newly added content class definition..."<<endl;
      cout <<"  cclass id = "<<cntDef.getId()<<endl;
      cout <<"  cclass type = "<<cntDef.getType()<<endl;
      cout <<"  cclass mimeType = "<<cntDef.getMimeType()<<endl;
      cout <<"  cclass descr = "<<cntDef.getDescription()<<endl;
      cout <<"  cclass fileExt = "<<cntDef.getFileExtension()<<endl; 
      adminDL->updateContentDef(cntDef);
      cout <<"update successfully !"<<endl;
      listContentCls(adminDL, mType2);

      // ### delete the added content class 
      cout <<"about to delete the newly added content class definition..."<<endl;
      adminDL->deleteContentDef(cntDef);
      cout <<"delete successfully !"<<endl;

      delete adminDL;
      dsDL.disconnect();
      cout << "datastore disconnected" << endl;
      return;  
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;
   
}
 //**********************************************************
 // listContentCls - list content class informations with 
 // specified mime type
 //**********************************************************
 void  listContentCls(DKDatastoreAdminDL* adminDL, DKString mimeType)
 {
      // ### list content definition with above mime type to verify is added
      DKContentDefDL *pContDef = 0;
      DKSequentialCollection *pCol = 0;
      dkIterator *pIter = 0;
      long i = 0;
      cout << "===list contentClasses with MIME type = "<<mimeType<<"===" << endl;
      pCol = (DKSequentialCollection*)((dkCollection*)adminDL->listContentDef(mimeType));
      pIter = pCol->createIterator();
      i = 0;
      while (pIter->more() == TRUE)
      {
       i++;
       pContDef = (DKContentDefDL*)((void*)(*pIter->next()));
       cout << " Id[" << i << "] - " << pContDef->getId() << endl;
       cout << " type[" << i << "] - " << pContDef->getType() << endl;
       cout << " description[" << i << "] - " << pContDef->getDescription() << endl;
       cout << " mimeType[" << i << "] - " << pContDef->getMimeType() << endl;
       cout << " fileExtension[" << i << "] - " << pContDef->getFileExtension() << endl;
      }
      delete pIter;
      delete pCol;
      cout <<"===Total: "<<i<< " content classes with mimeType="<<mimeType<<"====" << endl;
      cout << i << " content classes listed ===========" << endl;

      return;
 }
 
