/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

// sample connect/execute/disconnect to Text Search
/*
 * Filename       : TExecuteStructDocTS.cpp
 * Description    : Connect to Text Search Datastore.
 *                  Create query string.
 *                  Get results in a resultset cursor.
 *                  Loop thru results.
 *                  Display items found.
 *                  Disconnect from Text Search Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;
  
    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 3)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TExecuteStructDocTS <srchSrv> <srchIndex>" << endl;
     return;
    }
    // ### get input parameters
    DKString srchSrv = DKString(argv[1]);
    DKString strIndex = DKString(argv[2]);
    DKDatastoreTS dsTS; // must be outside try/catch block

    try {
    dkResultSetCursor* pCur = 0;
    cout << "Datastore TS created" << endl;
    cout << "connecting to datastore" << endl;
    dsTS.connect(srchSrv,"","");
    cout << "datastore connected " << srchSrv << endl;
    DKAny a;

    DKString cmd = "SEARCH=";
    cmd += "(COND=($CCSID=850,";
    cmd += "DOCMOD=(DOCMODNAME=SAMPCORPMOD,";
    cmd += "SECLIST=(SAMPCORPTITLE,SAMPCORPBODY))$ Corp));";
    cmd += "OPTION=(SEARCH_INDEX=";
    cmd += strIndex;
    cmd += ";MAX_RESULTS=5)";
    
    cout << "query string " << cmd << endl;
    cout << "executing query" << endl;
    pCur = dsTS.execute(cmd); 
    cout << "query executed" << endl;
    
    DKDDO *p = 0;
    ushort j = 0;
    ushort cnt = 1;
    DKString strData;
    short sData = 0;
    long lData = 0;
    DKString strDataName;
    long inc = 0;
	DKPid *pid;
	DKString strPid;
    while (pCur->isValid()) 
    {
     p = pCur->fetchNext();
     if (p != 0)
     {
      cout << "record " << cnt << endl;
	  pid = (DKPid*)p->getPidObject();
	  strPid = pid->pidString();
	  cout << "pid string " << strPid << endl;
	  cout << "pid id string " << pid->getId() << endl;
	  strPid = pid->getIdString();
	  cout << "pid idString " << strPid << endl;
      unsigned long pidIdCnt = pid->getIdStringCount();
	  cout << "pid idString cnt " << pidIdCnt << endl;
	  strPid = pid->getPrimaryId();
	  cout << "pid primary id " << strPid << endl;
	  cout << "data count " << p->dataCount() << endl;
      for (j = 1; j <= p->dataCount(); j++)
      {
       a = p->getData(j);
       strDataName = p->getDataName(j);
       switch (a.typeCode())
       {
        case DKAny::tc_null :
        {
         cout << "attribute name : " << strDataName << " value : null " << endl;
         break;
        }
        default :
        {
         cout << "attribute name : " << strDataName << " value :" << a << endl;
         break;
        }
       }
      }
      cnt++;
      delete p;
     }
    }
    cnt--;
    cout << "number of records " << cnt << endl;
    if (pCur != 0)
      delete pCur;
    
    dsTS.disconnect();
    cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      } 
      cout << "Exception Class Name: " << exc.name() << endl;
    }

    cout << "done ..." << endl;
}

