/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TADLsmTM.cpp
 * Description    : test add, delete, list schema-mappings
 *                : This program will create a fed entity 'FedTMIndex' which mapped to
 *                  TMIndex on Mufasa.
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKEntityDefFed.hpp>
#include <DKAttrMappingFed.hpp>
#include <DKAttrMapDef.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>



void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 5)  {
      cout <<"Usage syntax: "<<endl;
      cout <<"\t TADLsmTM <feddb> <userid> <pw> <add/del/list>" << endl;
      return;
   }
   // ### get input parameters
   DKString libsrv(argv[1]);
   DKString userid(argv[2]);
   DKString pw    (argv[3]);
   DKString cmd   (argv[4]);
   DKDatastoreFed dsFed; // must be outside try block to avoid exception trap error.
   
   try  {
   
      // All set; Connect and execute the compound query expression
      cout << " *** connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;
   
      // get the Def
      dkDatastoreDef*  defFed = dsFed.datastoreDef();
     
      // create a federated entity : FedTMINDEX
      DKEntityDefFed* entityObj = new DKEntityDefFed(&dsFed);        
      entityObj->setName("FedTMINDEX");
      entityObj->setDescription("FedTMINDEX to map to TMINDEX at mufasa");
      entityObj->setType(DK_FED_MAPPED_TEXT_ENTITY);
     

      // associate data sources to this entity:
      // get the Fed DS Def and Admin
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();

       // Now, interpret the command: add, del, or list
      if (cmd == "add") {
         entityObj->add();        // adding an entity to the federated database
         cout << " *** federated entity created " << endl;
      }
     
      dkSchemaMapping* tsm = new DKSchemaMappingFed(&dsFed);
      tsm->setServerName("TMMUF");
      tsm->setServerType(DK_DSTYPE_TS);
      tsm->setEntityName("TMINDEX");
      tsm->setMappedEntityName("FedTMINDEX");
      tsm->setMappedEntityType(DK_FED_MAPPED_TEXT_ENTITY);

      tsm->setAssocMappedEntityName("FedSample");
      tsm->setAssocEntityName("DLSAMPLE");
      tsm->setAssocServerName("LSMUFASA");       
      
      if (cmd == "add") {
         tsm->add();
         cout << " ** schema mapping for LSMUFASA added " << endl;
      }
      else if (cmd == "del") {
         try {
            tsm->del();
            cout << " ** schema mapping for LSMUFASA deleted " << endl;
         }
         catch(DKException exc) { 
            cout << "Error id " << exc.errorId() << endl;
            for(unsigned long i = 0; i< exc.textCount();i++)
               cout << "Error text: " << exc.text(i) << endl;
      
            for (unsigned long g=0; g< exc.locationCount();g++) {
               const DKExceptionLocation* p = exc.locationAtIndex(g);
               cout << "Filename: " << p->fileName() << endl;
               cout << "Function: " << p->functionName() << endl;
               cout << "LineNumber: " << p->lineNumber() << endl;
               }
      
            cout << "Exception Class Name: " << exc.name() << endl;
            
         }
      }

      if (cmd == "del")
      {
         entityObj->del();        // deleting an entity from the federated database
         cout << " *** federated entity and attributes deleted " << endl;
      } 
      
      
      // list federated entity:

      cout << " *** federated entities :" << endl;
      DKSequentialCollection* pCol = (DKSequentialCollection*) dsFed.listEntities();
      DKSequentialCollection* pCol2;
      dkIterator* pIter = pCol->createIterator();
      dkIterator* pIter2;
      dkEntityDef* enDef;
      dkAttrDef* attrDef;
      DKString entName;
      int k = 0, j = 0, i = 0;
      while (pIter->more()) {
         k++;
         enDef = (dkEntityDef*) ((void *) (*pIter->next())); 
         entName = enDef->getName();
         cout << " Entity Def name [" << k << "] - " << entName << endl;
         cout << "  list attributes for " << entName << " entity" << endl; 
         pCol2 = (DKSequentialCollection*) dsFed.listEntityAttrs(entName);
         pIter2 = pCol2->createIterator();
         j = 0;
         while (pIter2->more()) {
            j++;
            attrDef = (dkAttrDef*)  ((void *) (*pIter2->next()));
            cout << "  Attribute name [" << j << "] - " << attrDef->getName()          << endl;
            cout << "    datastoreType : "              << attrDef->datastoreType()    << endl;
            cout << "    attributeOf   : "              << attrDef->getEntityName()    << endl;
            cout << "    type          : "              << attrDef->getType()          << endl;
            cout << "    size          : "              << attrDef->getSize()          << endl;
            cout << "    id            : "              << attrDef->getId()            << endl;
            cout << "    nullable      : "              << attrDef->isNullable()       << endl;
            cout << "    precision     : "              << attrDef->getPrecision()     << endl;
            cout << "    scale         : "              << attrDef->getScale()         << endl;
            cout << "    stringType    : "              << attrDef->getStringType()    << endl;
            delete attrDef;
         }  
         delete pIter2;
         delete pCol2; 
         delete enDef;
         cout << "  " << j << " attr listed for " << entName << " entity name"  << endl;
      }
      delete pIter;
      delete pCol;
     
      // list schema mappings in this entity:
      cout << " list schema mappings in this entity : " << entityObj->getName() << endl;
      long count;
      DKString* smnames = entityObj->listSchemaMappingNames(count);
      if (count != 0 && smnames != NULL) {
         for (i = 0; i < count; i++) {
            cout << " SM name : " << smnames[i] << endl;
         }
         delete [] smnames;
      } 
      else cout << " no sm defined in this entity " << endl;

    
      // retrieve all schema mapping objects for this entity
      dkCollection* coll = entityObj->listSchemaMappings();
      dkIterator* iter;
      if (coll != NULL && coll->cardinality() > 0) {
         iter = coll->createIterator();
         cout << " " << endl;
         cout << "-- list AttrMapping -- " << endl;
         dkAttrMapping*   _am;
         dkAttrDef*       _ad;
         dkSchemaMapping* _sm;
         DKAny *elm, *elm1;
         while (iter->more()) {
            elm = iter->next();
            _sm = (dkSchemaMapping*) elm->value();

            cout << "\n *** Schema Mapping Name : " << _sm->getName() << endl;
            dkCollection* amCol = _sm->listAttrMappings();
            if (amCol == NULL || amCol->cardinality() == 0)
               continue;
            dkIterator* amIter = amCol->createIterator();
            while (amIter->more()) {
               elm1= amIter->next();
               _am = (dkAttrMapping*) elm1->value();
               cout << " " << endl;
               cout << "  ++ AttrMapping Name : " << _am->getName()        << endl;
               cout << "    -- To Function : "    << _am->getToNative()    << endl;
               cout << "    -- From Function : "  << _am->getFromNative()  << endl;
               cout << " -- list Mapped Attr : "  << endl;
               int num = _am->numberOfMappedAttr();
               cout << "\t Mapped attr size = " << num  << endl;
               for (i = 0; i < num; i++) {
                  _ad = (dkAttrDef*)_am->getMappedAttr((short)(i+1));
                  cout << "  ++ Mapped Attr Name   : " << _ad->getName()      << endl;
                  cout << "    -- Mapped Atrr Type : " << _ad->getType()      << endl;
                  cout << "    -- Size             : " << _ad->getSize()      << endl;
                  cout << "    -- Precision        : " << _ad->getPrecision() << endl;
                  cout << "    -- Scale            : " << _ad->getScale()     << endl;
                  cout << "    -- Max              : " << _ad->getMax()       << endl;
                  cout << "    -- Min              : " << _ad->getMin()       << endl;
                  cout << "    -- Nullable         : " << _ad->isNullable()   << endl;
               }
               cout << " -- list MapTo Attr -- " << endl;
               num = _am->numberOfMapToAttr();
               cout << "\t MapTo attr size = " << num << endl;
               for (i = 0; i < num; i++) {
                  _ad = (dkAttrDef*)_am->getMapToAttr((short)(i+1));
                  cout << "  ++ MapTo Attr Name   : " << _ad->getName()      << endl;
                  cout << "    -- MapTo Atrr Type : " << _ad->getType()      << endl;
                  cout << "    -- Size            : " << _ad->getSize()      << endl;
                  cout << "    -- Precision       : " << _ad->getPrecision() << endl;
                  cout << "    -- Scale           : " << _ad->getScale()     << endl;
                  cout << "    -- Max             : " << _ad->getMax()       << endl;
                  cout << "    -- Min             : " << _ad->getMin()       << endl;
                  cout << "    -- Searchable      : " << _ad->isQueryable()  << endl;
               }  
            }
            delete _sm;
            delete amIter;
            delete amCol;
         }  
         delete iter;

      } 
      else 
         cout << " schema mapping is null " << endl;
      delete coll;

      delete tsm;
      delete entityObj;

      // disconnects
       cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
   }
   catch(DKException &exc)
      {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
         cout << "Error text: " << exc.text(i) << endl;
      
      for (unsigned long g=0; g< exc.locationCount();g++) {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
     
      cout << "Exception Class Name: " << exc.name() << endl;
   }
 
   cout << "done ..." << endl;   
}
