/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Link Type Names / Definitions
	Link relationships have names.  Links are grouped into Link Collections within
	a DDO by this name.  Any number of User-Defined Link Type Names may be used or
	the generic system defined Link Type Name may be used, "Contains".  
	
	For more information on Linking & System-Defined Link Types, please refer to 
	Sample SLinksICM.
	
	NOTE:  The name may be a maximum of 15 characters.

---------------------------
Insurance Model Link Types:
---------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the Link Type
      definition is used by and was create by the samples.  This helps avoid using
      Link Type names that may already be in use.

NOTE: Since Link Types may be used by any Item Type for any Data Model.  The following
      Link Type is being created for the Insurance Model, but it may freely be used for
      any Item of any Item Type.

    Link Type Name   Part of Insurance Model it may be seen in
    ---------------  ----------------------------------------------------------------------
    S_policeReports  Auto Claim - Links to any number of Police Reports that may exist.
                     Insurance Policy - Links to all police reports relating to the policy.
    S_autoClaims     Insurance Policy - Links to all claims on the policy.
    S_adjustments    Insurance Policy - Links to all Adjuster Reports on the policy.

---------------------------------------
Additional Link Types Defined in Sample
---------------------------------------
    S_otherPolicies  An Insurance company could link one policy to other policies it 
                     may be associated with.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKLinkTypeDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SLinkTypeDefinitionCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining Link Types, Insurance Model Link Type(s)
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Defining Links Types
 *                    Creating a New Link Type Definition
 *                    Saving / Adding Link Type Definition in Datastore Definition
 *                    Link Type Name Requirements
 *                    Defining Link Types used by samples & Insurance Model
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SLinkTypeDefinitionCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types defined here must not exist.  If any have already been created, 
 *                    please run SLinkTypeDefinitionDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SLinkTypeDefinitionCreationICM_launch.cpp
 *                    SLinkTypeDefinitionCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SLinkTypeDefinitionCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                             << endl;
            cout << "  SLinkTypeDefinitionCreationICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."              << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="     << endl;
        cout << "IBM Information Integrator for Content     v" << ver << endl;
        cout << "Sample Program:  SLinkTypeDefinitionCreationICM"     << endl;
        cout << "-----------------------------------------------"     << endl;
        cout << " Database: " << database                             << endl;
        cout << " UserName: " << userName                             << endl;
        cout << "===============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Define a Link Type
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using Link Type names that may already be in use.
            //
            // For more information on options that can be set for Link Type
            // definitions, please consult the Programming Reference.
            //
            // Defining Link Type Named "S_otherPolicies".  An Insurance 
            // company may want to link a policy to other policies it may
            // be associated with.
            cout << "Defining Link Type 'S_otherPolicies'..." << endl;

                DKLinkTypeDefICM* linkTypeDef = new DKLinkTypeDefICM(dsICM); // Create a new Link Type Definition object.
                
                linkTypeDef->setName("S_otherPolicies");                     // Choose a name with a maximum of 15 characters.
                linkTypeDef->setDescription("Links an Insurance Policy to Other Related Policies");
                
                linkTypeDef->add();                                          // Add it to the persistent datastore.

                delete linkTypeDef;                              // clean up memory

            cout << "Defined Link Type 'S_otherPolicies'." << endl;

            //--------------------------------------------------------------
            // Define Link Types for the Insurance Model
            //--------------------------------------------------------------

            // S_policeReports: Links an Auto Claim to any number of Police
            //                  Reports that may exist.
            cout << "Defining Link Type 'S_policeReports'..." << endl;

                linkTypeDef = new DKLinkTypeDefICM(dsICM);       // Create a new Link Type Definition object.
                
                linkTypeDef->setName("S_policeReports");         // Choose a name with a maximum of 15 characters.
                linkTypeDef->setDescription("Links Auto Claims and Insurance Policies to Related Police / Accident Reports");
                
                linkTypeDef->add();                              // Add it to the persistent datastore.

                delete linkTypeDef;                              // clean up memory

            cout << "Defining Link Type 'S_policeReports'..." << endl;

            // S_autoClaims: Links an Insurance Policy to any claims on it.
            cout << "Defining Link Type 'S_autoClaims'..." << endl;

                linkTypeDef = new DKLinkTypeDefICM(dsICM);       // Create a new Link Type Definition object.
                
                linkTypeDef->setName("S_autoClaims");            // Choose a name with a maximum of 15 characters.
                linkTypeDef->setDescription("Links Insurance Policy to Claims Against the Policy.");
                
                linkTypeDef->add();                              // Add it to the persistent datastore.

                delete linkTypeDef;                              // clean up memory

            cout << "Defining Link Type 'S_autoClaims'..." << endl;

            // S_adjustments: Links an Insurance Policy to any Adjustment Reports on it.
            cout << "Defining Link Type 'S_adjustments'..." << endl;

                linkTypeDef = new DKLinkTypeDefICM(dsICM);       // Create a new Link Type Definition object.
                
                linkTypeDef->setName("S_adjustments");           // Choose a name with a maximum of 15 characters.
                linkTypeDef->setDescription("Links Insurance Policy to Adjustments / Adjuster Report for the Policy.");
                
                linkTypeDef->add();                              // Add it to the persistent datastore.

                delete linkTypeDef;                              // clean up memory

            cout << "Defining Link Type 'S_adjustments'..." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Define a new Link Type.
    * For more information on options that can be set for Link Type Definitions,
    * please consult the Programming Reference.
    * @param dsICM        Connect DKDatastoreICM object to define the attribute in.
    * @param name         Name of the new attribute.
    * @param description  More detailed description than name.
    * @return Returns a reference to the newly defined DKLinkTypeDefICM that was added to the datastore.
    **/
    DKLinkTypeDefICM* SLinkTypeDefinitionCreationICM::defineLinkType(DKDatastoreICM* dsICM, char* name, char* description){
        cout << "Defining Link Type '" << name << "'..." << endl;

            DKLinkTypeDefICM* linkTypeDef = new DKLinkTypeDefICM(dsICM); // Create a new Link Type Definition object.
                
            linkTypeDef->setName(name);                                 // Choose a name with a maximum of 15 characters.
            linkTypeDef->setDescription(description);
                
            linkTypeDef->add();                                         // Add it to the persistent datastore.

        cout << "Defined Link Type '" << name << "'." << endl;

        return(linkTypeDef);
    }
