/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    For information on Item Creation, Retrieval, Updating, & Deletion, please refer to 
    the SItemCreationICM, SItemRetrievalICM, SItemUpdateICM, & SItemDeletionICM Samples.

Links
	Links are a very flexable way to link a source to a target.  A Link simply relates
	two Items and provides the means to access the Items it links to or that link to it.
	Unlike Reference Attributes, usage of Links is determined at runtime by the User 
	Program and any number of links may be used.
	
	Since Links were designed to be very flexable building blocks for User Applications to
	mold for their interests, Links are very open and non-restrictive.  The User 
	Application is responsible for any further restrictions that it chooses to place
	upon links.

    Item Types
        An Item of any Item Type can be linked to any Item, regardless of
        Item Type.  

    Root Components
        Since Links relate entire "Items", which represent an instance or data for the
        entire hierarchial tree structure of an Item Type, the root components must be
        used.  Only the Root Component of an Item may be linked to or linked from.
        
    Child Components
        As explained in the "Root Components" section, by design, links do not and can 
        not link to or from Child Components.  
        
    Version Control
        Links simply relate an entire Source Item to a Target Item, regardless of 
        version.  In order to maintain flexability of linking, links are not restricted
        to specific versions.  The User Applicaiton is responsible for choosing to
        place a Version Control mechanism on Links.  The optional "Link Item", a
        description DDO associated with the link, would be a good place to store such
        information.  
        
        Please refer to Reference Attributes as a viable alternative to Linking if
        Version Control is desired.
        
    Non-Ownership of a Link
    	A link itself does not belong to either the source or the target.  A link just
    	connects a source and target.  For example, If source 'A' is linked to target 
    	'B', 'A' will always be the source and 'B' will always be the target, regardless
    	of which DDO, 'A' or 'B', it is considered in reference to.

    In-Memory, System-Managed Consistency (Java Only)
	    Both the Source & Target DDOs will obtain copies of the same DKLink object reference.
	    Since the DKLink object contains a reference to both source & target, a DDO containing 
	    a link will contain a link that refers to itself as well as another.  For example,
	    if Source A is linked to target B, both A & B will contain the same link which would
	    look like the following:

    		DKLink Defined 	      
		    for A -> B          DDO A        DDO B
		    --------------    ---------    ---------
		    DKLink            DKLink       DKLink
    			Src: A            Src: A       Src: A
			    Tgt: B            Tgt: B       Tgt: B
        
        REQUIREMENT:
        	- When adding or removing links, only perform the operation using one of the 
        	  two items, source or target.  The changes will automatically affect the 
        	  other.  The other item in the link will not have to be updated.  
        	- If the Source & Target DDO References were retrieved independently, the one
        	  with the DKLinkCollection that was not updated must be refreshed with a 
        	  re-retrieval if updated Link information is desired.
        	  
        IMPORTANT:
            The system can only manage consistency of the actual DDOs reference in the 
            DKLink objects.  Therefore, instead of retrieving both A & B seperately, 'A'
            should be retrieved, and then 'B' should be obtained directly from the Link
            object found within 'A'.

	Inbound & Outbound:
		As explained above, a Link is not owned by either the Source or Target.  However,
		the view of the link from either end of the link will appear different from the
		the perspective at each end.  
		
		Consider two people, Person A & Person B, standing in a street, throwing a baseball
		between them.  Person A throws the ball to Person B.  Person A considers the ball
		thrown in the air as moving away, while Person B considers the same ball as moving
		towards him.  Both are correct from each persons' perspective.
		
		Now consider a Link from A to B (A -> B), where 'A' is the Source and 'B' is the
		Target.  'A' will consider this link "to" 'B' as "Outbound".  'B' will consider
		the same link "from" 'A' as "Inbound".
		
		Outbound
		    When standing at the Source of a link, all links to a target are considered
		    "Outbound".
		
		Inbound
		    When standing at the Target of a link, all links from some other source to it
		    are considered "Inbound".
		    
	WARNING: Items Contain Cyclic Reference To Self
	    As explained above, both the Source & Target obtain a copy of the same Link object.
	    The Link object consists of a reference to the Source DDO and a reference to the 
	    Target DDO.  Since the Link contains a reference to both Source & Target DDOs, one
	    of the two must be the actual refernce back to itself.  Consider again the following
	    example.  
		
    		DKLink Defined 	      
		    for A -> B          DDO A        DDO B
		    --------------    ---------    ---------
		    DKLink            DKLink       DKLink
    			Src: A            Src: A       Src: A
			    Tgt: B            Tgt: B       Tgt: B
        	
    	Clearly 'A' contains a cyclic reference to itself through the DKLink's Source 
    	reference.  Clearlly 'B' contains a cyclic reference to itself through the
	    DKLink's Target reference.

    Link Data Structure
    	Links are described in DKLink objects described here:

		    DKLink
              LinkTypeName: The type of link.
                    Source: The Source Item of a link.
                    Target: The Target Item of a link.
                  LinkItem: (Optional) Description Item.
		
	Link Type Names
		Link relationships have names.  Links are grouped into Link Collections within
		a DDO by this name.  Any number of User-Defined Link Type Names may be used or
		the generic system defined Link Type Name may be used, "Contains".  For more
		information on defining new Link Types, please refer to the Link Type Definition
		Sample(s).  
		
		System Defined Link Type Name:
		
		Link Type Name   Constant
		--------------   ----------------------------------------------
		   Contains      DK_ICM_LINKTYPENAME_CONTAINS (DKConstantICM.h)
		   
		SPECIAL LINK TYPE NAME: DKFolder
		    A special Link Type Name is provided and used by the system to manage
		    Folders.  The DKFolder object is a simplified interface to an Outbound
		    Link Collection to make Folders as easy to use as possible.  However,
		    Folders must still be Searched using Links, specifying this Link Type
		    Name, "DKFolder".
		    
		    Special Link Type   Constant
		    -----------------   ----------------------------------------------
		        DKFolder        DK_ICM_LINKTYPENAME_DKFOLDER (DKConstantICM.h)

            REQUIREMENT:
                The "DKFolder" Link Type must not be used to define or delete links, or
                be used in any way with a DKLinkCollection.  Please refer to the SFolderICM
                Sample for correct usage of Folders.  

Creating, Modifying, & Removing Links
    PLEASE USE the wrappers provided in this sample for addLinkToDDO() and removeLinkFromDDO()
    to make links as easy as possible to use.  In C++, no links may be added to a new Item
    that has not yet been added to the datastore.  DKDDO.add() must already have been called.
    
    As stated above, links are grouped into Link Collection by the Link Type Name.  Link
    Collections consist of a DKLinkCollection object, which is a colleciton of DKLink objects.
    Empty DKLinkCollections do not exist by default and must be created by the User Program.
    The wrapper provided in this sample accomplishes this easily.

    Recommended, Most Efficient Method
        The most efficient method to add or remove any number of links is to group all
        additions and modifications into a single update to the Library Server.  This is
        accomplished by using the DKLinkCollections add "Element" or remove "Element" 
        functions.  When all modifications are complete, the DKDDO's add() or update()
        method may be called.
    
    Immediate Results
        For Immediate results, use the DKLinkCollection.addMember() or DKDatastoreExtICM's
        createLink() functions, which make the changes persistent immediately, but
        at an expensive cost of a call to the Library Server for every link added.

    Incremental Changes Tracked
        All changes to the each DKLinkCollection object are tracked internally.  
        Each incremental change will be packaged up and sent to the Library Server
        in single call when the root component's DKDDO::update() function is called.
        It is important to note that the incremental changes are tracked and it is
        not the end result that is made persistent.  Optimal modifications rely on
        the fewest changes that result in the same end result.

    Modifying a DKLink Object
        Links may only be added or removed.  They may not be modified.  Any modifications
        to existing DKLink objects within the DKLinkCollections will be ignored.  In
        order to modify a DKLink, remove the link and then create an entirely new 
        DKLink object and add it to the collection as a new link.  This new link may 
        be a modified form of the old link.

    IMPORTANT: C++ Performance & Memory Management Requirements

               In C++ only, memory management must be considered when creating links.
               Using shared DDOs is the fastest, most efficient way to use links. 
               However, by default, the DDOs given to the link object are to be copied
               first, since the link will be the owner of these objects.  If copies
               are not made and not declared as shared, the user may not control the
               deletion of these objects.
               
               RECOMMENDED: Shared DDOs

                    Marking the DDOs as shared will tell that DKLink object that it 
                    does not exclusively own the object.  The DKLink object will not
                    delete the DDO or require copies.

                    The DDOs used in a link are easily declared as shared when the
                    DKLink objects are created using the constructor and/or when the
                    source, target, or link item are set.  These functions contain
                    optional boolean parameters that state whether or not these objects
                    are shared.  By default these optional parameters are set to FALSE.
                 
               DEFAULT: Copies of Objects Required.               

                    copies of the desired DKDDOs must be given to the DKLink
                    constructor.  You may use the DKDatastoreICM.createDDO(pidString)
                    function to create a new DDO with the PID information filled out,
                    which is sufficient for linking.

                    When a DKDDO object's destructor is called (when the memory is 
                    freed / deleted), it deletes all objects within, including all
                    DKLinks in the DKLinkCollections.  The DKLinks delete all DKDDO
                    objects that they own.  Since a DKLink contains DKDDOs for the
                    source, target, & link item, it will delete all three.  If any
                    of these three are the original DDO that is being deleted in
                    memory, it will crash the program because delete(ddo) will be
                    called twice on the same pointer.

Retrieving Items with Links:
    Links are not retrieved by default.  This allows greater control by the User
    Program to control all performance implications.  A retrieval option must be
    set to retrieve links.  If Links are not retrieved, the DKLinkCollections
    will not exist until the program chooses explicitly to retrieve links.  
    
    Empty Link Collections Not Created
        Please note that empty DKLinkCollections will not be created during
        retrieval unless one or more links are retrieved for that particular
        link type.  The wrapper functions provided in this sample easily account
        for this.  If the wrappers provided are used, attention to this is not
        needed.

    Retrieval Option      Constant                     Explanation
    --------------------  ---------------------------  ------------------------------------------
    Outbound Links        DK_CM_CONTENT_LINKS_OUTBOUND Retrieve Outbound Link information.
    .............................................................................................
    Inbound Links         DK_CM_CONTENT_LINKS_INBOUND  Retrieve Inbound Link information.
    .............................................................................................
    Item Tree             DK_CM_CONTENT_ITEMTREE       Retrieve the entire tree of information,
                                                       including all meta data / attributes, 
                                                       children & sub-children, links, etc.
    ---------------------------------------------------------------------------------------------
    Note: Constants shown above are defined in DKConstant.h
    
    Retrieval Option Combinations
        Retrieval Options are binary, which means that they may be added together to combine
        multiple options.
        
        Common Link Retrieval Options
        ----------------------------------------------------------------------------------
        DK_CM_CONTENT_LINKS_OUTBOUND + DK_CM_CONTENT_LINKS_INBOUND
        DK_CM_CONTENT_LINKS_OUTBOUND
        DK_CM_CONTENT_LINKS_INBOUND
        DK_CM_CONTENT_ITEMTREE
        ----------------------------------------------------------------------------------
        Note: Contants shown are defined in DKConstant.h

    DKLink Object's Non-Retrieve DDOs
        When a Link is retrieved, a new DKLink object is created.  The if it is the Source,
        the DKLink's Source is set to the exact reference to itself and a non-retrieved
        DDO is created for the Target with the Pid information filled out.  If it is the
        Target, the DKLink's Target is set to the exact reference to itself and a non-
        retrieved DDO is created for the Source with complete PID information.
        
        The other item reference in the link is not retrieved.  It is up the User
        Application to explicitly choose to retrieve it.

OTHER REQUIREMENTS:
    - All other Items associated in a Link must be persistent in the datastore 
        before the DDO owning the Link Collection is added or updated.
    - It is up to the User Program to use valid DDOs and to not add multiple
        copies of the same link.
    - Please do not remove an added link in the Link Collection that has not
        yet been made persistent. 
    - Please do not add a link that was just removed, but not yet made persistent.

Element vs. Member
    The DKSequentialCollection's "Element" methods are very different from the
    "Member" methods.  "Element" Methods are the most efficient & recommended 
    methods, while "Member" Methods offer immediate results.
    
    Element Methods
        "Element" methods are the recommended and most efficient method to use.
        Changes are not made persistent until the DKDDO.add() or DKDDO.update()
        methods are called.  All operations are grouped into one call to the 
        Library Server.
    
    Member Methods
        For immediate results, use the "Member" methods.  Changes are made 
        persistent immediately, but at an expensive cost of a call to the 
        Library Server for every add or remove using these methods.
    
addElement() vs. addMember()
    This is an example of the "Element vs. Member" section above.

removeElement() vs. removeMember()
    This is an example of the "Element vs. Member" section above.

DKDatastoreExtICM.createLink() vs. addElement()
    The Datastore Extention's createLink() function is very much like the
    "Member" operations on the DKLinkCollection.  This is a slightly different
    immediate, but expensive method.

DKDatastoreExtICM.createLink() vs. addMember()
    There is very little difference between these two methods.  

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreExtICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKSequentialCollection.hpp>
#include <DKLinkCollection.hpp>
#include <DKLink.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SLinksICM.hpp"

/************************************************************************************************
 *          FILENAME: SLinksICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating, Retrieving, Modifying, Deleting, Printing Links.  In-Memory 
 *                    System-Managed Consistency, Inbound & Outbound Concepts, Link Types,
 *                    Retrieving Link Content / Data, Element vs. Member Methods
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Links, 
 *                    Modifying Links,
 *                    Deleting Links, 
 *                    In-Memory, System-Managed Consistency
 *                    Inbound & Outbound Concepts
 *                    Link Types
 *                    Retrieving Items with Links
 *                    Link Retrieval Options
 *                    Accessing DKLinkCollections
 *                    Inbound Iterator
 *                    Outbound Iterator
 *                    Element vs. Member Methods
 *                    Printing Links
 *                    Retrieving Link Contents / Data
 *                    Get All Links in a DDO from any Link Collection
 *                    Retrieve All Link Contents in a DDO from any Link Collection
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SLinksICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SLinksICM_launch.cpp
 *                    SLinksICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SLinksICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SLinksICM <database> <userName> <password>"             << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SLinksICM"                      << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create DDOs
            //-------------------------------------------------------------
            // create 4 Items, one of Item Property type Document, one Folder, & three of Item.
            cout << "Creating & Saving Sample Items..." << endl;

	            DKDDO* ddoA = dsICM->createDDO("S_simple", DK_CM_DOCUMENT);
	            DKDDO* ddoB = dsICM->createDDO("S_simple", DK_CM_FOLDER);   
	            DKDDO* ddoC = dsICM->createDDO("S_simple", DK_CM_ITEM);     
	            DKDDO* ddoD = dsICM->createDDO("S_simple", DK_CM_ITEM);
	            DKDDO* ddoDescription = dsICM->createDDO("S_simple", DK_CM_ITEM);

	            ddoA->add();
	            ddoB->add();
	            ddoC->add();
	            ddoD->add();
	            ddoDescription->add();

            cout << "Created & Saved Sample Items." << endl;

            //-------------------------------------------------------------
            // Check out an Item for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking DDO 'A' Before Update..." << endl;

                dsICM->checkOut(ddoA);  // Must check out / lock before updating.

            cout << "Checked Out / Locked DDO 'A' Before Update." << endl;

            //-------------------------------------------------------------
            // Creating a Link Object
            //-------------------------------------------------------------
            // Creates a DKLink object from ddoA to ddoB, A -> B.
            // This sample will use the System-Defined Link Type "Contains".  
            // User-Defined Link Types may be substituted.
            //
            // Usage: DKLink(<Link Type Name>, <Source DDO>, <Target DDO>);
            cout << "Creating a Link from 'A' to -> 'B'..." << endl;

                // IMPORTANT in C++:  In C++ only, DDOs used must be either declared
                //                    as shared (recommended) or copied.
                //
                //      RECOMMENDED:  Declare as Shared & Do Not Copy DDOs
                //
                //                    Declaring the DDOs as shared is the most efficient
                //                    option.  The user-program is responsible for deleting
                //                    the objects when appropriate.
                cout << "RECOMMENDED:  Declaring DDOs as Shared / No Copy..." << endl;

                    DKLink* dkLink = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoA, ddoB, TRUE, TRUE);  // Usage: DKLink(<Link Type Name>, <Source DDO>, <Target DDO>, Shared Source (Default = FALSE), Shared Target (Default = FALSE));
 
                delete(dkLink);  // Now deleting object in order to show example using alternative.

                //          DEFAULT:  Use Copies of DDOs
                //
                //                    By default, copies of the DDOs given to the DKLink()
                //                    constructor must be given.  Copies with PID information
                //                    alone is sufficient.  Please refer to the detailed 
                //                    explanation in theheader documentation.
                cout << "DEFAULT:  Creating Copies of DDOs to Hand to DKLink()..." << endl;
                
                    DKDDO* ddoAcopy = dsICM->createDDO(((DKPidICM*)ddoA->getPidObject())->pidString());
                    DKDDO* ddoBcopy = dsICM->createDDO(((DKPidICM*)ddoB->getPidObject())->pidString());

                    dkLink = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoAcopy, ddoBcopy);  // Usage: DKLink(<Link Type Name>, <Source DDO>, <Target DDO>, Shared Source (Default = FALSE), Shared Target (Default = FALSE));
            
            cout << "Created a Link from 'A' to -> 'B'." << endl;

            //-------------------------------------------------------------
            // Add Link to one of the DDOs
            //-------------------------------------------------------------
            // When the DDO that the link is added to is updated to make 
            // the changes persistent, the other DDO will automatically
            // get a copy of this link.
            //
            // DO NOT add it to both, unless the other DDO will definately
            // not be modified.  Please refer to the header documentation
            // section labeled "In-Memory, System-Managed Consistency".
            //
            // Simply using the wrapper funciton provided in this sample 
            // is the esiest & recommended way.  Please refer to the wrapper
            // function to learn about the specifics of what would be needed
            // to do this without the wrapper function.
            //
            // In this case, the link will be added to the Source DDO.  Since
            // no ownership is implied, it does not matter which it is added to.
            cout << "Adding Link to DDO 'A'..." << endl;

                addLinkToDDO(dkLink,ddoA); // Simply use wrapper function provided.
                
                // NOTE:  'A' sees this as an Outbound Link,
                //        'B' sees this same link as an Inbound Link.

            cout << "Added Link to DDO 'A'." << endl;

            //-------------------------------------------------------------
            // Make changes persistent
            //-------------------------------------------------------------
            cout << "Making Changes Persistent..." << endl;
        
                ddoA->update();         // Don't forget to explicitly unlock when finished.
                dsICM->checkIn(ddoA);   // May specify DK_CM_CHECKIN option or use the datastore method.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Printing Source DDO (A) Links
            //-------------------------------------------------------------
            cout << "Printing Source DDO's (A's) Links..." << endl;
                printLinks(ddoA);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed Source DDO's (A's) Links." << endl;

            //-------------------------------------------------------------
            // Printing Target DDO (B) Links
            //-------------------------------------------------------------
            // With the C++ API, ddoB will not have the latest information 
            // until it is refreshed by re-retrieving the Item.
            cout << "Printing Target DDO's (B's) Links..." << endl;
                ddoB->retrieve(DK_CM_CONTENT_LINKS_INBOUND+DK_CM_CONTENT_LINKS_OUTBOUND);
                printLinks(ddoB);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed Target DDO's (B's) Links." << endl;

            //-------------------------------------------------------------
            // Check Out the Target of the Last Link for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking Target (B) of Last Link Before Update..." << endl;

                dsICM->checkOut(ddoB);  // Must check out / lock before updating.

            cout << "Checked Out / Locked Target (B) of Last Link Before Update." << endl;

            //-------------------------------------------------------------
            // Creating a Link Object with a Description Item / Link Item
            //-------------------------------------------------------------
            // Creates a DKLink object from ddoB to ddoC, B -> C and
            // a DKLink object from ddoB to ddoD, B -> D.  
            // However, Link B -> C will have a description item associated with the link.
            //
            // Usage: DKLink(<Link Type Name>, <Source DDO>, <Target DDO>, (Optional) <Description Item / Link Item>);
            cout << "Creating a Link B -> C w/Desc, & B -> D..." << endl;

                        dkLink  = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoB, ddoC, ddoDescription, TRUE, TRUE, TRUE);  // Usage: DKLink(<Link Type Name>, <Source DDO>, <Target DDO>, (Optional) <Description Item / Link Item>, Shared Source (Default = FALSE), Shared Target (Default = FALSE), Shared Link Item (Default = FALSE));
                DKLink* dkLink2 = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoB, ddoD, TRUE, TRUE); 
            
            cout << "Created a Link B -> C w/Desc, & B -> D." << endl;

            //-------------------------------------------------------------
            // Add Link to one of the DDOs
            //-------------------------------------------------------------
            // In this case, the link will be added to the Source DDO.  Since
            // no ownership is implied, it does not matter which it is added to.
            cout << "Adding Links to DDO 'B'..." << endl;

                addLinkToDDO(dkLink,ddoB);  // Simply use wrapper function provided.
                addLinkToDDO(dkLink2,ddoB); 
                
                // NOTE:  'B' sees these as Outbound Links, since it is the Source of both.
                //        'C' sees the same link as an Inbound Link.
                //        'D' sees the same link as an Inbound Link.

            cout << "Added Links to DDO 'B'." << endl;

            //-------------------------------------------------------------
            // Make changes persistent
            //-------------------------------------------------------------
            // Any number of Link additions or removals can be grouped into
            // a single update() call.
            cout << "Making Changes Persistent..." << endl;
        
                ddoB->update();         // Don't forget to explicitly unlock when finished.
                dsICM->checkIn(ddoB);   // May specify DK_CM_CHECKIN option or use the datastore method.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Printing DDO B's Links
            //-------------------------------------------------------------
            cout << "Printing DDO B's Links..." << endl;
                printLinks(ddoB);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO B's Links." << endl;

            //-------------------------------------------------------------
            // Printing DDO C's Links
            //-------------------------------------------------------------
            cout << "Printing DDO C's Links..." << endl;
                printLinks(ddoC);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO C's Links." << endl;

            //-------------------------------------------------------------
            // Printing DDO D's Links
            //-------------------------------------------------------------
            cout << "Printing DDO D's Links..." << endl;
                printLinks(ddoD);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO D's Links." << endl;

            //-------------------------------------------------------------
            // Accessing a DKLinkCollection
            //-------------------------------------------------------------
            // Links are grouped by Link Type.  Each Link Type has its own
            // DKLinksCollection.  DKLinkCollections are accessed just like
            // attributes, using the Link Type Name, but using the Link 
            // namespace.
            //
            // Link Collections may be detected and read programatically as
            // shown in SItemRetrievalICM.printDDO().  The Data Items of the
            // may be read, checking for any that are of type Link Collection.
            //
            // Another way to programatically access link collections is to check
            // for the existance of each link type available in the system for this
            // ddo as shown in the printLinks() function at the bottom of this sample.
            cout << "Accessing B's 'Contains' Link Collection..." << endl;

                short dataidB = ddoB->dataId(DK_CM_NAMESPACE_LINK,DK_ICM_LINKTYPENAME_CONTAINS); // get the dataid of DDO B's "Contains" Link Collection.

                // If the dataid is 0, the Link Collection does not exist either 
                // because no links of that Link Type Name have been added yet or
                // the DDO has not yet been retrieved with Link retrieval options.
                
                // Please refer to the addLinkToDDO() function for a demonstration
                // of what to do in this case if you want to create an empty one
                // in its place.
                
                if(dataidB <= 0) // we should have a valid dataid, since it was just added in the addLinkToDDO() function.
                    throw DKException("The 'Contains' Link Collection should exist in this case.  Link Colleciton not found.");

                DKLinkCollection* linkCollectionB = (DKLinkCollection*)(dkCollection*) ddoB->getData(dataidB); // get the link Collection in DDO B for link type "Contains".

            cout << "Accessed B's 'Contains' Link Collection." << endl;

            //-------------------------------------------------------------
            // Accessing DKLinks within a Link Collection
            //-------------------------------------------------------------
            // NOTE: If only Outbound links were retrieved, only Outbound 
            //       links will be found.  If only Inbound links are retrieved,
            //       and the full iterator is used, it will still only find the
            //       retrieved Inbound links.
            cout << "Accessing DKLinks within a Link Collection..." << endl;
            
                // Access only what this DDO considers as Inbound Links:
                dkIterator* inboundIter = linkCollectionB->createInboundIterator();
                
                // Access only what this DDO considers as Outbound Links:
                dkIterator* outboundIter = linkCollectionB->createOutboundIterator();
                
                // Access all links, Inbound & Outbound.
                dkIterator* iter = linkCollectionB->createIterator();
                
                // Access Each Retrieved Inbound Link
                cout << "Links in B's 'Contains' Link Collection:" << endl;
                cout << "  Inbound Links:" << endl;
                while(inboundIter->more()){
                    dkLink = (DKLink*) inboundIter->next()->value();  // Move pointer to next element & return that object.

                    char*  typeName =          dkLink->getTypeName(); // Link Type Name
                    DKDDO* source   = (DKDDO*) dkLink->getSource();   // Should be DDO A
                    DKDDO* target   = (DKDDO*) dkLink->getTarget();   // Should be DDO B, itself.
                    DKDDO* linkItem = (DKDDO*) dkLink->getLinkItem(); // The optional description item.  Null if not specified.

                    // Print
                    cout << "    Link:"                          << endl;
                    cout << "          Link Type:  " << typeName << endl;
                    if(source != NULL)
                        cout << "             Source:  " << ((DKPidICM*)(source->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)source->getPidObject())->getObjectType() << ")" << endl;
                    else
                        cout << "             Source:  null" << endl;
                    if(target != NULL)
                        cout << "             Target:  " << ((DKPidICM*)(target->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)target->getPidObject())->getObjectType() << ")" << endl;
                    else
                        cout << "             Source:  null" << endl;
                    if(linkItem != NULL)
                        cout << "          Link Item:  " << ((DKPidICM*)(linkItem->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)linkItem->getPidObject())->getObjectType() << ")" << endl;
                    else
                        cout << "          Link Item:  null" << endl;
                }

                // Access Each Retrieved Outbound Link
                cout << "  Outbound Links:" << endl;
                while(outboundIter->more()){
                    dkLink = (DKLink*) outboundIter->next()->value(); // Move pointer to next element & return that object.
                    printLink(dkLink);                                // Use the wrapper provided to print the link.                  
                }

                // Access All Retrieved Links, Inbound & Outbound
                cout << "  All Links:" << endl;
                while(iter->more()){
                    dkLink = (DKLink*) iter->next()->value();        // Move pointer to next element & return that object.
                    printLink(dkLink);                               // Use the wrapper provided to print the link.                  
                }

                delete(iter);                                        // Free Memory
                delete(inboundIter);
                delete(outboundIter);

            cout << "Accessed DKLinks within a Link Collection." << endl;

            //-------------------------------------------------------------
            // Check out an Item for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking DDO 'B' Before Update..." << endl;

                dsICM->checkOut(ddoB);  // Must check out / lock before updating.

            cout << "Checked Out / Locked DDO 'B' Before Update." << endl;
            
            //-------------------------------------------------------------
            // Breaking / Deleting a Link
            //-------------------------------------------------------------
            // Simply remove it from the DKLinkCollection.  When all modifications
            // are complete, make the changes persistent using DKDDO.update().
            //
            // The following will use the "Element" Method.  Please refer to the
            // header documentation for information on the differences.  For
            // Immediately Persistent, but expensive results (if used for more than
            // one in a row), The "Member" method may be used or the Datastore 
            // Extension's deleteLink() method may be used.  In those cases, no
            // DKDDO.update() is needed.
            cout << "Breaking / Deleting a Link ( B -> D )..." << endl;

                // Remove Link B -> D.
                //
                // Simply Use the Wrapper Funciton Provided.  However, it is heavy with
                // string compares, comparing the source & target Item IDs for a match.
                DKLink* linkInfoPackage = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoB, ddoD, TRUE, TRUE);
                removeLinkFromDDO(linkInfoPackage,ddoB);
                delete(linkInfoPackage);

            cout << "Broke / Deleted a Link ( B -> D )." << endl;

            //-------------------------------------------------------------
            // Make changes persistent
            //-------------------------------------------------------------
            // Any number of Link additions or removals can be grouped into
            // a single update() call.
            cout << "Making Changes Persistent..." << endl;
        
                ddoB->update();         // Don't forget to explicitly unlock when finished.
                dsICM->checkIn(ddoB);   // May specify DK_CM_CHECKIN option or use the datastore method.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Printing DDO B's Links
            //-------------------------------------------------------------
            cout << "Printing DDO B's Links..." << endl;
                printLinks(ddoB);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO B's Links." << endl;

            //-------------------------------------------------------------
            // Printing DDO D's Links
            //-------------------------------------------------------------
            cout << "Printing DDO D's Links..." << endl;
                printLinks(ddoD);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO D's Links." << endl;

            //-------------------------------------------------------------
            // Recreate DDO to Demonstrate Retrieval
            //-------------------------------------------------------------
            cout << "Recreating DDO B..." << endl;
            
                DKString ddoBPidString = ((DKPidICM*)ddoB->getPidObject())->pidString();  // Get the PID String
                delete(ddoB);  // Free Memory since object is not needed anymore.
                ddoB = dsICM->createDDO(ddoBPidString);                                   // Recreate Blank DDO Using PID String.

            cout << "Recreated DDO B." << endl;

            //-------------------------------------------------------------
            // Retrieving An Item With Links
            //-------------------------------------------------------------
            // Since retrieval options are binary, they may be combined with
            // other retrieval options explained in SItemRetrievalICM.  Below
            // are common retrieval options used with Links.
            cout << "Retrieving Item with Links (DDO B)..." << endl;
            
                ddoB->retrieve(DK_CM_CONTENT_LINKS_OUTBOUND);  // Retrieve with Outbound Links.
                
                // OR
                
                ddoB->retrieve(DK_CM_CONTENT_LINKS_INBOUND);   // Retrieve with Inbound Links.
                
                // OR

                ddoB->retrieve(DK_CM_CONTENT_LINKS_INBOUND + DK_CM_CONTENT_LINKS_OUTBOUND);  // Retrieve with Inbound & Outbound Links.

                // OR

                ddoB->retrieve(DK_CM_CONTENT_ITEMTREE);         // Item Tree includes Links.

            cout << "Retrieved Item with Links  (DDO B)." << endl;

            //-------------------------------------------------------------
            // Printing DDO B's Links
            //-------------------------------------------------------------
            cout << "Printing DDO B's Links..." << endl;
                printLinks(ddoB);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed DDO B's Links." << endl;

            //-------------------------------------------------------------
            // Retrieving Link Contents / Data            
            //-------------------------------------------------------------
            // The DKLink objects are rebuilt.  The Source, Target, & Link Item
            // are recreated as new, non-retrieved DDOs with complete PID information.
            // In the case when the DDO being retrieved is the Source or Target, its
            // own reference to itself is used.
            //
            // This sample wills imply retrieve all non-retrieve contents that are in
            // the "Contains" Link Collection.
            cout << "Retrieving 'Contains' Link Contents / Data..." << endl;

                short dataid = ddoB->dataId(DK_CM_NAMESPACE_LINK,DK_ICM_LINKTYPENAME_CONTAINS); // get the dataid of DDO B's "Contains" Link Collection.
                if(dataid > 0){ // Only need to retrieve any if a link colleciton exists
                
                    DKLinkCollection* linkCollection = (DKLinkCollection*)(dkCollection*) ddoB->getData(dataid); // get the link Collection in DDO B for link type "Contains".
                    iter = linkCollection->createIterator();  // Access all links, Inbound & Outbound.
                
                    while(iter->more()){                          // Go through all links in the list
                        dkLink = (DKLink*) iter->next()->value(); // Move pointer to next element & return that object.

                        DKDDO* source   = (DKDDO*) dkLink->getSource();  
                        DKDDO* target   = (DKDDO*) dkLink->getTarget();  
                        DKDDO* linkItem = (DKDDO*) dkLink->getLinkItem(); // The optional description item.  Null if not specified.

                        // If the DDOs are not a reference to itself, retrieve them.
                        if(source!=ddoB)
                            source->retrieve();
                        if(target!=ddoB)
                            target->retrieve();
                        if(linkItem!=NULL)
                            linkItem->retrieve();    
                    }//end while

                    delete(iter);   // Free memory for an object that is not needed anymore.
                }//end if "Contains" Link Collections Exists.

            cout << "Retrieved 'Contains' Link Contents / Data." << endl;

            //-------------------------------------------------------------
            // Immediately Persistent, but Expensive
            //-------------------------------------------------------------
            // As explained in the header documentation, "Element" methods
            // are the best and most efficient way to group multiple modifications
            // into a single update.  
            // 
            // However, if an immediately persistent result is desired or if the 
            // changes are few and infrequent, the "Member" methods and Datastore
            // Extension Linking methods may be used at the cost of a call to the
            // server for every modification.
            //
            // No following DKDDO.update() method is required.
            cout << "Using Immediately Persistent, But Expensive Operations..." << endl;

                // Create a Link Object to link B -> D.
                dkLink = new DKLink(DK_ICM_LINKTYPENAME_CONTAINS, ddoB, ddoD); // Create a Link Object for B -> D.
                
                // Create a new Datastore Extension object.
                DKDatastoreExtICM* dsExtICM = (DKDatastoreExtICM*) dsICM->getExtension(DK_CM_DATASTORE_EXT); // Get the Datastore Extension Object

                // Use the Datastore Extension Methods
                dsExtICM->addLink(dkLink);    // No further action is required
                dsExtICM->removeLink(dkLink); // No further action is required                
                
                // ALTERNATIVE:  The "Element" methods in functions addLinkToDDO()
                //               may be switched with "Member" alternatives.  Please
                //               refer to the Application Programming Reference for
                //               complete information.

            cout << "Used Immediately Persistent, But Expensive Operations." << endl;

            //-------------------------------------------------------------
            // Clean Up Database After Sampe
            //-------------------------------------------------------------
            // This sample will clean up / undo everything that stored
            // in the database.
            cout << "Cleaning Up..." << endl;

                ddoA->del();
                ddoB->del();
                ddoC->del();
                ddoD->del();

                delete(ddoA);      // DDOs will delete all contents within.
                delete(ddoC);
                delete(ddoDescription);
                delete dkLink;     //will also free ddoB and ddoD

            cout << "Cleaned Up." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print the specified Link.  The DDOs within the Link will be printed with Item ID & Object
    * Type in parentheses.
    * @param dkLink  DKLink object to print.
    **/
    void SLinksICM::printLink(DKLink* dkLink){
        char*  typeName =          dkLink->getTypeName(); // Link Type Name
        DKDDO* source   = (DKDDO*) dkLink->getSource();   // Should be DDO A
        DKDDO* target   = (DKDDO*) dkLink->getTarget();   // Should be DDO B, itself.
        DKDDO* linkItem = (DKDDO*) dkLink->getLinkItem(); // The optional description item.  Null if not specified.

        // Print The Link Information
        cout << "    Link:" << endl;
        cout << "          Link Type:  " << typeName << endl;
        if(source != NULL)
            cout << "             Source:  " << ((DKPidICM*)(source->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)source->getPidObject())->getObjectType() << ")" << endl;
        else
            cout << "             Source:  null" << endl;
        if(target != NULL)
            cout << "             Target:  " << ((DKPidICM*)(target->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)target->getPidObject())->getObjectType() << ")" << endl;
        else
            cout << "             Source:  null" << endl;
        if(linkItem != NULL)
            cout << "          Link Item:  " << ((DKPidICM*)(linkItem->getPidObject()))->getItemId() << "  (" << ((DKPidICM*)linkItem->getPidObject())->getObjectType() << ")" << endl;
        else
            cout << "          Link Item:  null" << endl;
    }//end printLink

   /**
    * Print all Links for a DDO.
    * @param ddo  DKDDO to print all Links that currently exist in the DDO.
    **/
    void SLinksICM::printLinks(DKDDO* ddo){
     
        cout << endl;
        cout << "Links in DDO '" << ((DKPidICM*)(ddo->getPidObject()))->getItemId() << "' (" << ((DKPidICM*)ddo->getPidObject())->getObjectType() << ")" << endl;

        cout << "  Inbound Links:" << endl;

        dkCollection* linksColl = getInboundLinks(ddo); // Get all Inbound Links within the DDO.
        cout << "   Total:  " << linksColl->cardinality() << endl;
        dkIterator* iter = linksColl->createIterator();
        while(iter->more())                            // go through all links in the Vector.
            printLink((DKLink*)iter->next()->value()); // print each link.

        cout << "  Outbound Links:" << endl;
        delete(iter);                                  // Free Memory
        delete linksColl;                              // Free Memory

        linksColl = getOutboundLinks(ddo);             // Get all Outbound Links within the DDO.
        cout << "   Total:  " << linksColl->cardinality() << endl;
        iter = linksColl->createIterator();
        while(iter->more())                            // go through all links in the Vector.
            printLink((DKLink*)iter->next()->value()); // print each link.
        delete(iter);                                  // Free Memory
        delete linksColl;                              // Free Memory
        
        cout << endl;
    } // end printLinks

   /**
    * Add the DKLink to the owner DDO's Link Collecitons.  The next time the DDO's add or 
    * update operations are called, it will be added to the link table.  This function will
    * take care of any other setup for data structures & settings within the DDO.
    * @param dklink	    The link to add to the owner DDO.  C++ Only:  The link should contain copies of the DDOs.
    * @param ownerDDO	The owner DDO who has the links to the other DDO.
    */
    void SLinksICM::addLinkToDDO(DKLink* linkToAdd, DKDDO* ownerDDO){
        DKLinkCollection* linkCollection;
			
        // get the link colleciton for this DDO.
        short dataid = ownerDDO->dataId(DK_CM_NAMESPACE_LINK,linkToAdd->getTypeName());
        if (dataid == 0) // if none exists, go ahead and create a link collection for it.
	        dataid = createNewLinkCollection(ownerDDO, linkToAdd->getTypeName());          //add a link collection for this.
        linkCollection = (DKLinkCollection*)(dkCollection*) ownerDDO->getData(dataid);     // get the link Collection
        if(linkCollection==NULL){ // if there is no link collection, create one (This is just a double check)
	        dataid = createNewLinkCollection(ownerDDO, linkToAdd->getTypeName());          //add a link collection for this.
	        linkCollection = (DKLinkCollection*)(dkCollection*) ownerDDO->getData(dataid); // get the link Collection
        }

        //add the link to the link collection
        linkCollection->addElement(linkToAdd);
        //add the link collection as a data property to the ddo.
        ownerDDO->setData(dataid, linkCollection);
	}//end addLinkToDDO

   /**
    * Initialize the link collection for the DDO and set the DDO properties for the link collection.
    * @param ownerDDO	The DDO that contains the link to another ddo.
    * @param linkTypeName	The name of the Link Type.
    * @return	Returns the dataid of the new collection.
    */
    short SLinksICM::createNewLinkCollection(DKDDO* ownerDDO, char* linkTypeName){
		// "Creating new link collection of link type '"+linkTypeName+"'."
		short propid = ownerDDO->propertyId(DK_ICM_PROPERTY_IS_LINKED);
		if (propid == 0) {
			//set a property of the ddo to indicate that we have links.
			ownerDDO->addProperty(DK_ICM_PROPERTY_IS_LINKED, DKBoolean(true));
		} 
		 
		short dataid = ownerDDO->dataId(DK_CM_NAMESPACE_LINK,linkTypeName);
		if (dataid == 0) // if none exists, add one.
			dataid = ownerDDO->addData(DK_CM_NAMESPACE_LINK,linkTypeName); //add a property to the ddo with that link type.
		 
		//set a data property of the ddo to indicate that it has a collection
		propid = ownerDDO->dataPropertyId(dataid,DK_CM_PROPERTY_TYPE);
		if (propid == 0){
			ownerDDO->addDataProperty(dataid, DK_CM_PROPERTY_TYPE, (short) DK_CM_DATAITEM_TYPE_LINKCOLLECTION);
		} else {
			ownerDDO->setDataProperty(dataid, propid, (short) DK_CM_DATAITEM_TYPE_LINKCOLLECTION);
		}
		
		DKLinkCollection* linkCollection = new DKLinkCollection();

		//add the link collection as a data property to the ddo.
		ownerDDO->setData(dataid, linkCollection);

		// "Created new link collection of link type '"+linkTypeName+"'.	Dataid = '"+dataid+"'."
		
		return(dataid);
	}//end createNewLinkCollection

    /**
    * Remove the specified link from the DDO's Link Collections, based on Source & Target 
    * Item ID.  It does not have to be the same exact DKLink object reference, but instead
    * just describte the link to remove.  String comparisons of the Source & Target DDOs to 
    * find it in the lists.  Please not that this function is heavy with string compares.
    * The DKDDO.update() function must still be used to make the changes persistent.
    *
    * @param dklinkToRemove  Link to remove from the DDO, based on Source & Target Item IDs, not DKLink object reference.  C++ Only:  The DKLink should contain copies of the DDO.  The removed link should be deleted manually after this call.
    * @param ddo             DDO to remove the link from.
    * @return Returns true if the Link was found and removed, false if not found.
    */
    bool SLinksICM::removeLinkFromDDO(DKLink* dklinkToRemove, DKDDO* ddo){

        bool     fReturn = false;
        DKString linkSourcePid;
        DKString linkTargetPid;
        DKString linkToRemoveSourcePid = ((DKPidICM*)dklinkToRemove->getSource()->getPidObject())->pidString();
        DKString linkToRemoveTargetPid = ((DKPidICM*)dklinkToRemove->getTarget()->getPidObject())->pidString();

	    DKLinkCollection* linkCollection; // Temporary variable to hold each individual link collection that we find in the DDO.
	    dkIterator* iter;                 // Iterator to go through the linkColleciton list.
	    int collectionSize;               // Size of the Link Collection
	    DKLink* link;                     // temporary link object to get from each link collection and add to our vector

        // Determine if DKLinkCollection for this Link Type exists, and if so, obtain it.
        short dataid = ddo->dataId(DK_CM_NAMESPACE_LINK,dklinkToRemove->getTypeName()); // Get the data id of the collection
        if(dataid <= 0){    // since the end goal is a DDO without this DKLink in it, simply return.
            return(false);  // if no Link Collection was found for this Link Type, then the link cannot be found to be removed.
        }
	    linkCollection = (DKLinkCollection*)(dkCollection*) ddo->getData(dataid);  // Get the Link Collection.

	    // Go through all links or until the desired link to be removed is found.
	    collectionSize = linkCollection->cardinality();
	    if (collectionSize > 0) {  // if there are links in the link collection, continue searching.
		    iter = linkCollection->createIterator();
            while (iter->more()) {                     // While there are still links to iterate through, continue
			    link = (DKLink*)iter->next()->value(); // Obtain each link object from the individual collection
			    linkSourcePid = ((DKPidICM*)link->getSource()->getPidObject())->pidString();
			    linkTargetPid = ((DKPidICM*)link->getTarget()->getPidObject())->pidString();
			    
			    // Compare Pid Strings to determine if this is the link described.
			    if(    (strcmp(linkSourcePid,linkToRemoveSourcePid)==0) // if it is the link we want to remove, remove it and exit
			        && (strcmp(linkTargetPid,linkToRemoveTargetPid)==0)
			    ){
				    linkCollection->removeElementAt(*iter);
                    delete(link); // Free Memory before returning
                    fReturn = true;
				    break; // Exit now since it is found.
			    }
		    }//end while more
            delete(iter);         // Free Memory
	    }//end if any are in collection
	    return(fReturn);
    }//end removeLinkFromDDO


   /**
    * Obtain All Links within the DDO, combined from all Link Collections for each Link Type.
    * @param ddo  DDO to obtain all links from.
    * @return  dkCollection (DKSequentialCollection) containing all of the DKLink objects in the entire DDO.
    */
    dkCollection* SLinksICM::getLinks(DKDDO* ddo){
        
        // Veriable Declarations
	    dkCollection* linksColl = new DKSequentialCollection();
	    DKLinkCollection* linkCollection; // Temporary variable to hold each individual link collection that we find in the DDO.
	    dkIterator* iter;                 // Iterator to go through the linkColleciton list.
	    int collectionSize;               // Size of the Link Collection
	    DKLink* link;                     // Temporary link object to get from each link collection and add to our vector
        long arraySize;                   // Size of an array to be passed as reference to the OOAPI to be specified by the OOAPI.
	    char* linkTypeName = NULL;
	    DKDatastoreICM* dsICM = (DKDatastoreICM*) ddo->getDatastore();
	    if(dsICM==NULL)
	        throw DKException("DDO's Datastore Not Set.  DDO should be created using DKDatastoreICM.createDDO().");
	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.

        // Obtain a list of all Link Type Names defined in the System.  These will be all of the possible DKLinkCollections that can exist.
	    DKString* possibleLinkTypeNames = dsDefICM->listLinkTypeNames(arraySize); // List of all Link Type Names Possible in the System.

        // For each possible Link Type Name, check for it in the DDO.
	    for (int i = 0; i < arraySize; i++) {
   		    linkTypeName = possibleLinkTypeNames[i]; // get a copy of the next name to check.
	        short dataid = ddo->dataId(DK_CM_NAMESPACE_LINK,possibleLinkTypeNames[i]);  // If 0 is returned, it does not exist in this DDO.
	        if (dataid != 0){ // if it exists, then go through it.
        	    linkCollection = (DKLinkCollection*)(dkCollection*) ddo->getData(dataid); // get the link Collection of this Link Type.

			    // Go through all in that collection and get the link objects.
			    collectionSize = linkCollection->cardinality();
                if (collectionSize > 0) {                      // if there are any items in the Link Collection, go through each.
				    iter = linkCollection->createIterator();   // Create an iterator go go through all Inbound & Outbound Links.
                    while (iter->more()) {                     // while there are items left to iteratate through, continue
					    link = (DKLink*)iter->next()->value(); // get each link object from the individual collection and add to our single list
						linksColl->addElement(link);           // add link to our link Vector.
				    }//end while
                    delete(iter);                              // Free Memory
			    }//end if collectionSize > 0
		    }//end if dataid != 0
	    }//end for all possible link types

        delete[] possibleLinkTypeNames;                        // Free Memory

	    return(linksColl);
    }//end getLinks

   /**
    * Obtain All Inbound Links within the DDO, combined from all Link Collections for each Link
    * Type.  The only difference between this and getLinks() is that this uses an Inbound
    * Iterator and checks for null returned from next();
    * @param ddo  DDO to obtain all Inbound Links from.
    * @return  dkCollection (DKSequentialCollection) containing all of the DKLink objects in the entire DDO that would be considered "Inbound".
    */
    dkCollection* SLinksICM::getInboundLinks(DKDDO* ddo){
        
        // Veriable Declarations
	    DKSequentialCollection* linksColl = new DKSequentialCollection();
	    DKLinkCollection* linkCollection; // Temporary variable to hold each individual link collection that we find in the DDO.
	    dkIterator* iter;                 // Iterator to go through the linkColleciton list.
	    int collectionSize;               // Size of the Link Collection
	    DKLink* link;                     // Temporary link object to get from each link collection and add to our vector
        long arraySize;                   // Size of an array to be passed as reference to the OOAPI to be specified by the OOAPI.
	    char* linkTypeName = NULL;
	    DKDatastoreICM* dsICM = (DKDatastoreICM*) ddo->getDatastore();
	    if(dsICM==NULL)
	        throw DKException("DDO's Datastore Not Set.  DDO should be created using DKDatastoreICM.createDDO().");
	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.

        // Obtain a list of all Link Type Names defined in the System.  These will be all of the possible DKLinkCollections that can exist.
	    DKString* possibleLinkTypeNames = dsDefICM->listLinkTypeNames(arraySize); // List of all Link Type Names Possible in the System.

        // For each possible Link Type Name, check for it in the DDO.
	    for (int i = 0; i < arraySize; i++) {
   		    linkTypeName = possibleLinkTypeNames[i]; // get a copy of the next name to check.
	        short dataid = ddo->dataId(DK_CM_NAMESPACE_LINK,possibleLinkTypeNames[i]);  // If 0 is returned, it does not exist in this DDO.
	        if (dataid != 0){ // if it exists, then go through it.
        	    linkCollection = (DKLinkCollection*)(dkCollection*) ddo->getData(dataid); // get the link Collection of this Link Type.

			    // Go through all in that collection and get the link objects.
			    collectionSize = linkCollection->cardinality();
                if (collectionSize > 0) {             // if there are any items in the Link Collection, go through each.
			        iter = linkCollection->createInboundIterator(); // Create an Inbound Iterator go go through all Inbound Links.
                    while (iter->more()) {                     // while there are items left to iteratate through, continue
					    link = (DKLink*)iter->next()->value(); // get each link object from the individual collection and add to our single list
					    if(link==NULL)                         // if null is returned from next() when dealing with Inbound Iterators, it means that there are not more Inbound Links.
					        break;
						linksColl->addElement(link);           // add link to our link Vector.
				    }//end while
                    delete(iter);                              // Free Memory
			    }//end if collectionSize > 0
		    }//end if dataid != 0
	    }//end for all possible link types

        delete[] possibleLinkTypeNames;                        // Free Memory

	    return(linksColl);
    }//end getInboundLinks

   /**
    * Obtain All Outbound Links within the DDO, combined from all Link Collections for each 
    * Link Type.  The only difference between this and getLinks() is that this uses an Outbound
    * Iterator and checks for null returned from next();
    * @param ddo  DDO to obtain all Inbound Links from.
    * @return  dkCollection (DKSequentialCollection) containing all of the DKLink objects in the entire DDO that would be considered "Outbound".
    */
    dkCollection* SLinksICM::getOutboundLinks(DKDDO* ddo){
        
        // Veriable Declarations
	    DKSequentialCollection* linksColl = new DKSequentialCollection;
	    DKLinkCollection* linkCollection; // Temporary variable to hold each individual link collection that we find in the DDO.
	    dkIterator* iter;                 // Iterator to go through the linkColleciton list.
	    int collectionSize;               // Size of the Link Collection
	    DKLink* link;                     // Temporary link object to get from each link collection and add to our vector
        long arraySize;                   // Size of an array to be passed as reference to the OOAPI to be specified by the OOAPI.
        char* linkTypeName = NULL;
	    DKDatastoreICM* dsICM = (DKDatastoreICM*) ddo->getDatastore();
	    if(dsICM==NULL)
	        throw DKException("DDO's Datastore Not Set.  DDO should be created using DKDatastoreICM.createDDO().");
	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.

        // Obtain a list of all Link Type Names defined in the System.  These will be all of the possible DKLinkCollections that can exist.
	    DKString* possibleLinkTypeNames = dsDefICM->listLinkTypeNames(arraySize); // List of all Link Type Names Possible in the System.

        // For each possible Link Type Name, check for it in the DDO.
	    for (int i = 0; i < arraySize; i++) {
   		    linkTypeName = possibleLinkTypeNames[i]; // get a copy of the next name to check.
	        short dataid = ddo->dataId(DK_CM_NAMESPACE_LINK,possibleLinkTypeNames[i]);  // If 0 is returned, it does not exist in this DDO.
	        if (dataid != 0){ // if it exists, then go through it.
        	    linkCollection = (DKLinkCollection*)(dkCollection*) ddo->getData(dataid); // get the link Collection of this Link Type.

			    // Go through all in that collection and get the link objects.
			    collectionSize = linkCollection->cardinality();
                if (collectionSize > 0) {                      // if there are any items in the Link Collection, go through each.
				    iter = linkCollection->createOutboundIterator(); // Create an Inbound Iterator go go through all Inbound Links.
                    while (iter->more()) {                     // while there are items left to iteratate through, continue
					    link = (DKLink*)iter->next()->value(); // get each link object from the individual collection and add to our single list
					    if(link==NULL)                         // if null is returned from next() when dealing with Outbound Iterators, it means that there are not more Outbound Links.
					        break;
						linksColl->addElement(link);           // add link to our link Vector.
				    }//end while
                    delete(iter);                              // Free Memory
			    }//end if collectionSize > 0
		    }//end if dataid != 0
	    }//end for all possible link types

        delete[] possibleLinkTypeNames;                        // Free Memory

	    return(linksColl);
    }//end getOutboundLinks

   /**
    * Retrieve all Link Content / Data / DDOs within every Link that aren't iteself.  This 
    * function goes through all links.  If the source it not itself, the source is retrieved 
    * with the specified retrieval options.  Similarly, if the target is not itself, the target
    * is retrieved with the specified retrieval options.  Lastly, if the Link Item is not null, 
    * it too is retrieved.
    * @param ddo  DDO to retrieve all obtain all links from.
    * @param retrievalOptions  Retrieval Options to use when retrieving the Link Contents.
    */
    void SLinksICM::retrieveAllLinkContents(DKDDO* ddo, short retrievalOptions){
        
        // Veriable Declarations
	    DKLinkCollection* linkCollection; // Temporary variable to hold each individual link collection that we find in the DDO.
	    dkIterator* iter;                 // Iterator to go through the linkColleciton list.
	    int collectionSize;               // Size of the linkCollection
	    DKLink* dkLink;                   // Temporary link object to get from each link collection and add to our vector
        long arraySize;                   // Size of an array to be passed as reference to the OOAPI to be specified by the OOAPI.
	    char* linkTypeName = NULL;
	    DKDatastoreICM* dsICM = (DKDatastoreICM*) ddo->getDatastore();
	    if(dsICM==NULL)
	        throw DKException("DDO's Datastore Not Set.  DDO should be created using DKDatastoreICM.createDDO().");
	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.

        // Obtain a list of all Link Type Names defined in the System.  These will be all of the possible DKLinkCollections that can exist.
	    DKString* possibleLinkTypeNames = dsDefICM->listLinkTypeNames(arraySize); // List of all Link Type Names Possible in the System.

        // For each possible Link Type Name, check for it in the DDO.
	    for (int i = 0; i < arraySize; i++) {
   		    linkTypeName = possibleLinkTypeNames[i]; // get a copy of the next name to check.
	        short dataid = ddo->dataId(DK_CM_NAMESPACE_LINK,possibleLinkTypeNames[i]);  // If 0 is returned, it does not exist in this DDO.
	        if (dataid != 0){ // if it exists, then go through it.
        	    linkCollection = (DKLinkCollection*)(dkCollection*) ddo->getData(dataid); // get the link Collection of this Link Type.

			    // Go through all in that collection.
			    collectionSize = linkCollection->cardinality();
			    if (collectionSize > 0) { // if there are any items in the Link Collection, go through each.
                    iter = linkCollection->createIterator();  // Access all links, Inbound & Outbound.
                
                    while(iter->more()){  // Go through all links in the list
                        dkLink = (DKLink*) iter->next()->value(); // Move pointer to next element & return that object.

                        DKDDO* source   = (DKDDO*) dkLink->getSource();  
                        DKDDO* target   = (DKDDO*) dkLink->getTarget();  
                        DKDDO* linkItem = (DKDDO*) dkLink->getLinkItem(); // The optional description item.  Null if not specified.

                        // If the DDOs are not a reference to itself, retrieve them.
                        if(source!=ddo)
                            source->retrieve(retrievalOptions);
                        if(target!=ddo)
                            target->retrieve(retrievalOptions);
                        if(linkItem!=NULL)
                            linkItem->retrieve(retrievalOptions);    
                    }//end while
                    delete(iter);         // Free memory
			    }//end if collectionSize > 0
		    }//end if dataid != 0
	    }//end for all possible link types

        delete[] possibleLinkTypeNames;                      // Free Memory

    }//end retrieveAllLinkContents
