/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

System-Defined Semantic Types
    For information on System-Defined Semantic Types, please refer to the SItemCreationICM
    Sample.
    
Using Semantic Types
    For information on using Semantic Types, please refer to the SItemCreationICM Sample.
    
User-Defined Semantic Types
    User-Defined Semantic Types may be created and used by User Programs.  It is up to the
    User Program to enforce these semantics when used.

---------------------------------------
Semantic Types Defined in Sample
---------------------------------------
    S_myDocument

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKSemanticTypeDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SSemanticTypeCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SSemanticTypeCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining Semantic Types
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Defining Semantic Types
 *                    Creating a New Semantic Type Definition
 *                    Saving / Adding Semantic Type Definition in Datastore Definition
 *                    Semantic Type Name Requirements
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SSemanticTypeCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Semantic Types defined here must not exist.  If any have already been 
 *                    created, please run SSemanticTypeDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SSemanticTypeCreationICM_launch.cpp
 *                    SSemanticTypeCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSemanticTypeCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SSemanticTypeCreationICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SSemanticTypeCreationICM"       << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Define a Semantic Type
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using Semantic Type names that may already be in use.
            //
            // For more information on options that can be set for Semantic Type
            // definitions, please consult the Programming Reference.
            //
            // Defining Semantic Type Named "S_myDocument".
            cout << "Defining Semantic Type 'S_myDocument'..." << endl;

                DKSemanticTypeDefICM* semTypeDef = new DKSemanticTypeDefICM(dsICM); // Create a new Semantic Type Definition object.
                
                semTypeDef->setName("S_myDocument");                                // Choose a name with a maximum of 15 characters.
                semTypeDef->setDescription("My Document");
                
                semTypeDef->add();                                                  // Add it to the persistent datastore.

                delete(semTypeDef);                                                 // Free Memory

            cout << "Defined Semantic Type 'S_myDocument'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Define a new Semantic Type.
    * For more information on options that can be set for Semantic Type Definitions,
    * please consult the Programming Reference.
    * @param dsICM        Connect DKDatastoreICM object to define the attribute in.
    * @param name         Name of the new attribute.
    * @param description  More detailed description than name.
    * @return Returns a reference to the newly defined DKSemanticTypeDefICM that was added to the datastore.
    **/
    DKSemanticTypeDefICM* SSemanticTypeCreationICM::defineSemanticType(DKDatastoreICM* dsICM, char* name, char* description){
        cout << "Defining Semantic Type '" << name << "'..." << endl;

            DKSemanticTypeDefICM* semTypeDef = new DKSemanticTypeDefICM(dsICM); // Create a new Semantic Type Definition object.
                
            semTypeDef->setName(name);                                          // Choose a name with a maximum of 15 characters.
            semTypeDef->setDescription(description);
                
            semTypeDef->add();                                                  // Add it to the persistent datastore.

        cout << "Defined Semantic Type '" << name << "'." << endl;

        return(semTypeDef);
    }
