/************************************************************************/
/* @copyright(disclaimer)                                               */  
/*                                                                      */  
/* Digital Library Version 2 - 5765-C86.                                */   
/* (C) Copyright IBM Corp. 1997.  All Rights Reserved.                  */  
/*                                                                      */  
/* US Government Users Restricted Rights                                */  
/* Use, duplication or disclosure restricted by GSA ADP Schedule        */  
/* Contract with IBM Corp.  Licensed Materials - Property of IBM        */  
/*                                                                      */  
/* DISCLAIMER OF WARRANTIES :                                           */  
/*                                                                      */  
/* Permission is granted to copy and modify this code, and to           */
/* distribute modified versions provided that both the copyright        */
/* notice and this permission notice appear in all copies and modified  */
/* versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS   */  
/* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM  */
/* WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE */
/* OF THE CODE.                                                         */
/*                                                                      */  
/* @endCopyright                                                        */
/************************************************************************/

/********************************************************
 * TMimeTypeV4.cpp - test the xdo CRUD and open.
 * (the default open handler is "iexplore", the directory
 *  where the iexplore is should be in the PATH.  If
 *  the retrieved file is a text file then set the open  
 *  handler to "notepad".  This test case will run in 
 *  Window NT, for AIX you need to set open handler to "xv".
 ********************************************************/
#include <iostream.h>
#include <stdio.h>  
#include <stdlib.h>
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKDatastoreV4.hpp>
#include <DKException.hpp>
#include <DKHandle.hpp>
#include <DKBlobV4.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //== Following default partId and itemId will be provided if you
    //== run this test case without parameters, If you know existing
    //== partId and itemId, invoke: txdoCRUD <partId> <itemId>
    //== unsigned long    partId = 1;  
    //== DKString   itemId = "DA99120AA.AAK";
    //==   
    //== Following fileNames will be used to add a new part object,
    //== It must exist in the same directory, change the fileName
    //== if you want to use other file
    //== DKString   fileName = "p:\\cmvc\\dk\\samples\\cpp\\v4\\choice.gif";
    //==   
    //== Following fileName will be used for update the part object,
    //== It must exist in the same directory, change the fileName
    //== if you want to update with other file
    //== fileName = "p:\\cmvc\\dk\\samples\\cpp\\v4\\cheetah.gif";
                                                
    unsigned long    partId = 1;    
    DKString itemId = "DB02213AA.AAK";  // for stealth5
     
    cout << "argc is " << argc << endl;
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TxdoCRUDV4 <libsrv> <userid> <pw> <partId> <itemId>" << endl;
     return;
    }
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    if (argc == 4)
    {
      cout<<"no parameter for partId, itemId, following default will be provided:"<<endl;
      cout<<"   default partId = "<<partId<<endl;
      cout<<"   default itemId = "<<itemId<<endl;
    }
    else if (argc == 5)
    {
     partId = atoi(argv[4]);
     cout<<"partId entered: "<<argv[4]<<endl;
     cout<<"   The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 6)
    {
     partId = atoi(argv[4]);
     itemId = DKString(argv[5]);
     cout<<"partId/itemId entered: "<<argv[4]<<" "<<argv[5]<<endl;
    }

    DKDatastoreV4 dsV4; // must be outside try/catch block
    try 
    {
    //======== connect to Datastore =======================
    //DKDatastoreV4 dsV4("CC2MIMEFILE=(p:\\cmvc\\samples\\CMBCC2MIME.INI);");
    cout << "Connecting to datastore...." << endl;
    dsV4.connect(libsrv, userid, pw);
    cout << "Datasore connected" << endl;
    //long  hsession = (long)(dsV4.connection()->handle());
    // cout << "datastore handle = " << hsession << endl;
        
    DKBlobV4* axdo = new DKBlobV4(&dsV4);
    DKPidXDOV4*  apid = new DKPidXDOV4;
    
    //== create pid ==
    apid ->setPartId(partId);
    apid->setId(itemId);
    axdo ->setPid(apid);
    axdo->setRepType("        "); 
    axdo->setContentClass(DK_V4_CC_GIF);
    cout <<"  content class = " << axdo->getContentClass() << endl;
    //====== retrieve the existing part object =============
    
    cout <<"about to call retrieve()"<<endl;
    axdo->retrieve(); 
    cout <<"  content class = " << axdo->getContentClass() << endl;
    cout <<"  mime type = " << axdo->mimeType() << endl;
    axdo->setContentClass(DK_V4_CC_MIDI);
    cout <<"  content class = " << axdo->getContentClass() << endl;
    cout <<"  mime type = " << axdo->mimeType() << endl;
    axdo->setContentClass(500);
    cout <<"  content class = " << axdo->getContentClass() << endl;
    cout <<"  mime type = " << axdo->mimeType() << endl;
    
    //======== disconnect Datastore ======================= 
    dsV4.disconnect();
    cout<<"datastore disconnected"<<endl;
    cout << "done!" << endl;
    } //end of try
    catch(DKException &exc)
    {
      cout << "Error id: " << exc.errorId() << endl;
      cout << "Exception id: " << exc.exceptionId() << endl;
      
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      
      cout << "Exception Class Name: " << exc.name() << endl;
    }
 }

