/**************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/**************************************************************************/

// sample query using DKCQExpr, attribute and schema mapping for VI400
/*
 * Filename       : TQueryExprV4.cpp
 * Description    : test query expression, and attribute and schema mapping
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKAny.hpp>
#include <DKDatastoreV4.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKAttrDefV4.hpp>
#include <DKAttrMappingFed.hpp>
#include <dkResultsetCursor.hpp>

void displayDDO(DKDDO* ddo);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)  
    {
       cout << "Usage: " << endl;
       cout << "  TQueryExprV4 <Library Server> <userid> <password>" << endl;
       return;
    }
    
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw     = DKString(argv[3]);
    DKDatastoreV4 dsV4;
    
    try 
    {

      // NOTE:
      // Fed. Entity, fed. Attr, schema mapping, and search template 
      // information could have been obtained from the Federated Datastore.
      // For clarity of this example and for most practical purposes, 
      // they are created here in memory.

      // create federated attributes :
      dkAttrDef* fAuthor = new DKAttrDefFed(&dsV4);
      fAuthor->setEntityName("FedSample");
      fAuthor->setName("FedAuthor");
      fAuthor->setType(DK_CM_VSTRING);
      fAuthor->setId((short)95);
      fAuthor->setSize(50);
      fAuthor->setScale(1);
      fAuthor->setPrecision(50);
      fAuthor->setNullable(true);
      fAuthor->setStringType(DK_ATTR_ALPHA);
   
      dkAttrDef* fTitle = new DKAttrDefFed(&dsV4);
      fTitle->setEntityName("FedSample");
      fTitle->setName("FedTitle");
      fTitle->setType(DK_CM_VSTRING);
      fTitle->setId((short)99);
      fTitle->setSize(100);
      fTitle->setScale(1);
      fTitle->setPrecision(100);
      fTitle->setNullable(true);
      fTitle->setStringType(DK_ATTR_ALPHA);
   
      dkAttrDef* fSubject = new DKAttrDefFed(&dsV4);
      fSubject->setEntityName("FedSample");
      fSubject->setName("FedDate");
      fSubject->setType(DK_CM_VSTRING);
      fSubject->setId((short)100);
      fSubject->setSize(100);
      fSubject->setScale(1);
      fSubject->setPrecision(100);
      fSubject->setNullable(true);
      fSubject->setStringType(DK_ATTR_ALPHA);
         
      cout << "  *** federated attributes created in memory ***" << endl;
   
      // create native attributes : VI400 index_class V4DEMO
      dkAttrDef* nbAuthor = new DKAttrDefV4();
      nbAuthor->setEntityName("V4DEMO");
      nbAuthor->setName("V4USERID");
      nbAuthor->setType(DK_CM_VSTRING);
      nbAuthor->setId((short)95);
      nbAuthor->setSize(50);
      nbAuthor->setScale(1);
      nbAuthor->setPrecision(50);
      nbAuthor->setNullable(true);
      nbAuthor->setStringType(DK_ATTR_ALPHA);
   
      dkAttrDef* nbTitle = new DKAttrDefV4();
      nbTitle->setEntityName("V4DEMO");
      nbTitle->setName("V4TITLE");
      nbTitle->setType(DK_CM_VSTRING);
      nbTitle->setId((short)99);
      nbTitle->setSize(100);
      nbTitle->setScale(1);
      nbTitle->setPrecision(100);
      nbTitle->setNullable(true);
      nbTitle->setStringType(DK_ATTR_ALPHA);
   
      dkAttrDef* nbSubject = new DKAttrDefV4();
      nbSubject->setEntityName("V4DEMO");
      nbSubject->setName("V4DATE");
      nbSubject->setType(DK_CM_VSTRING);
      nbSubject->setId((short)100);
      nbSubject->setSize(100);
      nbSubject->setScale(1);
      nbSubject->setPrecision(100);
      nbSubject->setNullable(true);
      nbSubject->setStringType(DK_ATTR_ALPHA);
   
      cout << "  *** native attributes created in memory ***" << endl;
   
      //  create schema mapping : entity and attr mappings
      // maps "FedAuthor" to "V4USERID"
      
      dkAttrMapping* mapAuthor = new DKAttrMappingFed();
      mapAuthor->addMappedAttr(fAuthor);
      mapAuthor->addMapToAttr(nbAuthor);
   
      // maps "FedTitle" to "V4TITLE"
      dkAttrMapping* mapTitle = new DKAttrMappingFed();
      mapTitle->addMappedAttr(fTitle);
      mapTitle->addMapToAttr(nbTitle);
   
      // maps "FedDate" to "V4DATE"
      dkAttrMapping* mapSubject = new DKAttrMappingFed();
      mapSubject->addMappedAttr(fSubject);
      mapSubject->addMapToAttr(nbSubject);
   
      cout << "  *** attribute mapping created in memory *** " << endl;
   
      dkSchemaMapping* sm = new DKSchemaMappingFed();
      sm->setServerName(libsrv);
      sm->setServerType(DK_V4_DSTYPE);
      sm->setEntityName("V4DEMO");
      sm->setMappedEntityName("FedSample");
   
      // add attribute mapping to this schema
      sm->addAttrMapping(mapAuthor);
      sm->addAttrMapping(mapTitle);
      sm->addAttrMapping(mapSubject);
   
      cout << "  *** schema mapping created, name = " 
           << sm->getName() << " ***" << endl;

      cout << "  \t*** attribute mapping: " << endl;
      DKString fedEnt[] = {"FedAuthor", "FedTitle", "FedDate"};
      dkAttrMapping* amap;
      long len = 1;
      DKString* tMapToDispL;
         
      // debug the following methods:
      for (int m = 0; m < 3; m++) 
      {
        amap = sm->getAttrMappingByMappedName(fedEnt[m]);
        tMapToDispL = amap->listMapToAttrNames(len);
        cout << "  \ttMapToDispL = " << tMapToDispL 
             << ", len = " << len << endl;
        cout << "  \t" << fedEnt[m] << " -> " << tMapToDispL[0] << endl;
        delete [] tMapToDispL;
      }
   
      //  Create a compound query expression:
      //  String values must be enclosed within a pair of single-quotes or 
      //    backslah-double-quote (\"). See examples below.
   
      // Create a query term :
      // Search Term = 
      //   ((FedAuthor <> 'EDTEST01') AND (FedTitle <> NULL)) OR (FedAuthor LIKE 'Basu%')
        
      DKQTerm* sTerm = new DKQTerm(new DKQTerm("FedAuthor", DK_CM_OPCODE_NEQ, 
                                               "'EDTEST01'"),
                                     DK_CM_OPCODE_AND,
                                     new DKQTerm("FedTitle", DK_CM_OPCODE_NEQ, 
                                                 "NULL"));  
      sTerm = sTerm->or(new DKQTerm("FedAuthor", DK_CM_OPCODE_LIKE, 
                                    "'Basu%'"));

      cout << "  Search Term = " << sTerm->toString() << endl;
   
      // Create a query sub-expression
      DKQSubExpr* qse = new DKQSubExpr();
      qse->setTranslation(TRUE);
      qse->setMappingName(sm->getName());
      qse->setQueryTerm(sTerm);
      qse->setEntityName("FedSample");
      DKString dsl[3] = {"FedTitle","FedDate","FedAuthor"};
      qse->setDisplayList(dsl, 3);

      cout << "  mapping name in sub-query = " 
           << qse->getMappingName() 
           << "\n  mapped entity name = " << qse->getEntityName()
           << endl;
   
      // Generic Options : MAX_RESULTS=6,CONTENT=YES
      DKNVPair opt[1];
      opt[0].set("MAX_RESULTS", DKString("6"));
      long optCount = 1;
      qse->setOptionList(opt,optCount);
       
      for (int i = 0; i < optCount; i++)
        cout << "  sub-query expression option[" << i << "] : " 
             << opt[i].getName() << "=" <<  opt[i].getValue() << endl;
   
      // Create a query expression
      DKQExpr* qe = new DKQExpr();
      qe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
      qe->addSubQuery(qse);
      opt[0].set("CONTENT", DKString("YES"));
      // opt[0].set("CONTENT", DKString("ATTRONLY"));
      qe->setOptionList(opt, optCount);
       
      for (int j = 0; j < optCount; j++)
        cout << "  query expression option[" << j << "] : " 
             << opt[j].getName() << "=" << opt[j].getValue() << endl;
   
      // Create a compound query expression
      DKCQExpr* cqe = new DKCQExpr(qe);
      cqe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
   
      // All set; Connect and execute the compound query expression
       cout << "  *** connecting to " << libsrv << " ... *** " << endl;
      dsV4.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;

      // test retrieving SM from DS:
      cout << "  before calling getMapping()" << endl;
      dkSchemaMapping* temp = dsV4.getMapping("NotHere");
       
      if (temp == NULL)
        cout << "  after calling getMapping(), temp == null"  << endl;
      else 
        cout << "  after calling getMapping(), temp != null" << endl;
       
      delete temp;
      long count = 0;
      DKString* names = dsV4.listMappingNames(count);
       
      if (count == 0 || names == NULL)
        cout << "  mapping is null" << endl;
      else 
      {
        cout << "  the first mapping = " << names[0] << endl;
        delete [] names;
      }
   
      // Do not forget to register the schema mapping
      DKNVPair nvMap("SCHEMA", sm);
      DKString mappingName = dsV4.registerMapping(&nvMap);
      // remember that sm is absorbed by registerMapping so do not
      //  delete it later on
      cout << "  registered mapping name = " << mappingName << endl;
      cout << "  ready to execute the query expression " << endl;

      dkResultSetCursor* rsc = dsV4.execute(cqe);

      cout << "  query execution successful " << endl;
          
      // Process the result set cursor :
      if (rsc == NULL) 
        cout << "  *** cursor is null" << endl;
      else 
      {
        cout << "  *** cursor cardinality = " << rsc->cardinality() << endl;
        cout << "  *** the results are... " << endl << endl;
        DKDDO* ddo;
          
        while (rsc->isValid())  
        {
          ddo = rsc->fetchNext();
          if (ddo != NULL) 
          {  
            cout << "  pidString: " << ((DKPid&)ddo->getPid()).pidString()
                 << endl;
            displayDDO(ddo);
            delete ddo;
            ddo = NULL;
          }
        }
      }
       
      delete rsc;
      delete cqe;
      // delete sm; // sm is absorbed by the registerMapping; do not delete

      // disconnects
      cout << "\n***disconnect from the datastore V4 ... ***" << endl;
      dsV4.disconnect();
      cout << "*** datastore " << libsrv << " disconnected ***" << endl;

    }
    catch(DKException &exc)
    {
      //   exc.printTrace();
      cout << "Error id:             " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text:        " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename:          " << p->fileName() << endl;
         cout << "Function:          " << p->functionName() << endl;
         cout << "LineNumber:        " << p->lineNumber() << endl;
      }
      
      cout << "Exception Class Name: " << exc.name() << endl;
    }
   
    cout << "done!" << endl;   
}

// simple method to show DDO hit 
void displayDDO(DKDDO* ddo) 
{
    DKString   dataName;
    short      cnt;
    DKAny      any;
 
    if (ddo != NULL) 
    {
       cnt = ddo->dataCount();
       for (short i = 1; i <= cnt; i++) 
       {
           any = ddo->getData(i);
           dataName = ddo->getDataName(i);
           cout << "  " << i << ": Attribute Name: " << dataName;
           
           if (!any.isNull())
             cout << ",\t   Value: " << any << endl;
           else 
             cout << ",\t   Value: null"  << endl;
       }  
       
       cout << endl;
    }
}
