/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// Class Name  = SampleGuiLogon
// Description = Sample visual connection bean
// Author      = afic
// Date        = 04/01/1999
//
//*****************************************************************************
import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.io.*;
import java.util.Vector;

//import  javax.swing.*;
//import  javax.swing.border.*;

import javax.swing.*;
import javax.swing.border.*;

//*****************************************************************************
// class/interface definition
//*****************************************************************************
/**
 * Sample GUI bean to prompt for user ID and password. Can be used with CM
 * connection bean to perform the functions.
 *
 * <pre>
 *   exported properties    logon user ID and password. Can be vetoed if server
 *                          has been connected.
 *   source of events       CMBConnectionRequestEvent - to connect, disconnect
 *                          VetoableChange event      - to export properties
 *                          PropertyChange event      - to export properties
 * </pre>
 */
public class SampleGuiLogon implements Serializable, CMBUtilConstant, CMBConnectionReplyListener {

    //*****************************************************************************
    // constructor
    //*****************************************************************************
    /**
     * Default constructor.
     */
    public SampleGuiLogon() {
        listenerVector = new Vector();
        initGUI();
    }

    /**
     * Destructor,  dispose the window.
     */
    public void finalize() {
        frame.dispose();
    }

    /**
     * Destructor,  dispose the window.
     */
    public void hide() {
        frame.setVisible(false);
    }

    //*****************************************************************************
    // Event target: method implementation
    //*****************************************************************************
    // None
    //*****************************************************************************

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // VetoableChangeEvent
    //*****************************************************************************
    /**
     * VetoableChangeEvent registration method.
     * @param   listener - listener to be registered
     */
    public void addVetoableChangeListener(VetoableChangeListener listener) {
        vcSupport.addVetoableChangeListener(listener);
    }

    /**
     * VetoableChangeEvent unregistration method.
     * @param   listener - listener to be unregistered
     */
    public void removeVetoableChangeListener(VetoableChangeListener listener) {
        vcSupport.removeVetoableChangeListener(listener);
    }

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // PropertyChangeEvent
    //*****************************************************************************
    /**
     * PropertyChangeEvent registration method.
     * @param   listener - listener to be registered
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        pcSupport.addPropertyChangeListener(listener);
    }

    /**
     * PropertyChangeEvent unregistration method.
     * @param   listener - listener to be unregistered
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pcSupport.removePropertyChangeListener(listener);
    }

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // CMBConnectionEvent
    //*****************************************************************************
    /**
     * CMBConnectionEvent registration method.
     * It ignores a registration request if the listener is already registered.
     * @param listener listener to be registered
     */
    public void addCMBConnectionRequestListener(CMBConnectionRequestListener listener) {
        if (!listenerVector.contains(listener))
            listenerVector.addElement(listener);
    }

    /**
     * CMBConnectionRequestEvent unregistration method.
     * @param listener listener to be unregistered
     */
    public void removeCMBConnectionRequestListener(CMBConnectionRequestListener listener) {
        if (listenerVector.contains(listener))
            listenerVector.removeElement(listener);
    }

    /*
     * CMBConnectionRequestEvent notification method.
     * The event firing order is based on the order of registration.
     * @param evt the CMBConnectionRequestEvent
     */
    private void postCMBConnectionRequest(CMBConnectionRequestEvent evt) {
        // VisualCafe 3.0 support 04/21/99/afic
        connRequestEvent = evt;
        // VisualCafe 3.0 support 04/21/99/afic

        Vector v;
        synchronized (this) {
            v = (Vector) listenerVector.clone();
        }

        int cnt = v.size();
        CMBConnectionRequestListener listener = null;

        for (int i = 0; i < cnt; ++i) {
            listener = (CMBConnectionRequestListener) v.elementAt(i);
            listener.onCMBConnectionRequest(evt);
        }
    }

    /*
     * CMBConnectionRequestEvent notification method.
     * @param id         the event type id
     */
    private void postCMBConnectionRequest(int id) {
        postCMBConnectionRequest(new CMBConnectionRequestEvent(this, id));
    }

    /**
     *  Listens to CMBConnectionReplyEvent, if reply status is ok, hide the
     *  the window.  Otherwise, post error message to ask user to reenter correct
     *  userid and password pair.
     *  @param evt CMBConnectionReplyEvent object
     */
    public void onCMBConnectionReply(CMBConnectionReplyEvent evt) {
        int status = evt.getStatus();
        if (status == CMBBaseConstant.CMB_STATUS_OK) {
            frame.setVisible(false);
        }
    }

    //*****************************************************************************
    // property accessors
    //*****************************************************************************

    /**
     * Property getter.
     * @return  user ID
     */
    public String getUserid() {
        return userid;
    }

    /**
     * Property getter.
     * @return  password
     */
    public String getPassword() {
        return passwd;
    }

    /**
     * Property getter.
     * @return server name.
     */
    public String getServerName() {
        return serverName;
    }

    /**
     * Property getter.
     * @return server name.
     */
    public String[] getServer() {
        return serverNames;
    }

    /**
     * Property setter.
     * @param   newValue new user ID
     */
    public void setUserid(String newValue) throws PropertyVetoException {

        // VisualCafe 3.0 support 04/21/99/afic
        propertyEvent = new PropertyChangeEvent(this, PROP_USERID, userid, newValue);
        vetoableEvent = new PropertyChangeEvent(this, PROP_USERID, userid, newValue);
        /* jdk 1.2 only
        vcSupport.fireVetoableChange(propertyEvent);
        pcSupport.firePropertyChange(vetoableEvent);
        */
        // VisualCafe 3.0 support 04/21/99/afic

        vcSupport.fireVetoableChange(PROP_USERID, userid, newValue);
        pcSupport.firePropertyChange(PROP_USERID, userid, newValue);
        // not vetoed, notify change
        userid = newValue; // update prop value
        useridField.setText(userid); // update GUI
    }

    /**
     * Property setter.
     * @param   newValue new CM password
     */
    public void setPassword(String newValue) throws PropertyVetoException {
        // VisualCafe 3.0 support 04/21/99/afic
        propertyEvent = new PropertyChangeEvent(this, PROP_PASSWORD, passwd, newValue);
        vetoableEvent = new PropertyChangeEvent(this, PROP_PASSWORD, passwd, newValue);
        /* jdk 1.2 only
        vcSupport.fireVetoableChange(propertyEvent);
        pcSupport.firePropertyChange(vetoableEvent);
        */
        // VisualCafe 3.0 support 04/21/99/afic

        vcSupport.fireVetoableChange(PROP_PASSWORD, passwd, newValue);
        pcSupport.firePropertyChange(PROP_PASSWORD, passwd, newValue);
        // not vetoed, notify change
        passwd = newValue; // update prop value
        passwdField.setText(passwd); // update GUI
    }

    /**
     * Property setter.
     * @param   newValue new CM password
     */
    public void setServerName(String newValue) throws PropertyVetoException {
        // VisualCafe 3.0 support 04/21/99/afic
        propertyEvent = new PropertyChangeEvent(this, PROP_SERVER, serverName, newValue);
        vetoableEvent = new PropertyChangeEvent(this, PROP_SERVER, serverName, newValue);
        /* jdk 1.2 only
        vcSupport.fireVetoableChange(propertyEvent);
        pcSupport.firePropertyChange(vetoableEvent);
        */
        // VisualCafe 3.0 support 04/21/99/afic

        vcSupport.fireVetoableChange(PROP_SERVER, serverName, newValue);
        pcSupport.firePropertyChange(PROP_SERVER, serverName, newValue);
        // not vetoed, notify change
        serverName = newValue; // update prop value
        serverField.setSelectedItem(serverName); // update GUI
    }

    /**
     * Sets new list of server names
     * @param   newValue new server names
     */
    public void setServer(String[] newValues) throws PropertyVetoException {
        if (newValues != null) {
            for (int i = 0; i < newValues.length; i++) {
                serverField.addItem(newValues[i]);
            }
            serverNames = newValues;
        }
    }

    /**
     * Property getter.
     * @return  window title
     */
    public String getWindowTitle() {
        return frame.getTitle();
    }

    /**
     * Property setter.
     * @param   newValue new window title
     */
    public void setWindowTitle(String newValue) {
        if (newValue != null)
            frame.setTitle(newValue);
    }

    /**
     * Property getter.
     * @return  user ID field label
     */
    public String getUseridLabel() {
        return useridLabel.getText();
    }

    /**
     * Property setter.
     * @param   newValue new user ID field label
     */
    public void setUseridLabel(String newValue) {
        if (newValue != null)
            useridLabel.setText(newValue);
    }

    /**
     * Property getter.
     * @return  password field label
     */
    public String getPasswordLabel() {
        return passwdLabel.getText();
    }

    /**
     * Property setter.
     * @param   newValue new password field label
     */
    public void setPasswordLabel(String newValue) {
        if (newValue != null)
            passwdLabel.setText(newValue);
    }

    /**
     * Property getter.
     * @return  close button label
     */
    public String getCloseLabel() {
        return closeButton.getText();
    }

    /**
     * Property setter.
     * @param   newValue new close button label
     */
    public void setCloseLabel(String newValue) {
        if (newValue != null)
            closeButton.setText(newValue);
    }

    /**
     * Property getter.
     * @return  connect button label
     */
    public String getConnectLabel() {
        return connectButton.getText();
    }

    /**
     * Property setter.
     * @param   newValue new connect button label
     */
    public void setConnectLabel(String newValue) {
        if (newValue != null)
            connectButton.setText(newValue);
    }

    /**
     * Property getter.
     * @return  disconnect button label
     */
    public String getDisconnectLabel() {
        return disconnButton.getText();
    }

    /**
     * Property setter.
     * @param   newValue new disconnect button label
     */
    public void setDisconnectLabel(String newValue) {
        if (newValue != null)
            disconnButton.setText(newValue);
    }

    /**
     * Property getter.
     * @return  cancel button label
     */
    public String getCancelLabel() {
        return cancelButton.getText();
    }

    /**
     * Property setter.
     * @param   newValue new cancel button label
     */
    public void setCancelLabel(String newValue) {
        if (newValue != null)
            cancelButton.setText(newValue);
    }

    public boolean isVisible() {
        return frame.isVisible();
    }

    //*****************************************************************************
    // Private methods
    //*****************************************************************************
    // Serialize transient data
    //*****************************************************************************
    /**
     * @serialData the data is serialized in the following sequence:
     * <pre>
     *    this object
     *    event listeners if the event listener is serializable
     *    a null object   tells the end of serialization
     * </pre>
     */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.defaultWriteObject();

        Vector v = null;
        synchronized (this) {
            if (listenerVector.size() > 0)
                v = (Vector) listenerVector.clone();
        }

        if (v != null) {
            int cnt = v.size();
            CMBSchemaReplyListener listener = null;
            for (int i = 0; i < cnt; ++i) {
                listener = (CMBSchemaReplyListener) v.elementAt(i);
                if (listener instanceof Serializable)
                    stream.writeObject(listener);
            }
        }

        stream.writeObject(null); // tells the end of listener
    }

    //*****************************************************************************
    // De-serialize/initialize transient data
    //*****************************************************************************
    /**
     * @serialData the data is deserialized in the following sequence:
     * <pre>
     *    this object
     *    event listeners may be none
     *    a null object   the marker of serialization
     * </pre>
     */
    private void readObject(ObjectInputStream stream) throws Exception {
        listenerVector = new Vector(); // always initialize listener list
        stream.defaultReadObject();

        Object o;
        while ((o = stream.readObject()) != null)
            listenerVector.addElement((CMBSchemaReplyListener) o);
        frame.setVisible(true);
    }

    //*****************************************************************************
    // Fire propertyChange or vetoableChange when a text field is losing focus
    // Return: true  - need to update property with new value
    //         false - change vetoed, no need to update property with new value
    //*****************************************************************************
    private boolean
    //+++ temp workaround
    //fireVCFromField(JTextField fld, String propName, String propValue)
    fireVCFromField(TextField fld, String propName, String propValue)
    //+++ temp workaround
    {
        if (!(fld.getText().equals(propValue))) {
            try {

                // VisualCafe 3.0 support 04/21/99/afic
                propertyEvent = new PropertyChangeEvent(this, propName, propValue, fld.getText());
                vetoableEvent = new PropertyChangeEvent(this, propName, propValue, fld.getText());
                /* jdk 1.2 only
                vcSupport.fireVetoableChange(propertyEvent);
                pcSupport.firePropertyChange(vetoableEvent);
                */
                // VisualCafe 3.0 support 04/21/99/afic

                vcSupport.fireVetoableChange(propName, propValue, fld.getText());
                pcSupport.firePropertyChange(propName, propValue, fld.getText());
                return true;
            } catch (PropertyVetoException ve) {
                fld.setText(propValue); // restore GUI with old values
            }
        }
        return false;
    }

    //*****************************************************************************
    // initGUI
    //*****************************************************************************
    private void initGUI() {

        frame = new JFrame("Welcome to IBM Enterprise Information Portal Sample Application");
        mainPane = new CMBBasePanel();

        Border raisedBorder = new SoftBevelBorder(BevelBorder.RAISED);
        Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

        //*****************************************************************************
        // Widgets creation and event listener setup
        //*****************************************************************************
        // Center panel
        //*****************************************************************************
        useridLabel = new JLabel("User ID");
        passwdLabel = new JLabel("Password");
        serverLabel = new JLabel("Server Name");

        // +++ temp workaround
        //useridField   = new JTextField(15);
        //passwdField   = new JPasswordField(15);
        useridField = new TextField(15);
        passwdField = new TextField(15);
        passwdField.setEchoChar('*');
        // +++ temp workaround
        serverField = new JComboBox();
        serverField.addItemListener(new LogonItemListener());

        FocusListener fl = new SampleFocusAdapter() {
            public void focusLost(FocusEvent evt) {

                    //+++ temp workaround
        //if (!(evt.getSource() instanceof JTextField)) return;
        //JTextField fld = (JTextField) evt.getSource();
    if (!(evt.getSource() instanceof TextField))
                    return;
                TextField fld = (TextField) evt.getSource();
                //+++ temp workaround

                if (fld == useridField) {
                    if (fireVCFromField(useridField, PROP_USERID, userid)) {
                        userid = useridField.getText();
                    }
                } else if (fld == passwdField) {
                    if (fireVCFromField(passwdField, PROP_PASSWORD, passwd)) {
                        passwd = passwdField.getText();
                    }
                }
                evt = null;
            }
        };

        useridField.addFocusListener(fl);
        passwdField.addFocusListener(fl);

        useridLabel.setLabelFor(useridField);
        passwdLabel.setLabelFor(passwdField);

        //+++ temp workaround
        //useridField.setBorder(loweredBorder);
        //passwdField.setBorder(loweredBorder);
        //+++ temp workaround

        JPanel fieldPane = new JPanel() {
            public Insets getInsets() {
                return new Insets(20, 20, 20, 20);
            }
        };

        fieldPane.setLayout(new GridLayout(3, 2, 4, 4));
        fieldPane.add(useridLabel);
        fieldPane.add(useridField);
        fieldPane.add(passwdLabel);
        fieldPane.add(passwdField);
        fieldPane.add(serverLabel);
        fieldPane.add(serverField);

        //*****************************************************************************
        // Bottom panel
        //*****************************************************************************
        closeButton = new JButton("Close");
        connectButton = new JButton("Connect");
        disconnButton = new JButton("Disconnect");
        cancelButton = new JButton("Cancel Connect") {
            public Insets getInsets() {
                return new Insets(6, 12, 6, 12); // make it spacy
            }
        };

        closeButton.setActionCommand(_cmdClose);
        connectButton.setActionCommand(_cmdConnect);
        disconnButton.setActionCommand(_cmdDisconnect);
        cancelButton.setActionCommand(_cmdCancel);

        CMBActionAdapter al = new CMBActionAdapter() {
            public void actionPerformed(ActionEvent evt) {
                String cmd = evt.getActionCommand();

                if (cmd.equals(_cmdConnect))
                    postCMBConnectionRequest(CMBConnectionRequestEvent.CMB_REQUEST_CONNECT);

                else if (cmd.equals(_cmdDisconnect))
                    postCMBConnectionRequest(CMBConnectionRequestEvent.CMB_REQUEST_DISCONNECT);

                else if (cmd.equals(_cmdCancel))
                    postCMBConnectionRequest(CMBConnectionRequestEvent.CMB_REQUEST_CANCEL_CONNECT);

                else if (cmd.equals(_cmdClose))
                    frame.setVisible(false);
                evt = null;
            }
        };
        closeButton.addActionListener(al);
        connectButton.addActionListener(al);
        disconnButton.addActionListener(al);
        cancelButton.addActionListener(al);

        closeButton.setBorder(raisedBorder);
        connectButton.setBorder(raisedBorder);
        disconnButton.setBorder(raisedBorder);
        cancelButton.setBorder(raisedBorder);

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new GridLayout(1, 4, 4, 0));
        buttonPane.add(closeButton);
        buttonPane.add(connectButton);
        buttonPane.add(disconnButton);
        buttonPane.add(cancelButton);

        //*****************************************************************************
        // Add widgets to window frame
        //*****************************************************************************
        mainPane.addCenterPanel(fieldPane);
        mainPane.addBottomPanel(buttonPane);
        frame.getContentPane().add(mainPane, BorderLayout.CENTER);
        frame.pack();
        CMBUtil.setLocation(frame, CMBUtil.CENTER);
        frame.setVisible(true);
        frame.requestFocus();
    }

    //*****************************************************************************
    // private data
    //*****************************************************************************
    private final String PROP_USERID = "userid";
    private final String PROP_PASSWORD = "password";
    private final String PROP_SERVER = "serverName";

    private final String _cmdClose = "close";
    private final String _cmdConnect = "connect";
    private final String _cmdDisconnect = "disconnect";
    private final String _cmdCancel = "cancel";

    private VetoableChangeSupport vcSupport = new VetoableChangeSupport(this);
    private PropertyChangeSupport pcSupport = new PropertyChangeSupport(this);

    //*****************************************************************************
    // GUI widgets, all labels are editable properties
    //*****************************************************************************
    private JButton closeButton = null;
    private JButton connectButton = null;
    private JButton disconnButton = null;
    private JButton cancelButton = null;

    private JLabel useridLabel = null;
    private JLabel passwdLabel = null;
    private JLabel serverLabel = null;

    // +++ JTextComponent has serialization problem, temp workaround
    private TextField useridField = null;
    private TextField passwdField = null;
    private JComboBox serverField = null;
    //private JTextField      useridField   = null;
    //private JPasswordField  passwdField   = null;
    // +++ temp workaround

    private JFrame frame = null;
    private CMBBasePanel mainPane = null;

    //*****************************************************************************
    // Properties & their initial values
    //*****************************************************************************
    private String userid = ""; // to restore field when PC vetoed
    private String passwd = ""; // to restore field when PC vetoed
    private String serverName = "";
    private String[] serverNames = new String[0];
    //*****************************************************************************
    // Transient data members
    //*****************************************************************************
    private transient Vector listenerVector = null;

    //*****************************************************************************
    // VisualCafe 3.0 support 04/21/04/afic, reference cannot be private
    //*****************************************************************************
    /**
     * To support VisualCafe 3.0 "Add Interaction..."
     */
    transient CMBConnectionRequestEvent connRequestEvent = null;
    /**
     * To support VisualCafe 3.0 "Add Interaction..."
     */
    transient PropertyChangeEvent propertyEvent = null;
    /**
     * To support VisualCafe 3.0 "Add Interaction..."
     */
    transient PropertyChangeEvent vetoableEvent = null;

    //*****************************************************************************
    // Copyright
    //*****************************************************************************
    private static final String ibmid = "5648-D03 and 5648-D04  " + "(C) Copyright IBM Corp. 1994,1999";

    class LogonItemListener implements ItemListener {
        public void itemStateChanged(ItemEvent evt) {
            if (evt.getStateChange() == ItemEvent.SELECTED) {
                String name = (String) evt.getItem();
                try {

                    // VisualCafe 3.0 support 04/21/99/afic
                    propertyEvent = new PropertyChangeEvent(this, PROP_SERVER, serverName, name);
                    vetoableEvent = new PropertyChangeEvent(this, PROP_SERVER, serverName, name);
                    /* jdk 1.2 only
                    vcSupport.fireVetoableChange(propertyEvent);
                    pcSupport.firePropertyChange(vetoableEvent);
                    */
                    // VisualCafe 3.0 support 04/21/99/afic

                    vcSupport.fireVetoableChange(PROP_SERVER, serverName, name);
                    pcSupport.firePropertyChange(PROP_SERVER, serverName, name);
                } catch (PropertyVetoException ve) {
                    serverField.setSelectedItem(serverName); // restore GUI with old values
                }
            }
        }
    }

}