/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Demonstrates the use of CMBConnectionPool bean.  Connects and disconnects
 * from a server several times using connections obtained from the connection
 * pool.
 * <p>
 * In order for the connection pool to reuse connections, the connections in
 * the pool must share a userid.  This type of environment occurs typically
 * when read-only access is required by many users to the content server.
 */
public class TConnectionPool {

    public static void main(String[] args) throws Exception {
        if (args.length < 4) {
            System.out.println(
                "Uses connection pool to connect and disconnect repeatedly. \n"
                    + "Syntax: \n"
                    + "  java TConnectionPool <dstype> <server> <userid> <password> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n");
               System.exit(0);
        }
        
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        
        // Create the pool
        CMBConnectionPool connectionPool = new CMBConnectionPool();
        connectionPool.setDsType(dstype);
        if (server.indexOf("(") > 0) { // connect string follows server name
            // Typical connect parms: ODPORT=3219,ENTITY_TYPE=TEMPLATES
            String connectString = server.substring(server.indexOf("(") + 1);
            server = server.substring(0, server.indexOf("("));
            if (connectString.endsWith(")")) {
                connectString = connectString.substring(0, connectString.length() - 1);
            }
            connectionPool.setConnectString(connectString);
        }
        connectionPool.setServerName(server);

		// get/free connections using the pool
        for (int i = 0; i < 5; i++) {
            CMBConnection connection = connectionPool.getConnection(userid, password);
            System.out.println("Connection obtained.");

            // Note: Connection does not need to be connected.  This
            //       was performed by the connection pool.

            // Here you would perform logic using the connection.
            // Just displaying some schema information in this sample
            CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
            String[] entityNames = schemaManagement.getEntityNames();
            System.out.print("  Entities: ");
            for (int j = 0; j < entityNames.length; j++) {
            	System.out.print(entityNames[j]+" ");
            }
            System.out.println();

            connectionPool.freeConnection(connection);
            System.out.println("Connection freed.");

            // Notes:
            // - Connections should not be disconnected.  This
            //   will be performed by connection pool as needed.
            // - After the connection has been freed, do not
            //   continue to use it.  Also do not use
            //   schema, data management, query service, and other
            //   beens that use the connection bean.
        }
                
        System.exit(0);
    }

}