/*
 * @copyright(disclaimer) 
 * Licensed Materials - Property of IBM
 * IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
 * 
 * US Government Users Restricted Rights                                  
 *  Use, duplication or disclosure restricted by GSA ADP Schedule          
 *  Contract with IBM Corp.     
 * 
 * DISCLAIMER OF WARRANTIES :                                             
 *                                                                        
 * Permission is granted to copy and modify this  Sample code, and to           
 * distribute modified versions provided that both the copyright        
 * notice,- and this permission notice and warranty disclaimer  appear in all copies and modified  
 * versions. 
 * 
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. 
 * IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM     
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN 
 * SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF 
 * NON-INFRINGEMENT AND THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 * PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
 * SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT 
 * OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, 
 * DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION 
 * OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO 
 * EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE 
 * LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, 
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED 
 * AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF 
 * IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGES.                                     
 *                                                                  
 * @endCopyright
 */
import javax.swing.*;

import ResourceMgr;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.*;

/**
 * Client applet
 * Note:  The standalone application is named CMBClient.
 *
 */
public class SampleClientApplet extends JApplet implements
                ClientInterface {

   // Do not remove - needed for copyright protection
   private static String COPYRIGHT = ClientPane.IBM_LONG_COPYRIGHT;

   private JFrame appletFrame;  // used if java_window=yes
   private ClientPane mainPane;
   private boolean javaWindow;

   public static final String DEFAULT_HELP_FILE = "ClientHelp.html";
   private boolean loggedOn=true;

   public SampleClientApplet() {
      super();
   }

   public void init() {
      super.init();
      String parm = getParameter("JAVA_WINDOW");
      javaWindow = (parm != null && parm.equalsIgnoreCase("YES")) ? true : false;

      if (javaWindow) {  // run in a JFrame
         appletFrame = new JFrame(ClientPane.PRODUCT_NAME);
         appletFrame.setBounds(150, 150, 570, 400);
         SwingUtilities.invokeLater(new Runnable()  {
         public void run() {
            mainPane = createMainPane(appletFrame.getContentPane());
            if (!(loggedOn)) {
               System.out.println(" we are not logged on exit");
               return;
            }
            appletFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE );
            appletFrame.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent e) {
                try {
                  killApplet();
                 }catch (ShouldNotOccurException exc) {
                   System.out.println("About to close the applet. Warning: exception was caught while logging off.");
                   if (javaWindow) {
                    appletFrame.dispose();
                   }
                 }
                  // Do not call System.exit as that will kill all browser windows.
               }
               });
               appletFrame.setBounds(150, 150, 570, 400);
               appletFrame.setVisible(true);
            }
         } );
      }

      else {  // run inside browser's window
         mainPane = createMainPane(this.getContentPane());
      }

    }

   private ClientPane createMainPane(Container contentPane) {
      // CMBClientPane cannot be created in applet constructor because
      // applet parameters aren't available until init().  CMBClientPane's
      // constructor request parameters from applet.
      return new ClientPane(contentPane, this);
   }

   /*
   public void paint(Graphics g) {
      if (javaWindow) {
         g.drawString("Applet is running in another window.", 25, 25);
         g.drawString("You may want to minimize this browser window.", 25,30);
      }
   }
   */

   private void killApplet() throws ShouldNotOccurException {
    try {
     if (mainPane!=null) {
       mainPane.logoff();
      }
     }catch (ShouldNotOccurException exc) {
       System.out.println("Warning: exception occured while logging off of EIP database.");
       throw exc;
      }
      if (javaWindow) {
         loggedOn=false;
         appletFrame.dispose();
      }
      // Do not call System.exit as that will kill all browser windows.
   }


   public void exitProgram() {
      try {
       killApplet();
       }catch (ShouldNotOccurException exc) {
          System.out.println("Warning: exception occured while logging off of EIP database.");
       }
       if (javaWindow) {
        loggedOn=false;
        appletFrame.dispose();
       }
     }

   /**
    * Called by browser when user leaves page from which applet
    * was started.  If applet opened a JFrame, it is disposed here.
    * Logoff FederationManager.
    */
   public void destroy() {
      // Warning:  Don't call System.exit since that will kill <B>all</B>
      // browser windows, even if the applet's html file was spawned
      // in a new browser frame.
      try {
        loggedOn=false;
        killApplet();
       }catch (ShouldNotOccurException exc) {
        System.out.println("Warning: exception occured while logging off of EIP database.");
        super.destroy();//return;
       }
      super.destroy();
   }

   /* Fetches a parameter
    * 
    * @param parameter name of param
    * @return value of parameter or null
    */
   public String getParm(String parameter) {
     return this.getParameter(parameter);
   }

   public void displayHelp() { 

      // Two files are needed: the CMBClientHelp.bat file is installed in the EIP
      // install directory (or where the application is started from). The ClientHelp.html 
      // file should be in the DOC subdirectory relative to the parent install directory.  
      
      try{
           // EJMTrace.trace( EJMTrace.INFO, this, "Launching URL: "+getDocumentBase()+"ClientHelp.html");
           getAppletContext().showDocument(new URL (getDocumentBase()+"ClientHelp.html"), "_blank");
      }
      catch (MalformedURLException exc) {
           // EJMTrace.trace(EJMTrace.INFO,this,exc.toString() + ":" + exc.getMessage());
      }
         
   }

   public JFrame getMainFrame() {
      return appletFrame;
   }

}
