/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//*****************************************************************************
//
// Sample class to list all subEntities in a Domino.Doc datastore
// Library (a Domino.Doc Datastore)
// -- contains Rooms
// -- -- contains Cabinets (has a profile with attributes)
// -- -- -- contains Binders (has a profile with attributes)
// -- -- -- -- contains Documents (has a profile with attributes)
//
//*****************************************************************************

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;



 public class TListAttributesDD implements DKConstantDD
 {

   public static String Me = new String("TListAttributesDD");


   public void TListAttributesDD(){}

   public static void main(String argv[])
   {
     System.out.println(" ");
     System.out.println(Me + ": Starting");
     String str;

     TListAttributesDD thisTest = new TListAttributesDD();

     // Input parameters
     String libURL = new String("");   // library server
     String userid = new String("");
     String pw     = new String("");
     String cabinetName = new String("");

     // Indicate this is a Console Application on Windows NT/95 for error
     // message display in dos console code page. If this was a Windows
     // GUI (Graphic User Interface) Application this would not be necessary
     // because DK_CM_SS_WINDOWS is the default.
     DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

     //--------------------------------------------------------------
     // Checking for input parameters
     //--------------------------------------------------------------

     if (argv.length < 3)
     {
       System.out.println(Me + " Usage: " );
       System.out.println("  java " + Me +
       " <libURL> <userid> <pw>" );
       return;
     }
     else
     {
       libURL = argv[0];
       userid = argv[1];
       pw = argv[2];
       System.out.println(Me + ": Parameter1=" + argv[0] );
       System.out.println(Me + ": Parameter2=" + argv[1] );
       System.out.println(Me + ": Parameter3=" + argv[2] );
     }

     DKDatastoreDD dsDD = null;
     try
     {
       dsDD = new DKDatastoreDD();

       // get Connection
       System.out.println("\n" + Me + ": Connecting to datastore");
       dsDD.connect(libURL,userid,pw,"");
       if( dsDD == null )
       {
         System.out.println("\n" + Me + ": returned connection object is null");
       }

       // Cabinets are the only thing with interesting attributes.
       // (If you try to list entity attributes for rooms, there is nothing in the collection.
       dkCollection cabinets = dsDD.listSearchableEntities();
       dkIterator itCabinets = cabinets.createIterator();
       while( itCabinets.more() )
       { // for each cabinet, list it's attributes.
         dkEntityDef aCabinet = (dkEntityDef)itCabinets.next();
         cabinetName = aCabinet.getName();
         // list Document Profiles w/o sub-attributes
         System.out.println("\n" + Me + ": calling listAttrs for" + cabinetName );
         DKSequentialCollection coll = (DKSequentialCollection) aCabinet.listAttrs();
         if( coll.getSortFunction() != null ) // did the builder set a sort function for me?
         { // then sort
           coll.sort( true );                 // ascending is "true"; decending is "false"
         }
         int nbrElements = coll.cardinality();
         System.out.println(Me + ":print " + nbrElements + " attributes for this datastore.");
         if( nbrElements > 0 )
         {
           dkIterator it = coll.createIterator();
           while( it.more() )
           {
             DKAttrDefDD anObject = (DKAttrDefDD)it.next();
             if( anObject != null )
             {
                switch (anObject.getDDType())
                {
                  case (short) DK_DD_PROFILE:
                 { // print name, fields, and keywords
                 System.out.println("Profile name: " + anObject.getName() );
                 dkCollection fields = ((DKAttrProfileDefDD)anObject).getFields();

                 if((fields != null) &&( fields.cardinality() > 0 ))
                 {
                   dkIterator itFields = fields.createIterator();
                   while( itFields.more() )
                   { // print name, keywords
                     DKAttrDefDD aField = (DKAttrDefDD)itFields.next();
                     System.out.println("  FieldName: " + aField.getName() );
                     dkCollection keywords = ((DKAttrFieldDefDD)aField).getKeywords();
                     if( keywords != null )
                     {
                       if( keywords.cardinality() > 0 )
                       { // print name
                         dkIterator itKeywords = keywords.createIterator();
                         while( itKeywords.more() )
                         {
                           DKAttrDefDD aKeyword = (DKAttrDefDD)itKeywords.next();
                           System.out.println("    Keyword: " + aKeyword.getName() );
                         }
                       }
                     }
                   }
                 }
                 break;
               }
               case (short) DK_DD_ATTR:
               {
                 System.out.println("Domino.Doc Attribute name: " + anObject.getEntityName() + ":" + anObject.getName() );
                 break;
               }
               default:
               {
                 System.out.println(Me + anObject.toString() );
               }
                }
             }
           }
         }
       }
       // disconect
       System.out.println(Me + ": Disconnecting from datastore");
       dsDD.disconnect();
       dsDD.destroy();
     }
     catch (DKException exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": DKException message " + exc.getMessage());
       exc.printStackTrace();
     }
     catch(Exception exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": Exception message " + exc.getMessage());
       exc.printStackTrace();
     }

     System.out.println(Me + ": Finished");
   }

  public void printList( dkCollection coll)
  throws DKException
  {
    if( coll == null )
    {
      System.out.println(Me + ":printList Input Collection object is null.");
      return;
    }
    int nbrElements = coll.cardinality();
    Class  collClass = coll.getClass();
    String collClassName = collClass.getName();
    System.out.println(Me + ":printList for a " + collClassName + ", nbrElements=" + nbrElements);
    int i = 1;
    if( nbrElements > 0 )
    {
      dkIterator it = coll.createIterator();
      while( it.more() )
      {
        Object anObject = it.next();
        if( anObject != null )
        {
          Class objClass = anObject.getClass();
          String objClassName = objClass.getName();
          System.out.println("..toString[" + i++ + "]");
          String objString = anObject.toString();
          System.out.println(objString);
        }
      }
    }
    return;
  }
 }

