/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/* -------------------------------------------------------------------------
 * Class Name  : TQStringDLTS.java
 * Description : Test program to run ParametricQuery (PQ) or TextQuery (TQ), 
 *               in a loop for DKDatastoreDL/DKDatastoreTS.  This also logs
 *               Memory usage information to stdout and logfile (MemTest.log)
 *               
 *               This program uses TMemUsage.c (TMemUsage.dll), 
 *               TMemUsage.java to get Virtual Memory size on NT.  To use 
 *               on AIX comment line that has getVMSize() 
 *
 *               java TQStringDLTS client DL 1000 LSIKE edms1 edms1
 *
 * Author      : Chandra S Potula
 * Create Date : 10/15/2000
 * 
 * Change History
 * -------------------------------------------------------------------------
 */

import com.ibm.mm.sdk.common.*;

import java.io.*;
import java.text.*;
import java.util.*;

public class TQStringDLTS implements DKConstant {

public TQStringDLTS(String aPackageName, String aDBType) {
	_startTime   = System.currentTimeMillis();
	_dateFormat  = new SimpleDateFormat("HH:mm:ss"); 
	_dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
	_databaseDL  = !aDBType.equalsIgnoreCase("TS");

	try {
		if(     aPackageName.equalsIgnoreCase("cs")) {
			if(_databaseDL) {
				_datastore = new com.ibm.mm.sdk.cs.DKDatastoreDL();
			}
			else {
				_datastore = new com.ibm.mm.sdk.cs.DKDatastoreTS();
			}
			_packageName = "cs";
		}
		else if(aPackageName.equalsIgnoreCase("client")) {
			if(_databaseDL) {
				_datastore = new com.ibm.mm.sdk.client.DKDatastoreDL();
			}
			else {
				_datastore = new com.ibm.mm.sdk.client.DKDatastoreTS();
			}
			_packageName = "client";
		}
		else {
			if(_databaseDL) {
				_datastore = new com.ibm.mm.sdk.server.DKDatastoreDL();
			}
			else {
				_datastore = new com.ibm.mm.sdk.server.DKDatastoreTS();
			}
			_packageName = "server";
		}

		logmsg("loaded : " + _datastore.getClass().getName());
		_logFile       = new PrintWriter(new FileWriter("MemTest.log", false), true);
	}
	catch(Exception e) {
		e.printStackTrace();
	}
}

// -------------------------------------------------------------------------
public static void logmsgdkResultSetCursor(dkResultSetCursor aRSCursor) {
	if(aRSCursor == null) {
		//logmsg("*** error *** got null cursor");
	}
	else {
		try {
			//logmsg("RSCCardinality : " + aRSCursor.cardinality());

			DKDDO lDKDDO = null;
			int nPosition = 0;
			while(aRSCursor.isValid()) {
				lDKDDO = aRSCursor.fetchNext();

				if(aRSCursor.isValid()) {
					nPosition = aRSCursor.getPosition();
				}

				if(lDKDDO != null) {
					//lDKDDO.retrieve();
					int lDDOCount = lDKDDO.dataCount();
					//logmsg("    " + nPosition + ". DDOPid:" + lDKDDO.getPidObject().pidString() + "  Count:" + lDDOCount);
				}
			}
			aRSCursor.destroy();
		}
		catch (Exception exc) {
			exc.printStackTrace();
		}
	}
}

// -------------------------------------------------------------------------
public static void logmsg(String aMsg) {
	System.out.println(aMsg);
}

// -------------------------------------------------------------------------
public long executeQString(String aQString, String strInd, short nQLType) {
	long              lTimeTaken = 0;
	dkResultSetCursor lCursor    = null;

	try {
		long   lQStart;
		long   lQEnd;
		String lstrClock = "";
		String lstrTime;
		String lstrInd;

		//logmsg(aQString);
		lQStart    = System.currentTimeMillis();
		lCursor    = _datastore.execute(aQString, nQLType, null);
		lQEnd      = System.currentTimeMillis();
		lTimeTaken = lQEnd - lQStart;
		lstrTime   = "      " + lTimeTaken;
		logmsgdkResultSetCursor(lCursor);

		lstrClock  = _dateFormat.format(new Date(lQEnd - _startTime));
		lstrInd    = "     " + strInd + ".";

		lstrTime   = lstrTime.substring(lstrTime.length() - 6);
		lstrInd    = lstrInd.substring(lstrInd.length() - 6);


		logmsg(lstrInd + lstrClock + " Took:" + lstrTime + "ms  " + getMemUsage());
		//logmsg("---------------------------------------------------------------------------");
		_logFile.println(lstrInd + lstrClock + " " + lQStart + " Took:" + lstrTime + "ms " + getMemUsage());
		_logFile.flush();

		lCursor = null;

		// This will notify RMI server that the the cursor is dereferenced.
		System.gc();
		System.runFinalization();
		System.gc();
	}
	catch(Exception e) {
		lTimeTaken = -1;
		e.printStackTrace();
	}

	return lTimeTaken;
}

// -------------------------------------------------------------------------
public void startQueryLoop(String nLoopTimes, 
		String aDBName, String aUserID, String aPassword) {
	try {

		try {
			// following lines are to trace Server activity if running 
			// Client/Server.
			// To do this the server shoud define a counter and increment
			// every time execute is called, also a boolean _printMemUsage
			// static variable and if it is set to true print Server
			// stats every time execute is called.
			com.ibm.mm.sdk.server.DKDatastoreFed._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreDL ._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreTS ._printMemUsage = false;
			com.ibm.mm.sdk.server.DKDatastoreDES._printMemUsage = false;
		}
		catch(Exception e) {
		}

		boolean fParametric  = _databaseDL;
		boolean fText        = !_databaseDL;
		boolean fCombinedOr  = false;
		boolean fCombinedAnd = false;

		long    nRunTimes   = toLong(nLoopTimes);
		if(nRunTimes <= 0) {
			nRunTimes  = 1;
		}

		// String queries are hard coded this uses schema mapping to the 
		// server LSIKE (DL) and TMHSIA (TS).  You may need to change these 
		// if you want to run this program against some other server.
		//
		// SchemaMapping to LSIKE, TMHSIA is as below...
		// Entity_DL1
		//     Entity_DL1;LSIKE;DLSAMPLE
		//         Entity_DL1.FedAttr_Name     -> LSIKE.DLSAMPLE.DLSEARCH_Author
		//         Entity_DL1.FedAttr_Publisher-> LSIKE.DLSAMPLE.DLSEARCH_Publisher
		//         Entity_DL1.FedAttr_Title    -> LSIKE.DLSAMPLE.DLSEARCH_Title
		// Fed_TMHSIA
		//     Fed_TMHSIA;TMHSIA;TMINDEX
		//         Fed_TMHSIA-Entity_DL1       -> TMHSIA-LSIKE.TMINDEX-DLSAMPLE
		String lstrPCond   = "DLSEARCH_Author <> 'Chandra'";
		String lstrTCond   = "'IBM' OR 'Tivoli'";
		String lstrContent = ", OPTION=(CONTENT=yes);";
		String lstrPSQuery = "SEARCH=(INDEX_CLASS=DLSAMPLE, MAX_RESULTS=4, COND=(" + lstrPCond + "))";
		String lstrTSQuery = "SEARCH=(COND=(" + lstrTCond + "));OPTION=(SEARCH_INDEX=(TMINDEX);MAX_RESULTS=4)" + lstrContent;
		String lstrCOQuery = lstrPSQuery + " OR  " + lstrTSQuery;
		String lstrCAQuery = lstrPSQuery + " AND " + lstrTSQuery;

		_datastore.connect(aDBName, aUserID, aPassword, "");

		TThread lTh;
		for(long nInd=1; nInd <= nRunTimes;) {
			// use following lines to do NormalRun (which is single threaded 
			// in case of DL and TS
			//if(fParametric ) executeQString(lstrPSQuery, ("" + nInd++), DK_PARAMETRIC_QL_TYPE);
			//if(fText       ) executeQString(lstrTSQuery, ("" + nInd++), DK_TEXT_QL_TYPE      );
			//if(fCombinedOr ) executeQString(lstrCOQuery, ("" + nInd++), nQueryType           );
			//if(fCombinedAnd) executeQString(lstrCAQuery, ("" + nInd++), nQueryType           );


			// use following lines to run Multithreaded way using TThread subclass 
			// (to simulate the way Fed runs query).  These lines will run the query 
			// in a seperate thread (instead of main thread) and will wait until the 
			// thread finishes to grab the results.
			if(fParametric ) executeThread(("" + nInd++), this, lstrPSQuery, DK_PARAMETRIC_QL_TYPE);
			if(fText       ) executeThread(("" + nInd++), this, lstrTSQuery, DK_TEXT_QL_TYPE      );
			//if(fCombinedOr ) executeThread(("" + nInd++), this, lstrCOQuery, nQueryType           );
			//if(fCombinedAnd) executeThread(("" + nInd++), this, lstrCAQuery, nQueryType           );

			if(false) {
				break;
			}
		}

		// after executing specified number times give a chance to the user
		// to note down any memory information or do any observation/analysis
		logmsg("Ran execute for : " + nRunTimes + "\ntype QUIT to disconnect and quit");
		BufferedReader in = new BufferedReader( new InputStreamReader( System.in ) );
		String strIn = "";
		while(!strIn.equalsIgnoreCase("QUIT")) {
			strIn = in.readLine();
		}

		_datastore.disconnect();
	}
	catch(Exception exp) {
		exp.printStackTrace();
	}
}

// -------------------------------------------------------------------------
private void executeThread(String aName, TQStringDLTS aTQStringDLTS, 
		String aQuery, short nQueryType) {
	TThread lTh = new TThread(aName, aTQStringDLTS, aQuery, nQueryType);
	lTh.setDaemon(true);
	lTh.start();

	while(!lTh.done()) {
		try {
			Thread.currentThread().sleep(10);
		}
		catch (InterruptedException e) {
		}
	}
}

// -------------------------------------------------------------------------
public static void main(String argv[]) {
	if(argv.length < 3) {
		logmsg("Invalid number of arguments specified...");
		printUsage();
	}
	else {
		String  ldbName   = (argv.length > 3) ? argv[3] : "LSIKE";
		String  luserId   = (argv.length > 4) ? argv[4] : "edms1";
		String  lPassword = (argv.length > 5) ? argv[5] : "edms1";

		TQStringDLTS lQStringDLTS = new TQStringDLTS(argv[0], argv[1]);
		lQStringDLTS.startQueryLoop(argv[2], ldbName, luserId, lPassword);
	}
}

// -------------------------------------------------------------------------
public static void printUsage() {
	logmsg("usage:");
	logmsg("    java TQStringDLTS packageName      DBType ExecuteTimes DBName UID Password");
	logmsg("    java TQStringDLTS packageName      DBType ExecuteTimes");
	logmsg("    java TQStringDLTS client/server/cs DL/TS  ExecuteTimes\n");
	logmsg("    e.g. java TQStringDLTS server DL 10000 LSIKE edms1 edms1");
	logmsg("         java TQStringDLTS server TS 10000");
}

// -------------------------------------------------------------------------
	private  dkDatastore  _datastore   = null;
	private  PrintWriter  _logFile     = null;
	private  DateFormat   _dateFormat  = null;
	private  long         _startTime   = 0;
	private  String       _packageName = "";
	private  boolean      _databaseDL  = true;

// -------------------------------------------------------------------------
public static final String getMemUsage() {
	Runtime lRuntime = Runtime.getRuntime();
	long    lFree    = lRuntime.freeMemory();
	long    lTotal   = lRuntime.totalMemory();
	float   nUsed    = (float) ((int)(10000 * ((float)lFree / lTotal) + 0.005) / 100.0);
	String  strPCent = (nUsed > 9) ? ("" + nUsed) : (" " + nUsed);

	// make sure that lFree, lTotal are of same length
	String  strFree  = ("        " + lFree );
	String  strTotal = ("        " + lTotal);
	strFree  = strFree .substring(strFree .length() - 8);
	strTotal = strTotal.substring(strTotal.length() - 8);

	return("MemUsage:" +  strPCent + "% Free (" + strFree + "/" + strTotal + 
			")  VMSize:" + (TMemUsage.getVMSize() / 1024) + "K");
}

// -------------------------------------------------------------------------
public static final long toLong(String aNum) {
	long rc = 0;

	if(aNum != null) {
		try {
			rc   = (Long.valueOf(aNum.trim())).longValue();
		}
		catch(Exception e) {
		}
	}
	return rc;
}
}

// -------------------------------------------------------------------------

class TThread extends Thread implements java.io.Serializable {
	private       TQStringDLTS _TQStringDLTS = null;
	public static int          _instCount    = 0;
	private       byte[]       _holdMem      = null;
	private       boolean      _isDone       = false;

	private       String       _query;
	private       short        _queryType;

private TThread(String aName) {
	super(aName);
	_instCount++;
	_holdMem = new byte[512];
}
public TThread(String aName, TQStringDLTS aTQStringDLTS, String aQuery, short nQueryType) {
	this(aName);
	_TQStringDLTS = aTQStringDLTS;
	_query        = aQuery;
	_queryType    = nQueryType;
}
public void run() {
	_TQStringDLTS.executeQString(_query, getName(), _queryType);
	_isDone = true;
	_instCount--;
}

public boolean done() {
	return _isDone;
}
}
// -------------------------------------------------------------------------

