/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TADLsmCombine.java
 * Description    : test add, delete, list schema-mappings for DLs, IP, and V4
 *                : Create FedDemo entity and map it to DLDEMO on Mufasa, Talisman2, and
 *                : V4DEMO on V4 and DOCUMENT on IP.
 *
 */

import java.sql.*;
import java.lang.*;
import java.util.*;
import com.ibm.mm.sdk.common.dkSchemaMapping;
import com.ibm.mm.sdk.common.dkAttrMapping;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

class TADLsmCombine implements DKConstantFed, DKConstantIP, DKConstantV4, DKConstantDL
{
    public static void main(String argv[]) 
    { 
        // checking input parameters
        String libSrv = "",    // library server
               userid = "",    // userid 
               pw     = "",    // pw
               cmd    = "";    // command: add, del, or list
        
        //-------------------------------------------------------------- 
        // Checking for input parameters
        //-------------------------------------------------------------- 
        if (argv.length < 4) {
           System.out.println("Usage Syntax: ");
           System.out.println("\t java TADLsmCombine <feddb> <userid> <pw> <add/del/list>" );
           return;
        }
        else {
           libSrv = argv[0];
           userid = argv[1];
           pw     = argv[2];
           cmd    = argv[3];
        }   

        try {
            // create federated datastore
            System.out.println(" *** connecting to datastore : " + libSrv);
            dkDatastore ds = new DKDatastoreFed();
            ds.connect(libSrv, userid, pw, "");
            System.out.println(" *** datastore connected *** "); 

            // get the Def
            dkDatastoreDef  defFed = ds.datastoreDef();

            // create a federated entity : FedDemo
            DKEntityDefFed entityObj = new DKEntityDefFed(ds);        
            entityObj.setName("FedCombine");
            entityObj.setDescription("FedCombine to map to DLDEMO & DLSAMPLE");
    
            // create a federated attributes :
            dkAttrDef fUserId = new DKAttrDefFed();
                      fUserId.setEntityName("FedCombine");
                      fUserId.setName("FedUserId");
                      fUserId.setType(DK_CM_VSTRING);
                      fUserId.setId((short)95);
                      fUserId.setSize(50);
                      fUserId.setScale(1);
                      fUserId.setPrecision(50);
                      fUserId.setNullable(true);
                      fUserId.setStringType(DK_ATTR_ALPHA);
    
            dkAttrDef fTitle = new DKAttrDefFed();
                      fTitle.setEntityName("FedCombine");
                      fTitle.setName("FedTitle");
                      fTitle.setType(DK_CM_VSTRING);
                      fTitle.setId((short)99);
                      fTitle.setSize(100);
                      fTitle.setScale(1);
                      fTitle.setPrecision(100);
                      fTitle.setNullable(true);
                      fTitle.setStringType(DK_ATTR_ALPHA);
    
            dkAttrDef fDate = new DKAttrDefFed();
                      fDate.setEntityName("FedCombine");
                      fDate.setName("FedDate");
                      fDate.setType(DK_CM_VSTRING);
                      fDate.setId((short)100);
                      fDate.setSize(100);
                      fDate.setScale(1);
                      fDate.setPrecision(100);
                      fDate.setNullable(true);
                      fDate.setStringType(DK_ATTR_ALPHA);

            dkAttrDef fAuthor = new DKAttrDefFed();
                      fAuthor.setEntityName("FedCombine");
                      fAuthor.setName("FedAuthor");
                      fAuthor.setType(DK_CM_VSTRING);
                      fAuthor.setId((short)95);
                      fAuthor.setSize(50);
                      fAuthor.setScale(1);
                      fAuthor.setPrecision(50);
                      fAuthor.setNullable(true);
                      fAuthor.setStringType(DK_ATTR_ALPHA);
    
            dkAttrDef fSubject = new DKAttrDefFed();
                      fSubject.setEntityName("FedCombine");
                      fSubject.setName("FedSubject");
                      fSubject.setType(DK_CM_VSTRING);
                      fSubject.setId((short)100);
                      fSubject.setSize(100);
                      fSubject.setScale(1);
                      fSubject.setPrecision(100);
                      fSubject.setNullable(true);
                      fSubject.setStringType(DK_ATTR_ALPHA);

            // associate data sources to this entity:
            // get the Fed DS Def and Admin
            DKDatastoreAdminFed adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();

            // list back-end servers
            System.out.println(" *** list all back-end servers : ");
            String servers[] = adm.listServerNames();
            int len = servers.length;
            for (int i = 0; i < len; i++) {
                System.out.println("server [" + i + "] = " + servers[i]);
            }  
              
            // retrieve the definition of the first server
            DKServerMapDefFed serverdat;
            String serverName = null;
            for (int i = 0; i < len; i ++) {
                serverdat = adm.getServer(servers[i]);
                serverName = serverdat.getName().trim();
                System.out.println(" DKServerMapDefFed = " + serverName );
                System.out.println("       name : " + serverName);
                System.out.println("       type : " + serverdat.getServerType());
                System.out.println(" Connect Str: " + serverdat.getConnectString().trim());
                System.out.println("       desc : " + serverdat.getDescription());
                System.out.println("    refTime : " + serverdat.getRefreshTime());
                
            }     

            // Now, interpret the command: add, del, or list
            if (cmd.equalsIgnoreCase("add")) {
                entityObj.add();        // adding an entity to the federated database
                entityObj.add(fUserId);
                entityObj.add(fTitle);
                entityObj.add(fDate);
                entityObj.add(fAuthor);
                entityObj.add(fSubject);
                System.out.println(" *** federated entity and attributes created ");
            }

            // list federated entity:
            System.out.println(" *** federated entities :");
            DKSequentialCollection pCol = (DKSequentialCollection) ds.listEntities();
            DKSequentialCollection pCol2;
            dkIterator pIter = pCol.createIterator();
            dkIterator pIter2 = null;
            dkEntityDef enDef = null;
            dkAttrDef attrDef = null;
            String entName;
            int i = 0, j = 0;
            while (pIter.more()) {
                i++;
                enDef = (dkEntityDef) pIter.next();
                entName = enDef.getName();
                System.out.println(" Entity Def name [" + i + "] - " + entName);
                System.out.println("  list attributes for " + entName + " entity");
                pCol2 = (DKSequentialCollection) ds.listEntityAttrs(entName);
                pIter2 = pCol2.createIterator();
                j = 0;
                while (pIter2.more()) {
                   j++;
                   attrDef = (dkAttrDef) pIter2.next();
                   System.out.println("  Attribute name [" + j + "] - " + attrDef.getName());
                   System.out.println("    datastoreType : " + attrDef.datastoreType());
                   System.out.println("    attributeOf   : " + attrDef.getEntityName());
                   System.out.println("    type          : " + attrDef.getType());
                   System.out.println("    size          : " + attrDef.getSize());
                   System.out.println("    id            : " + attrDef.getId());
                   System.out.println("    nullable      : " + attrDef.isNullable());
                   System.out.println("    precision     : " + attrDef.getPrecision());
                   System.out.println("    scale         : " + attrDef.getScale());
                   System.out.println("    stringType    : " + attrDef.getStringType());
                }  
                System.out.println("  " + j + " attr listed for " + entName + " entity name");
            }
            System.out.println(i + " entity def listed");

            // create native attributes in-memory : DL index_class DLDEMO at mufasa
            dkAttrDef nmUserId = new DKAttrDefDL();
                      nmUserId.setEntityName("DLDEMO");
                      nmUserId.setName("DLUSERID");
                      nmUserId.setType(DK_CM_VSTRING);
                      nmUserId.setId((short)95);
                      nmUserId.setSize(50);
                      nmUserId.setScale(1);
                      nmUserId.setPrecision(50);
                      nmUserId.setNullable(true);
                      nmUserId.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmTitle = new DKAttrDefDL();
                      nmTitle.setEntityName("DLDEMO");
                      nmTitle.setName("DLTITLE");
                      nmTitle.setType(DK_CM_VSTRING);
                      nmTitle.setId((short)99);
                      nmTitle.setSize(100);
                      nmTitle.setScale(1);
                      nmTitle.setPrecision(100);
                      nmTitle.setNullable(true);
                      nmTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmDate = new DKAttrDefDL();
                      nmDate.setEntityName("DLDEMO");
                      nmDate.setName("DLDATE");
                      nmDate.setType(DK_CM_VSTRING);
                      nmDate.setId((short)100);
                      nmDate.setSize(100);
                      nmDate.setScale(1);
                      nmDate.setPrecision(100);
                      nmDate.setNullable(true);
                      nmDate.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSMUFASA, DLDEMO ");

            // create native attributes in-memory : DL index_class DLDEMO at talisman2
            dkAttrDef ncUserId = new DKAttrDefDL();
                      ncUserId.setEntityName("DLDEMO");
                      ncUserId.setName("DLUSERID");
                      ncUserId.setType(DK_CM_VSTRING);
                      ncUserId.setId((short)95);
                      ncUserId.setSize(50);
                      ncUserId.setScale(1);
                      ncUserId.setPrecision(50);
                      ncUserId.setNullable(true);
                      ncUserId.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncTitle = new DKAttrDefDL();
                      ncTitle.setEntityName("DLDEMO");
                      ncTitle.setName("DLTITLE");
                      ncTitle.setType(DK_CM_VSTRING);
                      ncTitle.setId((short)99);
                      ncTitle.setSize(100);
                      ncTitle.setScale(1);
                      ncTitle.setPrecision(100);
                      ncTitle.setNullable(true);
                      ncTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncDate = new DKAttrDefDL();
                      ncDate.setEntityName("DLDEMO");
                      ncDate.setName("DLDATE");
                      ncDate.setType(DK_CM_VSTRING);
                      ncDate.setId((short)100);
                      ncDate.setSize(100);
                      ncDate.setScale(1);
                      ncDate.setPrecision(100);
                      ncDate.setNullable(true);
                      ncDate.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSCHIUN, DLDEMO ");

            // create native attributes in-memory : V4 index_class V4DEMO at stealth3
            dkAttrDef nvUserId = new DKAttrDefV4();
                      nvUserId.setEntityName("V4DEMO");
                      nvUserId.setName("V4USERID");
                      nvUserId.setType(DK_CM_VSTRING);
                      nvUserId.setId((short)95);
                      nvUserId.setSize(50);
                      nvUserId.setScale(1);
                      nvUserId.setPrecision(50);
                      nvUserId.setNullable(true);
                      nvUserId.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nvTitle = new DKAttrDefV4();
                      nvTitle.setEntityName("V4DEMO");
                      nvTitle.setName("V4TITLE");
                      nvTitle.setType(DK_CM_VSTRING);
                      nvTitle.setId((short)99);
                      nvTitle.setSize(100);
                      nvTitle.setScale(1);
                      nvTitle.setPrecision(100);
                      nvTitle.setNullable(true);
                      nvTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nvDate = new DKAttrDefV4();
                      nvDate.setEntityName("V4DEMO");
                      nvDate.setName("V4DATE");
                      nvDate.setType(DK_CM_VSTRING);
                      nvDate.setId((short)100);
                      nvDate.setSize(100);
                      nvDate.setScale(1);
                      nvDate.setPrecision(100);
                      nvDate.setNullable(true);
                      nvDate.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at STEALTH, V4DEMO ");

            // create native attributes in-memory : IP index_class DOCUMENT at mvsd9
            dkAttrDef npUserId = new DKAttrDefIP();
                      npUserId.setEntityName("DOCUMENT");
                      npUserId.setName("FolderId");
                      npUserId.setType(DK_CM_VSTRING);
                      npUserId.setId((short)95);
                      npUserId.setSize(50);
                      npUserId.setScale(1);
                      npUserId.setPrecision(50);
                      npUserId.setNullable(true);
                      npUserId.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef npTitle = new DKAttrDefIP();
                      npTitle.setEntityName("DOCUMENT");
                      npTitle.setName("Description");
                      npTitle.setType(DK_CM_VSTRING);
                      npTitle.setId((short)99);
                      npTitle.setSize(100);
                      npTitle.setScale(1);
                      npTitle.setPrecision(100);
                      npTitle.setNullable(true);
                      npTitle.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef npDate = new DKAttrDefIP();
                      npDate.setEntityName("DOCUMENT");   
                      npDate.setName("FormCode");
                      npDate.setType(DK_CM_VSTRING);
                      npDate.setId((short)100);
                      npDate.setSize(100);
                      npDate.setScale(1);
                      npDate.setPrecision(100);
                      npDate.setNullable(true);
                      npDate.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at FAFCICS61, IPDEMO ");



            // create native attributes in-memory : DL index_class DLSAMPLE at mufasa
            dkAttrDef nmAuthor1 = new DKAttrDefDL();
                      nmAuthor1.setEntityName("DLSAMPLE");
                      nmAuthor1.setName("DLSEARCH_Author");
                      nmAuthor1.setType(DK_CM_VSTRING);
                      nmAuthor1.setId((short)95);
                      nmAuthor1.setSize(50);
                      nmAuthor1.setScale(1);
                      nmAuthor1.setPrecision(50);
                      nmAuthor1.setNullable(true);
                      nmAuthor1.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmTitle1 = new DKAttrDefDL();
                      nmTitle1.setEntityName("DLSAMPLE");
                      nmTitle1.setName("DLSEARCH_Title");
                      nmTitle1.setType(DK_CM_VSTRING);
                      nmTitle1.setId((short)99);
                      nmTitle1.setSize(100);
                      nmTitle1.setScale(1);
                      nmTitle1.setPrecision(100);
                      nmTitle1.setNullable(true);
                      nmTitle1.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef nmSubject1 = new DKAttrDefDL();
                      nmSubject1.setEntityName("DLSAMPLE");
                      nmSubject1.setName("DLSEARCH_Subject");
                      nmSubject1.setType(DK_CM_VSTRING);
                      nmSubject1.setId((short)100);
                      nmSubject1.setSize(100);
                      nmSubject1.setScale(1);
                      nmSubject1.setPrecision(100);
                      nmSubject1.setNullable(true);
                      nmSubject1.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSMUFASA, DLSAMPLE ");

            // create native attributes in-memory : DL index_class DLSAMPLE at talisman2
            dkAttrDef ncAuthor1 = new DKAttrDefDL();
                      ncAuthor1.setEntityName("DLSAMPLE");
                      ncAuthor1.setName("DLSEARCH_Author");
                      ncAuthor1.setType(DK_CM_VSTRING);
                      ncAuthor1.setId((short)95);
                      ncAuthor1.setSize(50);
                      ncAuthor1.setScale(1);
                      ncAuthor1.setPrecision(50);
                      ncAuthor1.setNullable(true);
                      ncAuthor1.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncTitle1 = new DKAttrDefDL();
                      ncTitle1.setEntityName("DLSAMPLE");
                      ncTitle1.setName("DLSEARCH_Title");
                      ncTitle1.setType(DK_CM_VSTRING);
                      ncTitle1.setId((short)99);
                      ncTitle1.setSize(100);
                      ncTitle1.setScale(1);
                      ncTitle1.setPrecision(100);
                      ncTitle1.setNullable(true);
                      ncTitle1.setStringType(DK_ATTR_ALPHA);
     
            dkAttrDef ncSubject1 = new DKAttrDefDL();
                      ncSubject1.setEntityName("DLSAMPLE");
                      ncSubject1.setName("DLSEARCH_Subject");
                      ncSubject1.setType(DK_CM_VSTRING);
                      ncSubject1.setId((short)100);
                      ncSubject1.setSize(100);
                      ncSubject1.setScale(1);
                      ncSubject1.setPrecision(100);
                      ncSubject1.setNullable(true);
                      ncSubject1.setStringType(DK_ATTR_ALPHA);
     
            System.out.println(" *** native attributes defined at LSCHIUN, DLSAMPLE ");





            //  create schema mapping : entity and attr mappings 
            //   from FedDemo to DLDEMO in LSMUFASA
            dkAttrMapping mapUserId = new DKAttrMappingFed();
                          mapUserId.addMappedAttr(fUserId);
                          mapUserId.addMapToAttr(nmUserId);
     
            dkAttrMapping mapTitle = new DKAttrMappingFed();
                          mapTitle.addMappedAttr(fTitle);
                          mapTitle.addMapToAttr(nmTitle);
     
            dkAttrMapping mapDate = new DKAttrMappingFed();
                          mapDate.addMappedAttr(fDate);
                          mapDate.addMapToAttr(nmDate);
     
            System.out.println(" ** attribute mapping to LSMUFASA defined ");
     
            dkSchemaMapping msm = new DKSchemaMappingFed(ds);
                            msm.setServerName("LSMUFASA");
                            msm.setServerType(DK_DL_DSTYPE);
                            msm.setEntityName("DLDEMO");
                            msm.setMappedEntityName("FedCombine");
     
            // add attribute mapping to this schema
            msm.addAttrMapping(mapUserId);
            msm.addAttrMapping(mapTitle);
            msm.addAttrMapping(mapDate);

            try {
                if (cmd.equalsIgnoreCase("add")) {
                    msm.add();
                    System.out.println(" ** schema mapping for LSMUFASA added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    msm.del();
                    System.out.println(" ** schema mapping for LSMUFASA deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }

            //  create schema mapping : entity and attr mappings 
            //   from FedSample to DLDEMO in LSCHIUN
            dkAttrMapping cmapUserId = new DKAttrMappingFed();
                          cmapUserId.addMappedAttr(fUserId);
                          cmapUserId.addMapToAttr(ncUserId);
     
            dkAttrMapping cmapTitle = new DKAttrMappingFed();
                          cmapTitle.addMappedAttr(fTitle);
                          cmapTitle.addMapToAttr(ncTitle);
     
            dkAttrMapping cmapDate = new DKAttrMappingFed();
                          cmapDate.addMappedAttr(fDate);
                          cmapDate.addMapToAttr(ncDate);
     
            System.out.println(" ** attribute mapping to LSCHIUN defined ");
     
            dkSchemaMapping csm = new DKSchemaMappingFed(ds);
                            csm.setServerName("LSCHIUN");
                            csm.setServerType(DK_DL_DSTYPE);
                            csm.setEntityName("DLDEMO");
                            csm.setMappedEntityName("FedCombine");
     
            // add attribute mapping to this schema
            csm.addAttrMapping(cmapUserId);
            csm.addAttrMapping(cmapTitle);
            csm.addAttrMapping(cmapDate);
    
            try {
                if (cmd.equalsIgnoreCase("add")) {
                    csm.add();
                    System.out.println(" ** schema mapping for LSCHIUN added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    csm.del();
                    System.out.println(" ** schema mapping for LSCHIUN deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }
 
            //  create schema mapping : entity and attr mappings 
            //   from FedDemo to V4DEMO in STEALTH3
            dkAttrMapping vmapUserId = new DKAttrMappingFed();
                          vmapUserId.addMappedAttr(fUserId);
                          vmapUserId.addMapToAttr(nvUserId);
     
            dkAttrMapping vmapTitle = new DKAttrMappingFed();
                          vmapTitle.addMappedAttr(fTitle);
                          vmapTitle.addMapToAttr(nvTitle);
     
            dkAttrMapping vmapDate = new DKAttrMappingFed();
                          vmapDate.addMappedAttr(fDate);
                          vmapDate.addMapToAttr(nvDate);
     
            System.out.println(" ** attribute mapping to STEALTH3 defined ");
     
            dkSchemaMapping vsm = new DKSchemaMappingFed(ds);
                            vsm.setServerName("STEALTH3");
                            vsm.setServerType(DK_V4_DSTYPE);
                            vsm.setEntityName("V4DEMO");
                            vsm.setMappedEntityName("FedCombine");
     
            // add attribute mapping to this schema
            vsm.addAttrMapping(vmapUserId);
            vsm.addAttrMapping(vmapTitle);
            vsm.addAttrMapping(vmapDate);

            try {
                if (cmd.equalsIgnoreCase("add")) {
                    vsm.add();
                    System.out.println(" ** schema mapping for STEALTH3 added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    vsm.del();
                    System.out.println(" ** schema mapping for STEALTH3 deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }
 
            //  create schema mapping : entity and attr mappings 
            //   from FedDemo to DOCUMENT in MVSD9
            dkAttrMapping pmapUserId = new DKAttrMappingFed();
                          pmapUserId.addMappedAttr(fUserId);
                          pmapUserId.addMapToAttr(npUserId);
     
            dkAttrMapping pmapTitle = new DKAttrMappingFed();
                          pmapTitle.addMappedAttr(fTitle);
                          pmapTitle.addMapToAttr(npTitle);
     
            dkAttrMapping pmapDate = new DKAttrMappingFed();
                          pmapDate.addMappedAttr(fDate);
                          pmapDate.addMapToAttr(npDate);
     
            System.out.println(" ** attribute mapping to MVSD9 defined ");
     
            dkSchemaMapping psm = new DKSchemaMappingFed(ds);
                            psm.setServerName("FAFCICS61");
                            psm.setServerType(DK_IP_DSTYPE);
                            psm.setEntityName("DOCUMENT");
                            psm.setMappedEntityName("FedCombine");
     
            // add attribute mapping to this schema
            psm.addAttrMapping(pmapUserId);
            psm.addAttrMapping(pmapTitle);
            psm.addAttrMapping(pmapDate);

            try {
                if (cmd.equalsIgnoreCase("add")) {
                    psm.add();
                    System.out.println(" ** schema mapping for MVSD9 added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    psm.del();
                    System.out.println(" ** schema mapping for MVSD9 deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }

            //  create schema mapping : entity and attr mappings 
            //   from FedSample to DLSAMPLE in LSMUFASA
            dkAttrMapping mapAuthor1 = new DKAttrMappingFed();
                          mapAuthor1.addMappedAttr(fAuthor);
                          mapAuthor1.addMapToAttr(nmAuthor1);
     
            dkAttrMapping mapTitle1 = new DKAttrMappingFed();
                          mapTitle1.addMappedAttr(fTitle);
                          mapTitle1.addMapToAttr(nmTitle1);
     
            dkAttrMapping mapSubject1 = new DKAttrMappingFed();
                          mapSubject1.addMappedAttr(fSubject);
                          mapSubject1.addMapToAttr(nmSubject1);
     
            System.out.println(" ** attribute mapping to LSMUFASA defined ");
     
            dkSchemaMapping msm1 = new DKSchemaMappingFed(ds);
                            msm1.setServerName("LSMUFASA");
                            msm1.setServerType(DK_DSTYPE_DL);
                            msm1.setEntityName("DLSAMPLE");
                            msm1.setMappedEntityName("FedCombine");
     
                            // add attribute mapping to this schema
                            msm1.addAttrMapping(mapAuthor1);
                            msm1.addAttrMapping(mapTitle1);
                            msm1.addAttrMapping(mapSubject1);
            
            try {
                if (cmd.equalsIgnoreCase("add")) {
                    msm1.add();
                    System.out.println(" ** schema mapping for LSMUFASA added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    msm1.del();
                    System.out.println(" ** schema mapping for LSMUFASA deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }
                 
            //  create schema mapping : entity and attr mappings 
            //   from FedSample to DLSAMPLE in LSCHIUN
            dkAttrMapping cmapAuthor1 = new DKAttrMappingFed();
                          cmapAuthor1.addMappedAttr(fAuthor);
                          cmapAuthor1.addMapToAttr(ncAuthor1);
     
            dkAttrMapping cmapTitle1 = new DKAttrMappingFed();
                          cmapTitle1.addMappedAttr(fTitle);
                          cmapTitle1.addMapToAttr(ncTitle1);
     
            dkAttrMapping cmapSubject1 = new DKAttrMappingFed();
                          cmapSubject1.addMappedAttr(fSubject);
                          cmapSubject1.addMapToAttr(ncSubject1);
     
            System.out.println(" ** attribute mapping to LSCHIUN defined ");
     
            dkSchemaMapping csm1 = new DKSchemaMappingFed(ds);
                            csm1.setServerName("LSCHIUN");
                            csm1.setServerType(DK_DSTYPE_DL);
                            csm1.setEntityName("DLSAMPLE");
                            csm1.setMappedEntityName("FedCombine");
     
            // add attribute mapping to this schema
            csm1.addAttrMapping(cmapAuthor1);
            csm1.addAttrMapping(cmapTitle1);
            csm1.addAttrMapping(cmapSubject1);

            try {
                if (cmd.equalsIgnoreCase("add")) {
                    csm1.add();
                    System.out.println(" ** schema mapping for LSCHIUN added ");
                 }
                 else if (cmd.equalsIgnoreCase("del")) {
                    csm1.del();
                    System.out.println(" ** schema mapping for LSCHIUN deleted ");
                 }
            }
            catch(Exception e) { e.printStackTrace(); }







            if (cmd.equalsIgnoreCase("del")) {
                entityObj.del(fDate);
                entityObj.del(fTitle);
                entityObj.del(fUserId);
                entityObj.del(fAuthor);
                entityObj.del(fSubject);
                entityObj.del();        // deleting an entity from the federated database
                System.out.println(" *** federated entity and attributes deleted ");
            } 

            // list schema mappings in this entity:
            System.out.println(" list schema mappings in this entity : " + entityObj.getName());
            String[] smnames = entityObj.listSchemaMappingNames();
            if (smnames != null) {
                len = smnames.length;
                for (i = 0; i < len; i++) {
                    System.out.println(" SM name : " + smnames[i]);
                }
            } else 
                System.out.println(" no sm defined in this entity ");

            // retrieve all schema mapping objects for this entity
            dkCollection coll = entityObj.listSchemaMappings();
            dkIterator iter = null;
            if (coll != null) {
                iter = coll.createIterator();
                System.out.println(" ");
                System.out.println("-- list AttrMapping -- ");
                dkAttrMapping _am;
                dkAttrDef     _ad;
                dkSchemaMapping _sm;
                while (iter.more()) {
                    _sm = (dkSchemaMapping) iter.next();
                    System.out.println("\n *** Schema Mapping Name : " + _sm.getName());
                    dkCollection amCol = _sm.listAttrMappings();
                    dkIterator amIter = amCol.createIterator();
                    while (amIter.more()) {
                        _am = (dkAttrMapping) amIter.next();
                        System.out.println(" ");
                        System.out.println("  ++ AttrMapping Name : " +_am.getName());
                        System.out.println("    -- To Function : " + _am.getToNative());
                        System.out.println("    -- From Function : " + _am.getFromNative());
                        System.out.println(" -- list Mapped Attr : ");
                        int num = _am.numberOfMappedAttr();
                        System.out.println("\t Mapped attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMappedAttr((short)(i+1));
                            System.out.println("  ++ Mapped Attr Name   : " +_ad.getName());
                            System.out.println("    -- Mapped Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size             : " +_ad.getSize());
                            System.out.println("    -- Precision        : " +_ad.getPrecision());
                            System.out.println("    -- Scale            : " +_ad.getScale());
                            System.out.println("    -- Max              : " +_ad.getMax());
                            System.out.println("    -- Min              : " +_ad.getMin());
                            System.out.println("    -- Nullable         : " +_ad.isNullable());
                        }
                        System.out.println(" -- list MapTo Attr -- ");
                        num = _am.numberOfMapToAttr();
                        System.out.println("\t MapTo attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMapToAttr((short)(i+1));
                            System.out.println("  ++ MapTo Attr Name   : " +_ad.getName());
                            System.out.println("    -- MapTo Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size            : " +_ad.getSize());
                            System.out.println("    -- Precision       : " +_ad.getPrecision());
                            System.out.println("    -- Scale           : " +_ad.getScale());
                            System.out.println("    -- Max             : " +_ad.getMax());
                            System.out.println("    -- Min             : " +_ad.getMin());
                            System.out.println("    -- Searchable      : " +_ad.isQueryable());
                        }  
                    }
                }  
            } else System.out.println(" schema mapping is null ");

            ds.disconnect();
            System.out.println(" ** disconnects... ");
        } 
        catch (java.lang.Exception ex) 
        { 
            ex.printStackTrace(); 
        } 
    } 
}





