/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    This sample pulls together all concepts learned so far, using the IBM COntent Management
    Data Modeling Concepts in a business model example.  The Insurance Model is defined and
    explained in SItemTypeCreationICM.
    
    Minimum Required Reading & Samples Run
        SConnectDisconnectICM                   SFolderICM
        SAttributeDefinitionCreationICM         SLinksICM
        SAttributeGroupDefinitionCreationICM    SResourceMgrDefCreationICM
        SReferenceAttrDefCreationICM            SResourceMgrSetDefaultICM
        SItemTypeCreationICM                    SSMSCollectionDefCreationICM
        SItemCreationICM                        SSMSCollectionDefSetDefaultICM
        SItemRetrievalICM                       SResourceItemCreaitonICM
        SItemUpdateICM                          SResourceItemRetrievalICM
        SItemDeletionICM

This sample shows how to simply retrieve each created in the creation sample.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SInsuranceModelItemRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Putting everything together into a real-world example, the Sample 
 *                    Insurance Model.  Retrieves the Insurance Model Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Putting everything together into a real-world example
 *                    Retrieving Insurance Model's Auto Claims
 *                    Retrieving Insurance Model's Adjustment Reports
 *                    Retrieving Insurance Model's Police Reports
 *                    Retrieving Insurance Model's Insurance Policies
 *                    Retrieving Insurance Model's Auto Photos
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SInsuranceModelItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemRetrievalICM.java
 *                    SInsuranceModelItemCreationICM.java
 *                    SInsuranceModelICM_AccidenReport_0120200130.tif
 *                    SInsuranceModelICM_AccidenReport_0506200110.tif
 *                    SInsuranceModelICM_AccidenReport_729200173.tif
 *                    SInsuranceModelICM_Adjustment_4-852369.tif
 *                    SInsuranceModelICM_Adjustment_5-654123.tif
 *                    SInsuranceModelICM_Adjustment_6-987654.tif
 *                    SInsuranceModelICM_Claim_4-852369.tif
 *                    SInsuranceModelICM_Claim_5-654123.tif
 *                    SInsuranceModelICM_Claim_6-987654.tif
 *                    SInsuranceModelICM_Photo_1ABC089.jpg
 *                    SInsuranceModelICM_Photo_4IBM134-1.tif
 *                    SInsuranceModelICM_Photo_4IBM134-2.tif
 *                    SInsuranceModelICM_Photo_6usa676-1.tif
 *                    SInsuranceModelICM_Photo_6usa676-2.tif
 *                    SInsuranceModelICM_Policy_10285265410.tif
 *                    SInsuranceModelICM_Policy_10315975308.tif
 *                    SInsuranceModelICM_Policy_10452136412.tif
 ************************************************************************************************/
public class SInsuranceModelItemRetrievalICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SInsuranceModelItemRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SInsuranceModelItemRetrievalICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Uses Function provided in the creation sample.
            System.out.println("Creating Insurance Model Items...");

                java.util.Vector ddoVector = SInsuranceModelItemCreationICM.createInsuranceModelItems(dsICM);  // Uses Function provided in the creation sample.
                
                DKLobICM autoClaim1      = (DKLobICM) ddoVector.elementAt(0);
                DKLobICM autoClaim2      = (DKLobICM) ddoVector.elementAt(1);
                DKLobICM autoClaim3      = (DKLobICM) ddoVector.elementAt(2);
                DKImageICM photo1a       = (DKImageICM) ddoVector.elementAt(3);
                DKImageICM photo1b       = (DKImageICM) ddoVector.elementAt(4);
                DKImageICM photo2        = (DKImageICM) ddoVector.elementAt(5);
                DKImageICM photo3a       = (DKImageICM) ddoVector.elementAt(6);
                DKImageICM photo3b       = (DKImageICM) ddoVector.elementAt(7);
                DKLobICM adjusterReport1 = (DKLobICM) ddoVector.elementAt(8);
                DKLobICM adjusterReport2 = (DKLobICM) ddoVector.elementAt(9);
                DKLobICM adjusterReport3 = (DKLobICM) ddoVector.elementAt(10);
                DKLobICM policeReport1   = (DKLobICM) ddoVector.elementAt(11);
                DKLobICM policeReport2   = (DKLobICM) ddoVector.elementAt(12);
                DKLobICM policeReport3   = (DKLobICM) ddoVector.elementAt(13);
                DKLobICM policeReport4   = (DKLobICM) ddoVector.elementAt(14);
                DKLobICM policy1         = (DKLobICM) ddoVector.elementAt(15);
                DKLobICM policy2         = (DKLobICM) ddoVector.elementAt(16);
                DKLobICM policy3         = (DKLobICM) ddoVector.elementAt(17);

            System.out.println("Created Insurance Model Items.");

            //-------------------------------------------------------------
            // Recreate Blank DDOs using Pid Stirngs
            //-------------------------------------------------------------
            System.out.println("Recreating Blank DDOs uisng Pid Strings...");

                autoClaim1      = (DKLobICM) dsICM.createDDO(autoClaim1.getPidObject().pidString());
                autoClaim2      = (DKLobICM) dsICM.createDDO(autoClaim2.getPidObject().pidString());
                autoClaim3      = (DKLobICM) dsICM.createDDO(autoClaim3.getPidObject().pidString());
                photo1a         = (DKImageICM) dsICM.createDDO(photo1a.getPidObject().pidString());
                photo1b         = (DKImageICM) dsICM.createDDO(photo1b.getPidObject().pidString());
                photo2          = (DKImageICM) dsICM.createDDO(photo2.getPidObject().pidString());
                photo3a         = (DKImageICM) dsICM.createDDO(photo3a.getPidObject().pidString());
                photo3b         = (DKImageICM) dsICM.createDDO(photo3b.getPidObject().pidString());
                adjusterReport1 = (DKLobICM) dsICM.createDDO(adjusterReport1.getPidObject().pidString());
                adjusterReport2 = (DKLobICM) dsICM.createDDO(adjusterReport2.getPidObject().pidString());
                adjusterReport3 = (DKLobICM) dsICM.createDDO(adjusterReport3.getPidObject().pidString());
                policeReport1   = (DKLobICM) dsICM.createDDO(policeReport1.getPidObject().pidString());
                policeReport2   = (DKLobICM) dsICM.createDDO(policeReport2.getPidObject().pidString());
                policeReport3   = (DKLobICM) dsICM.createDDO(policeReport3.getPidObject().pidString());
                policeReport4   = (DKLobICM) dsICM.createDDO(policeReport4.getPidObject().pidString());
                policy1         = (DKLobICM) dsICM.createDDO(policy1.getPidObject().pidString());
                policy2         = (DKLobICM) dsICM.createDDO(policy2.getPidObject().pidString());
                policy3         = (DKLobICM) dsICM.createDDO(policy3.getPidObject().pidString());

            System.out.println("Recreated Blank DDOs uisng Pid Strings.");

            //-------------------------------------------------------------
            // Retrieve One of Each
            //-------------------------------------------------------------
            // IMPORTANT NOTE: In order to greatly improve performance of 
            //                 retrieval, specify specific retrieval options
            //                 other than "Item Tree" to only retrieve what
            //                 is desired.  Performance will improve immensely.
            //                 The following will check for all possibilities
            //                 in a worst-case scenerio.
            System.out.println("Retrieving One of Each Type...");

                autoClaim2.retrieve(DKConstant.DK_CM_CONTENT_YES + DKConstant.DK_CM_CONTENT_ITEMTREE);
                photo2.retrieve(DKConstant.DK_CM_CONTENT_YES + DKConstant.DK_CM_CONTENT_ITEMTREE);
                adjusterReport2.retrieve(DKConstant.DK_CM_CONTENT_YES + DKConstant.DK_CM_CONTENT_ITEMTREE);
                policeReport2.retrieve(DKConstant.DK_CM_CONTENT_YES + DKConstant.DK_CM_CONTENT_ITEMTREE);
                policy2.retrieve(DKConstant.DK_CM_CONTENT_YES + DKConstant.DK_CM_CONTENT_ITEMTREE);

            System.out.println("Retrieved One of Each Type.");

            //-------------------------------------------------------------
            // Print Each
            //-------------------------------------------------------------
            // Using the SItemRetrievalICM's printDDO funciton, this will
            // demonstrate what each DDO looks like now.
            System.out.println("Printing Items...");

                SItemRetrievalICM.printDDO(autoClaim2);
                SItemRetrievalICM.printDDO(photo2);
                SItemRetrievalICM.printDDO(adjusterReport2);
                SItemRetrievalICM.printDDO(policeReport2);
                SItemRetrievalICM.printDDO(policy2);

            System.out.println("Printed Items.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SInsuranceModelItemRetrievalICM
              
