/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

When Updating Reference Attributes, use it as an Attribute Group.  Updating Reference 
Attribute Definitions work the same as Attribute Groups, but no attributes may be added 
or removed.  Updating it is simply for the other options, such as name, description,
and delete rule.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Reference Attribute Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Reference Attribute Definitions in Memory
 *                    Saving / Updating Reference Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SReferenceAttrDefUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here must already exist.  If not, please
 *                    run the SReferenceAttrDefCreationICM Sample first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SReferenceAttrDefRetrievalICM.java
 ************************************************************************************************/
public class SReferenceAttrDefUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SReferenceAttrDefUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SReferenceAttrDefUpdateICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Retrieve a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Treat the Reference Attribute (Group) as an actual
            // Attribute Group, since it was created as such.
            //
            // Retrieving the "S_myRef" Reference Attribute (Group) Definition.
            System.out.println("Retrieving Reference Atttribute 'S_myRef'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_myRef"); // Retrieve the definition.

            System.out.println("Retrieved Reference Attribute 'S_myRef'.");

            //--------------------------------------------------------------
            // Modify an Reference Attribute in Memory
            //--------------------------------------------------------------
            // Updating Reference Attribute Definitions work the same as 
            // Attribute Groups, but no attributes may be added or removed.
            // Updating it is simply for the other options, such as name, 
            // description, and delete rule.
            //
            // This example will modify the description of the S_myRef Reference
            // Attribute.
            System.out.println("Modifying Reference Attribute Definition 'S_myRef' in Memory...");
            
                attrGroupDef.setDescription("My Modified Reference Attribute");  // Change the description
                
            System.out.println("Modified Reference Attribute Definition 'S_myRef' in Memory.");
            
            //--------------------------------------------------------------
            // Save / Update Reference Attribute Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's update() method can be called
            // as well as the Datastore Definition's update(attrGroupDef) method.
            //
            // This example will use the Attribute Group's update() method.
            System.out.println("Saving / Updating Reference Attribute Definition 'S_myRef' in Datastore...");

                attrGroupDef.update();  // Save / Update Changes in Datastore Definition

            System.out.println("Updated Reference Attribute Definition 'S_myRef' in Datastore.");

            //-------------------------------------------------------------
            // Print Updated Reference Attribute (Group) Definition
            //-------------------------------------------------------------
            System.out.println("Printing the Updated Reference Attribute Definition of 'S_myRef'...");
                SReferenceAttrDefRetrievalICM.printReferenceAttrDef(attrGroupDef); // Use wrapper funciton from the Retrieval Sample.
            System.out.println("Printed the Updated Reference Attribute Definition of 'S_myRef'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
   
}//end class SReferenceAttrDefUpdateICM
              
