/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Resource Item Storage
    Resource Item's content is stored in a Resource Manager.  It is possible to specify
    which Resource Manager & which SMS Collection within the Resource Manager that each
    Resource Item is to be stored.  Multiple Resource Managers and multiple SMS 
    Collections may be available for use.
    
    Default Resource Manager & SMS Collection
        The Item Type Definition contains information that specifies the default
        Resource Manager and default SMS Collection that Items of that Item Type
        are to be stored if not otherwise specified.  Please refer to the corresponding
        wrapper function at the bottom of this sample, and to the Item Type Creation
        sample, SItemTypeCreationICM for more information and examples of how to 
        set and modify these properties.
    
    For more information on Resource Manager Definitions or SMS Collections, please refer
    to the corresponding section below.

SMS Collections
    SMS Collections are a logical grouping of objects.  Within each Resource Manager
    defined in the system, an SMS Collection may be defined as a grouping or collection
    that Resource Items may belong to.  One advantage of grouping & seperating Resource
    Items in a particular Resource Manger server is for administrative purposes to help
    manage and diagnose needs.  

    RM Pre-defined SMS Collections:
        The Non-Media colleciton listed below is created automatically with a Resource Manager
        Definition.
    
         Collection      Constant                                  Description
    -------------------  ----------------------------------------  ----------------------------
    Non-Media (Default)  DKConstantICM.RM_SMS_COLLECTION_NONMEDIA  Default Collection.
                                                                   May be used for non-media.
    ...........................................................................................
           Media         DKConstantICM.RM_SMS_COLLECTION_MEDIA     Used for Media Types.
    ...........................................................................................
           Video         DKConstantICM.RM_SMS_COLLECTION_VIDEO     Used with Video Charger.
    -------------------------------------------------------------------------------------------

    The ICM Content Manager v8.2 install on Windows will create a Resource Manager object
    named "RMDB" by default and one SMS Collection, Non-Media by default.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefCreationICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: SMS Collection Definition, Explanation, Setup / Configuration.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Setting up an SMS Collection
 *                    Pre-defined SMS Collections
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SSMSCollectionDefCreationICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <RM User Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, password, or
 *                          RM username are not specified.  All others are required.
 *
 *                    ex: java SSMSCollectionDefCreationICM RMDB
 *                    ex: java SSMSCollectionDefCreationICM MY_RM icmadmin icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SResourceMgrDefRetrievalICM
 *                    SSMSCollectionDefRetrievalICM
 ************************************************************************************************/
public class SSMSCollectionDefCreationICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        String rmUserName                  = "icmadmin";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("   java SSMSCollectionDefCreationICM");
            System.out.println("                         <Existing RM User-Defined Reference Name>");
            System.out.println("                         <RM User Name>");
            System.out.println("                         <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.");
            System.out.println("");
            System.out.println("  NOTE: Defaults will be used if the database, username, password, or");
            System.out.println("        RM username are not specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SSMSCollectionDefCreationICM RMDB");
            System.out.println("  ex: java SSMSCollectionDefCreationICM MY_RM icmadmin icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  RM Reference Name Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 5)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];
        if (argv.length > 1) rmUserName                 = argv[1];

        // Store optional commandline arguments.
        if (argv.length > 2)  database = argv[2];
        if (argv.length > 3)  userName = argv[3];
        if (argv.length > 4)  password = argv[4];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("=============================================");
        System.out.println("IBM Information Integrator for Content   v"+ver);
        System.out.println("Sample Program:  SSMSCollectionDefCreationICM");
        System.out.println("---------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("        RM UserName:  "+rmUserName);       
        System.out.println("=============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtain the RM Code, Needed for SMS Collection Definition
            //-------------------------------------------------------------
            System.out.println("Obtaining RM Code of RM '"+rmUserDefinedReferenceName+"'...");

                if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw new Exception("Resource Manager Definition Named '"+rmUserDefinedReferenceName+"' Not Defined.");

                short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.

            System.out.println("Obtained RM Code of RM '"+rmUserDefinedReferenceName+"'.");

            //-------------------------------------------------------------
            // Setup New SMS Collection Definition (if not already)
            //-------------------------------------------------------------
            // Setup a new SMS Collection Definition if not already defined.
            //
            // This sample will setup the system's pre-determined collection names.
            // User-Defined Collections may be substituted.  Any unique name can be
            // specified.
            System.out.println("Setting up system pre-determined SMS Collection Definitions (if not already)...");

                // Only if not already defined, define the specified SMS Collection.  Please
                // refer to SSMSCollectionDefRetrievalICM for more information.
                if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode, rmUserName); // Uses wrapper at bottom of sample.

                if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode, rmUserName); // Uses wrapper at bottom of sample.

                if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode)==false)
                    setupSMSCollectionForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode, rmUserName); // Uses wrapper at bottom of sample.
                
            System.out.println("Set up system pre-determined SMS Collection Definitions (if not already).");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setupSMSCollectionForRM
    //******************************************************************
   /**
    * Setup the SMS Collection for the specified Resource Manger.
    * @param dsICM       Connected DKDatastoreICM object.
    * @param newSMSCollectionUserDefinedReferenceName  A new User-Defined Reference name to refer to it as in the RM.
    * @param rmCode      Resource Manager ID Code.
    * @param rmUserName  UserName to log onto the Resource Manager.
    * @return Returns the SMS Collection's ID Code.
    **/
    public static short setupSMSCollectionForRM(DKDatastoreICM dsICM, String newSMSCollectionUserDefinedReferenceName, short rmCode, String rmUserName) throws DKException, Exception{
        System.out.println("Setting up SMS Collection with new user defined reference name '"+newSMSCollectionUserDefinedReferenceName+"' for RM with ID Code '"+rmCode+"'...");
        
        // If the SMS Collection is already setup, throw error.
        if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, newSMSCollectionUserDefinedReferenceName,rmCode))
            throw new Exception("An SMS Collection with Reference Name '"+newSMSCollectionUserDefinedReferenceName+"' for Resource Manager with ID Code '"+rmCode+"' already exists.");

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Setup New SMS Collection Definition Object
        DKSMSCollectionDefICM smsCollDef = new DKSMSCollectionDefICM(); // Create new SMS Collection Definition.
        smsCollDef.setName(newSMSCollectionUserDefinedReferenceName);   // Set a new User Defined Reference Name to refer to it later as.
        smsCollDef.setRMCode((short)rmCode);                            // Set the RM Code for the Resource Manager that it belongs to.
        smsCollDef.setUserid(rmUserName);                               // Set the Resource Manager's User Name to use.

        rmCfg.addSMSCollection(smsCollDef);          // Add the SMS Collection to the Resource Manager Configuration.

        short collCode = smsCollDef.getSMSCollectionCode();  // Get the SMS Collection's ID Code
        System.out.println("Added SMS Collection.  ID Code: '"+collCode+"'.");

        // Double Checking...
        DKSMSCollectionDefICM retrSMS = rmCfg.retrieveSMSCollection(newSMSCollectionUserDefinedReferenceName, rmCode); // Retrieve the SMS Collection.
        short retrCollCode = retrSMS.getSMSCollectionCode(); // Get the retrieve collections ID Code.
        if(collCode != retrCollCode)// if they aren't the same, there is a problem.
            throw new Exception("Retrieved SMS Collection Definition's ID Code ('"+retrCollCode+"') does not match the one just added ('"+collCode+"').");

        System.out.println("Setup SMS Collection with for RM.");
        return(collCode);
    }//end setupSMSCollectionForRM
   
}//end class SSMSCollectionDefCreationICM
              
