// IBM Confidential OCO Source Material
// 5648-C83, 5648-C84 (C) COPYRIGHT International Business Machines Corp. 1997,1998,1999
// The souce code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright office.

import java.io.*;
import org.omg.Security.*;
import org.omg.SecurityLevel2.Credentials;
import com.ibm.IExtendedSecurity.*;

/**

ServerSideAuthenticator is a utility class that uses CORBA security APIs to perform
a programmatic login. It uses the CORBA "SecurityCurrent" to obtain a
CORBA login_helper and invokes request_login on that object. This method
sets up the userid and password on the thread. It also optionally
forces authentication to take place. If the authentication is
successfuly, any further method invocation will be invoked under that
authenticated user identity.

 */
public class ServerSideAuthenticator extends LoginHelper {

    public ServerSideAuthenticator() throws IllegalStateException {
        super();
    }

    /*
    The method login()
     * sets the authentication data on the thread of execution
    by calling the login_helper.request_login
     * forces authentication to take place if force_authn is set to tre
     */

    public org.omg.SecurityLevel2.Credentials login(String userid, String password,
    boolean force_authn)
    throws org.omg.SecurityLevel2.LoginFailed,
    com.ibm.IExtendedSecurity.RealmNotRegistered,
    com.ibm.IExtendedSecurity.UnknownMapping,
    com.ibm.IExtendedSecurity.MechanismTypeNotRegistered,
    com.ibm.IExtendedSecurity.InvalidAdditionalCriteria
    {

        Credentials creds = super.login(userid, password);
        Credentials mapCreds = creds;

        if (force_authn) {

            // getting the mapped credentials forces authentication to take place
            //if (creds instanceof com.ibm.ISecurityLocalObjectBasicAuthImpl.CredentialsImpl) {
            if (creds instanceof com.ibm.IExtendedSecurity.Credentials) {
                mapCreds = ((com.ibm.IExtendedSecurity.Credentials)
                creds).get_mapped_credentials(null, "", null);
            }

        }
        return mapCreds;
    }


    /**

    The authenticate method performs authentication using the specified
    userid and password. The security context is not necessarily affected by the
    authentication.
     */

    public org.omg.SecurityLevel2.Credentials authenticate(String userid, String password)
    throws org.omg.Security.InvalidCredentialType,
    org.omg.SecurityLevel2.LoginFailed,
    org.omg.SecurityLevel2.InvalidCredential,
    com.ibm.IExtendedSecurity.RealmNotRegistered,
    com.ibm.IExtendedSecurity.UnknownMapping,
    com.ibm.IExtendedSecurity.MechanismTypeNotRegistered,
    com.ibm.IExtendedSecurity.InvalidAdditionalCriteria

    {

        Credentials invocCreds = getInvocationCredentials();
        Credentials authCreds = login(userid, password, true);
        setInvocationCredentials(invocCreds);
        return authCreds;
    }

}
