/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.Properties;
import java.util.Locale;

import com.ibm.mm.viewer.*;
import com.ibm.mm.viewer.annotation.*;

import com.ibm.mm.beans.CMBPrivilege;

/**
 * A basic applet to show rapid viewer applet development using the java viewer toolkit.
 *
 * Please read the <CMBROOT>/Samples/java/viewer/readme.html for instructions on how to setup and run
 * the samples.
 */
public class TViewerApplet extends JApplet
                          implements ItemListener
{
    private Properties currEngineProperties = null;
    private Properties currConfigProperties = null;
    private CMBStreamingDocServices currStreamingDocServices;
    private CMBAnnotationServices currAnnoServices;
    private CMBGenericDocViewer myGenDocViewer;
    private CMBDocument currDocument;
    private int paramNumDocs;
    private String [] docPaths;
    private InputStream annis = null;
    private JMenuBar mainMenuBar = new JMenuBar();
    private boolean showingThumbnails = true;
    private boolean openingDoc = false;

    public void init()
    {
	try {
                UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) { }
        // This line prevents the "Swing: checked access to system event queue" message seen in some browsers.
	getRootPane().putClientProperty("defeatSystemEventQueueCheck", Boolean.TRUE);
	getContentPane().setLayout(new BorderLayout());
        // Create engine properties
        currEngineProperties = new Properties();
        currEngineProperties.put("ENGINES","2");
        currEngineProperties.put("ENGINE1_CLASSNAME", "com.ibm.mm.viewer.mstech.CMBMSTechDocumentEngine");
        currEngineProperties.put("ENGINE2_CLASSNAME","com.ibm.mm.viewer.CMBJavaDocumentEngine");
        currEngineProperties.put("DOCTYPE_AUTODETECT", "true");
        // End create engine properties

        // Get the config file and create configproperties
        URL configURL = null;
        URLConnection configURLConn = null;
        InputStream configInputStream = null;
	currConfigProperties = new Properties();
        // Get the annotation file and create annotation stream
	URL annFileURL = null;
	URLConnection annFileURLConn = null;
        try {
	    configURL = new URL(getCodeBase(), "TViewerDefaultConfiguration.properties");
	}catch (MalformedURLException e) {
	    System.out.println("JApplet1.start() - Creating docURL: " + e);
        }
        try {
            configURLConn = configURL.openConnection();
       	    configInputStream = configURLConn.getInputStream();
            currConfigProperties.load( configInputStream );
        }catch (IOException e) {
            System.out.println("JApplet1.start() - IOException opening docURLConn: " + e);
         }
         // End read config file

	    /*
	    // Get the annotation file and create annotation stream
	    try {
	        annFileURL = new URL(getCodeBase(), "12310001.T_L");
	    }catch (MalformedURLException e) {
	        System.out.println("JApplet1.start() - Creating annFileURL: " + e);
	    }
	    try {
	        annFileURLConn = configURL.openConnection();
    	    annis = configURLConn.getInputStream();
    	    System.out.println("AnnoStream is " + annis);
	    } catch (IOException e) {
	        System.out.println("JApplet1.start() - IOException opening annFileURLConn: " + e);
	    }
		if (annis == null)
		{
		        annis = new ByteArrayInputStream(new byte[0]);
		}
        // End read annotation file
		*/

                // Create and add genDocViewer
		myGenDocViewer = new CMBGenericDocViewer(getStreamingDocServices(), getAnnotationServices(), currConfigProperties);
		getContentPane().add(myGenDocViewer, BorderLayout.CENTER);
		myGenDocViewer.setBackground(Color.white);
		// End create and add genDocViewer

		//{{INIT_CONTROLS
		//getContentPane().setLayout(new BorderLayout(0,0));
		setSize(0,0);
		//}}
	} // End Init
	//{{DECLARE_CONTROLS
	//}}

	public void start()
	{
	    setupMenus();
	    return;
	} // End start

	public void setupMenus()
	{
		getContentPane().add(mainMenuBar, BorderLayout.NORTH);
		JMenu OpenMenu = new JMenu("Open");
		mainMenuBar.add(OpenMenu);
		JCheckBoxMenuItem checkMenuItem = new JCheckBoxMenuItem("Use delayInputStream");
		checkMenuItem.addItemListener(this);
		OpenMenu.add(checkMenuItem);
		checkMenuItem.setSelected(true);
		OpenMenu.addSeparator();
		JMenuItem mFile1 = OpenMenu.add(new appletActions("View document"));

		JMenu thumbMenu = new JMenu("Thumbnails");
		mainMenuBar.add(thumbMenu);

		JMenu posSubmenu = new JMenu("Set Position");
		thumbMenu.add(posSubmenu);
		posSubmenu.add(new appletActions("North"));
		posSubmenu.add(new appletActions("South"));
		posSubmenu.add(new appletActions("East"));
		posSubmenu.add(new appletActions("West"));
		thumbMenu.addSeparator();
	}

	public void itemStateChanged(ItemEvent iEvent)
	{
	    JMenuItem posSubMenu = (JMenuItem)mainMenuBar.getMenu(0).getMenuComponent(0);
	}

//*  Private class to handle actions
    private class appletActions extends AbstractAction
    {
        public appletActions(String actCommand)
        {
            super(actCommand);
        }
        public void actionPerformed(java.awt.event.ActionEvent actEvent)
        {
            String actionCommand = null;
            Object comp = actEvent.getSource();
            URL docURL = null;
            URLConnection docURLConn = null;
            InputStream docInputStream = null;
            String docName = "doc";

            if( comp instanceof JMenuItem ){
            //to work with current version of jdk 1.2.2 used with EIP
                actionCommand = ((JMenuItem)comp).getActionCommand();
            }

            if(actionCommand == null)   return;
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            // Get annotation set from annoservices
            CMBAnnotationSet annSet=null, annSet1=null;
            try {
              annSet = myGenDocViewer.loadAnnotations(annis, "application/vnd.ibm.modcap",1390, 1, 0);
            }
            catch (Exception e) {
	      System.out.println("JApplet.start() - Exception loading annotation set" + e);
	      e.printStackTrace();
            }
            // End get annotation sets

            if(actionCommand.equalsIgnoreCase("View document"))
            {
                openingDoc = true;
	            try
	            {
                        String filename = JOptionPane.showInputDialog("Enter a file name from the codebase to view:" );
	                //Get the URLInputstream from the jpeg file
	                docURL = new URL(getCodeBase(), filename);
	            }catch (MalformedURLException e)
	            {
	                System.out.println("JApplet1.start() - Creating docURL: " + e);
	            }
	            docName = "Document";
            }
            if(openingDoc == true)
            {
	        try {
	            docURLConn = docURL.openConnection();
	            docInputStream = docURLConn.getInputStream();
	            String strMime = docURLConn.getContentType();
	            // load document from URLInputstream
	            try {
                   	currDocument = myGenDocViewer.loadDocument(docInputStream, 1, strMime, strMime, null, null);

	            }catch (Exception e){
	                System.out.println("JApplet1.start() - Exception loading document: " + e);
	            }
	            if(currDocument != null) {
	                myGenDocViewer.showDocument(currDocument, annSet, docName);
                    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	            }
	            else {
	                System.out.println("Japplet.start - currDocument is null");
	            }
	        } catch (IOException e) {
	            System.out.println("JApplet1.start() - IOException opening docURLConn: " + e);
	        }
	        openingDoc = false;
	        }

            if(actionCommand.equalsIgnoreCase("West"))
            {
                myGenDocViewer.setThumbnailsPosition("West");
                myGenDocViewer.revalidate();
            }
            if(actionCommand.equalsIgnoreCase("South"))
            {
                myGenDocViewer.setThumbnailsPosition("South");
                myGenDocViewer.revalidate();
            }
            if(actionCommand.equalsIgnoreCase("East"))
            {
                myGenDocViewer.setThumbnailsPosition("East");
                myGenDocViewer.revalidate();
            }
            if(actionCommand.equalsIgnoreCase("North"))
            {
                myGenDocViewer.setThumbnailsPosition("North");
                myGenDocViewer.revalidate();
            }
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }

    /** Returns the instance of CMBStreamingDocServices.  Creates one if needed. */
    private CMBStreamingDocServices getStreamingDocServices()
    {
        if (currStreamingDocServices == null)
            currStreamingDocServices = new CMBStreamingDocServices(new StreamingDocServicesCallbacks(),currEngineProperties);
        return currStreamingDocServices;
    }

    /** This class handles callbacks from CMBStreamingDocServices. */
    private class StreamingDocServicesCallbacks extends CMBStreamingDocServicesCallbacks implements Serializable
    {
        StreamingDocServicesCallbacks()
        {

        }

        /** Retrieve a background form. */
        // For this particular implementation, formName will always be a URL
        public InputStream getForm(CMBDocument document, String formName)
        {
            try {
                InputStream formStream;
                URL formURL = new URL(getCodeBase(), formName);
                try {
                    URLConnection urlConn = formURL.openConnection();
                if (urlConn == null)
                    formStream = new ByteArrayInputStream(new byte[0]);
                else
                    formStream = urlConn.getInputStream();
                return formStream;
                }catch (IOException e) {
                return null;
            }

            } catch (MalformedURLException e) {
                return null;
                }
        }

        /** Retrieve a document part. */
        public InputStream getPart(CMBDocument document, int partNumber, StringBuffer outMimeType)
        {
                return null;
        }

        /** Retrieve a document annotation. */
        public InputStream getAnnotation(CMBDocument document, int annotationNumber)
        {
                return null;
        }

        /** Retrieve a document's resources. */
        public InputStream getResources(CMBDocument document)
        {
            InputStream resourceStream = null;
            if (resourceStream == null)
                resourceStream = new ByteArrayInputStream(new byte[0]);
            return resourceStream;
        }

        /** Returns true if tracing is enabled */
        public boolean traceEnabled() {
            return false;
        }

        /** Writes a trace message. */
        public void trace(String message) {
            System.out.println("StreamingDocServicesCallbacks.trace -- " + message);
            return;
        }

    }

    /** Returns the instance of CMBAnnotationServicesCallbacks.  Creates one if needed. */
    private CMBAnnotationServices getAnnotationServices()
    {
        if (currAnnoServices == null)
            currAnnoServices = new CMBAnnotationServices(new AnnotationServicesCallbacks());
        return currAnnoServices;
    }

    /** This class handles callbacks from CMBAnnotationDocServices. */
    private class AnnotationServicesCallbacks extends CMBAnnotationServicesCallbacks implements Serializable
    {
        public InputStream getAnnotationPart(CMBAnnotationSet as, int i)
        {
            System.out.println("----------AnnotationServicesCallbacks.getAnnotationPart()");
            return null;
        }

        public boolean getPrivilege(CMBAnnotationSet as, int i)
        {
            System.out.println("----------AnnotationServicesCallbacks.getPrivilege()");
            if(i == CMBPrivilege.CMB_PRIV_CREATE_ANNOTATION)
            	return true;
            if(i== CMBPrivilege.CMB_PRIV_DELETE_ANNOTATION)
            	return true;
            if(i== CMBPrivilege.CMB_PRIV_VIEW_ANNOTATION)
            	return true;
            if(i== CMBPrivilege.CMB_PRIV_PRINT)
            	return true;
            return false;
        }

        public int addAnnotationPart(CMBAnnotationSet as, byte[] data)
        {
            System.out.println("----------AnnotationServicesCallbacks.saveAnnotationPart()");
            return 1;
        }

        public void updateAnnotationPart(CMBAnnotationSet as, byte[] data, int i)
        {
            // Get the annotation file and create annotation stream
	        URL annFileURL = null;
	        URLConnection annFileURLConn = null;
	        OutputStream annos = null;


            System.out.println("----------AnnotationServicesCallbacks.updateAnnotationPart()");
	        try {
	            annFileURL = new URL(getCodeBase(), "test.eipann");
	        }catch (MalformedURLException e) {
	            System.out.println("JApplet1.start() - Creating annFileURL: " + e);
	        }
	        try {
	            annFileURLConn = annFileURL.openConnection();
	            annFileURLConn.setDoOutput(true);
    	        annos = annFileURLConn.getOutputStream();
    	        System.out.println("AnnotationServicesCallbacks.updateAnnotationPart() - got annostream - " + annos.toString());
		        if (annos == null)
		        {
		            System.out.println("AnnotationServicesCallbacks.updateAnnotationPart() - Could not get anno outputstream");
		                annos = new ByteArrayOutputStream(data.length);
		        }
                annos.write(data);
                System.out.println("AnnotationServicesCallbacks.updateAnnotationPart() - Wrote data to annos" + data);
                annos.flush();
                annos.close();
                System.out.println("AnnotationServicesCallbacks.updateAnnotationPart() - Closed annos");
	        } catch (IOException e) {
	            System.out.println("JApplet1.start() - IOException opening annFileURLConn: " + e);
	        }
        }

        public void removeAnnotationPart(CMBAnnotationSet as, int i)
        {
            System.out.println("----------AnnotationServicesCallbacks.removeAnnotationPart()");
        }

        public boolean traceEnabled()
        {
            System.out.println("----------AnnotationServicesCallbacks.traceEnabled()");
            return true;
        }

        public void trace(String s)
        {
            System.out.println(s);
        }
    }
}
